@@ -1,4 +1,7 @@
-/* keyring.c
+/* $Id: keyring.c,v 1.42 2005/10/24 19:15:41 judd Exp $ */
+
+/*******************************************************************************
+ * keyring.c
  *
  * This is a plugin for J-Pilot for the KeyRing Palm program.
  * It keeps records and uses DES3 encryption.
@@ -17,7 +20,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,6 +43,8 @@
 #include <pi-file.h>
 
 #include "../i18n.h"
+#include "prefs.h"
+#include "stock_buttons.h"
 
 #define KEYRING_CAT1 1
 #define KEYRING_CAT2 2
@@ -52,11 +58,12 @@
 #define CONNECT_SIGNALS 400
 #define DISCONNECT_SIGNALS 401
 
+#define NUM_KEYRING_CAT_ITEMS 16
 
 /* Global vars */
 /* This is the category that is currently being displayed */
-static int show_category;
-static int glob_category_number_from_menu_item[16];
+static int show_category = CATEGORY_ALL;
+static int glob_category_number_from_menu_item[NUM_KEYRING_CAT_ITEMS];
 
 static GtkWidget *clist;
 static GtkWidget *entry_name;
@@ -68,13 +75,19 @@ static GObject   *text_note_buffer;
 #endif
 static GtkWidget *menu_category1;
 static GtkWidget *menu_category2;
-static GtkWidget *menu_item_category2[16];
+static int old_category;
+static GtkWidget *menu_item_category1[NUM_KEYRING_CAT_ITEMS+1];
+static GtkWidget *menu_item_category2[NUM_KEYRING_CAT_ITEMS];
 static GtkWidget *scrolled_window;
 static GtkWidget *new_record_button;
 static GtkWidget *apply_record_button;
 static GtkWidget *add_record_button;
 static GtkWidget *delete_record_button;
 static GtkWidget *copy_record_button;
+static GtkWidget *pane = NULL;
+#ifndef ENABLE_STOCK_BUTTONS
+static GtkAccelGroup *accel_group;
+#endif
 
 static int record_changed;
 static int clist_hack;
@@ -166,8 +179,8 @@ static int pack_KeyRing(struct KeyRing *kr, unsigned char *buf, int buf_size,
    jp_logf(JP_LOG_DEBUG, "pack n=%d\n", n);
 
    if (n+2>buf_size) {
-      jp_logf(JP_LOG_WARN, "KeyRing: pack_KeyRing(): buf_size too small\n");
-      return 0;
+      jp_logf(JP_LOG_WARN, _("KeyRing: pack_KeyRing(): buf_size too small\n"));
+      return EXIT_FAILURE;
    }
 
    memset(buf, 0, n+1);
@@ -182,7 +195,7 @@ static int pack_KeyRing(struct KeyRing *kr, unsigned char *buf, int buf_size,
    for (i=strlen(kr->name)+1; i<n; i=i+8) {
       /* des_encrypt3((DES_LONG *)&buf[i], s1, s2, s1); */
 #ifdef HEADER_NEW_DES_H
-      DES_ecb3_encrypt((const_des_cblock *)&buf[i], (des_cblock *)(&buf[i]), 
+      DES_ecb3_encrypt(&buf[i], &buf[i], 
 		       &s1, &s2, &s1, DES_ENCRYPT);
 #else
       des_ecb3_encrypt((const_des_cblock *)&buf[i], (des_cblock *)(&buf[i]),
@@ -214,7 +227,7 @@ static int set_password_hash(unsigned char *buf, int buf_size, char *passwd)
    /* Must wipe passwd out of memory after using it */
    
    if (buf_size < MD5_HASH_SIZE) {
-      return -1;
+      return EXIT_FAILURE;
    }
    memset(buffer, 0, MESSAGE_BUF_SIZE);
    memcpy(buffer, buf, SALT_SIZE);
@@ -225,7 +238,7 @@ static int set_password_hash(unsigned char *buf, int buf_size, char *passwd)
    memset(buffer, 0, MESSAGE_BUF_SIZE);
 
    if (memcmp(md, buf+SALT_SIZE, MD5_HASH_SIZE)) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    MD5(passwd, strlen(passwd), md);
@@ -239,7 +252,7 @@ static int set_password_hash(unsigned char *buf, int buf_size, char *passwd)
    des_set_key(&current_key2, s2);
 #endif
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 static int unpack_KeyRing(struct KeyRing *kr, unsigned char *buf, int buf_size)
@@ -281,7 +294,7 @@ static int unpack_KeyRing(struct KeyRing *kr, unsigned char *buf, int buf_size)
       /* des_decrypt3((DES_LONG *)chunk, s1, s2, s1); */
       /* memcpy(clear_text+i, chunk, 8); */
 #ifdef HEADER_NEW_DES_H
-      DES_ecb3_encrypt((const_des_cblock *)&P[i], (DES_cblock *)(clear_text+i),
+      DES_ecb3_encrypt(&P[i], clear_text+i,
 		       &s1, &s2, &s1, DES_DECRYPT);
 #else
       des_ecb3_encrypt((const_des_cblock *)&P[i], (des_cblock *)(clear_text+i),
@@ -476,7 +489,7 @@ int plugin_get_name(char *name, int len)
 {
    jp_logf(JP_LOG_DEBUG, "KeyRing: plugin_get_name\n");
    strncpy(name, "KeyRing 0.01", len);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -485,8 +498,8 @@ int plugin_get_name(char *name, int len)
  */
 int plugin_get_menu_name(char *name, int len)
 {
-   strncpy(name, "KeyRing", len);
-   return 0;
+   strncpy(name, _("KeyRing"), len);
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -496,8 +509,8 @@ int plugin_get_menu_name(char *name, int len)
  */
 int plugin_get_help_name(char *name, int len)
 {
-   strncpy(name, "About KeyRing", len);
-   return 0;
+   g_snprintf(name, len, _("About %s"), _("KeyRing"));
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -507,7 +520,7 @@ int plugin_get_help_name(char *name, int len)
 int plugin_get_db_name(char *name, int len)
 {
    strncpy(name, "Keys-Gtkr", len);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /* This function gets called when the "delete" button is pressed */
@@ -619,10 +632,16 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
    kr.note = (char *)gtk_editable_get_chars(GTK_EDITABLE(text_note), 0, -1);
 #endif
 
-   jp_charset_j2p((unsigned char *)kr.name, strlen(kr.name)+1);
-   jp_charset_j2p((unsigned char *)kr.account, strlen(kr.account)+1);
-   jp_charset_j2p((unsigned char *)kr.password, strlen(kr.account)+1);
-   jp_charset_j2p((unsigned char *)kr.note, strlen(kr.note)+1);
+   kr.name = strdup(kr.name);
+   jp_charset_j2p(kr.name, strlen(kr.name)+1);
+
+   kr.account = strdup(kr.account);
+   jp_charset_j2p(kr.account, strlen(kr.account)+1);
+   
+   kr.password = strdup(kr.password);
+   jp_charset_j2p(kr.password, strlen(kr.password)+1);
+   
+   jp_charset_j2p(kr.note, strlen(kr.note)+1);
 
    pack_KeyRing(&kr, buf, 0xFFFF, &new_size);
 
@@ -755,8 +774,11 @@ static int display_record(struct MyKeyRing *mkr, int at_row)
       sprintf(temp, "#%03d", at_row);
       gtk_clist_set_text(GTK_CLIST(clist), at_row, 0, temp);
    } else {
-      temp_str = strdup(mkr->kr.name);
-      jp_charset_p2j((unsigned char *)temp_str, strlen(mkr->kr.name)+1);
+	  int len;
+
+	  temp_str = malloc((len = strlen(mkr->kr.name)*2+1));
+	  multibyte_safe_strncpy(temp_str, mkr->kr.name, len);
+	  jp_charset_p2j(temp_str, len);
       gtk_clist_set_text(GTK_CLIST(clist), at_row, 0, temp_str);
       free(temp_str);
    }
@@ -764,13 +786,16 @@ static int display_record(struct MyKeyRing *mkr, int at_row)
    if ( (!(mkr->kr.account)) || (mkr->kr.account[0]=='\0') ) {
       gtk_clist_set_text(GTK_CLIST(clist), at_row, 1, "");
    } else {
-      temp_str = strdup(mkr->kr.account);
-      jp_charset_p2j((unsigned char *)temp_str, strlen(mkr->kr.account)+1);
+	  int len;
+
+	  temp_str = malloc((len = strlen(mkr->kr.account)*2+1));
+	  multibyte_safe_strncpy(temp_str, mkr->kr.account, len);
+	  jp_charset_p2j(temp_str, len);
       gtk_clist_set_text(GTK_CLIST(clist), at_row, 1, temp_str);
       free(temp_str);
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -791,7 +816,7 @@ static void display_records()
    
    jp_logf(JP_LOG_DEBUG, "KeyRing: display_records\n");
 
-   row_count=((GtkCList *)clist)->rows;
+   row_count=GTK_CLIST(clist)->rows;
 
    /* Need to remove pointers to data we are about to delete */
    for (i=0; i<row_count; i++) {
@@ -815,6 +840,9 @@ static void display_records()
 
    /* This function takes care of reading the Database for us */
    num = jp_read_DB_files("Keys-Gtkr", &records);
+   if (-1 == num)
+     return;
+
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
       records = temp_list;
@@ -833,7 +861,9 @@ static void display_records()
 
       /* Since deleted and modified records are also returned and we don't
        * want to see those we skip over them. */
-      if ((br->rt == DELETED_PALM_REC) || (br->rt == MODIFIED_PALM_REC)) {
+      if ((br->rt == DELETED_PALM_REC) || 
+	  (br->rt == DELETED_PC_REC)   ||
+	  (br->rt == MODIFIED_PALM_REC) ) {
 	 continue;
       }
       if (show_category < 16) {
@@ -911,7 +941,6 @@ static void cb_clist_selection(GtkWidget      *clist,
    jp_logf(JP_LOG_DEBUG, "KeyRing: cb_clist_selection\n");
 
    if ((!event) && (clist_hack)) return;
-   if (event==0) return;
 
    if (row<0) {
       return;
@@ -957,8 +986,11 @@ static void cb_clist_selection(GtkWidget      *clist,
    gtk_option_menu_set_history(GTK_OPTION_MENU(menu_category2), item_num);
 
    if (mkr->kr.name) {
-      temp_str = strdup(mkr->kr.name);
-      jp_charset_p2j((unsigned char *)temp_str, strlen(mkr->kr.name)+1); 
+	  int len;
+
+	  temp_str = malloc((len = strlen(mkr->kr.name)*2+1));
+	  multibyte_safe_strncpy(temp_str, mkr->kr.name, len);
+      jp_charset_p2j(temp_str, len);
       gtk_entry_set_text(GTK_ENTRY(entry_name), temp_str);
       free(temp_str);
    } else {
@@ -966,8 +998,11 @@ static void cb_clist_selection(GtkWidget      *clist,
    }
 
    if (mkr->kr.account) {
-      temp_str = strdup(mkr->kr.account);
-      jp_charset_p2j((unsigned char *)temp_str, strlen(mkr->kr.account)+1); 
+	  int len;
+
+	  temp_str = malloc((len = strlen(mkr->kr.account)*2+1));
+	  multibyte_safe_strncpy(temp_str, mkr->kr.account, len);
+      jp_charset_p2j(temp_str, len);
       gtk_entry_set_text(GTK_ENTRY(entry_account), temp_str); 
       free(temp_str);
    } else {
@@ -975,8 +1010,11 @@ static void cb_clist_selection(GtkWidget      *clist,
    }
 
    if (mkr->kr.password) {
-      temp_str = strdup(mkr->kr.password);
-      jp_charset_p2j((unsigned char *)temp_str, strlen(mkr->kr.password)+1);
+	  int len;
+
+	  temp_str = malloc((len = strlen(mkr->kr.password)*2+1));
+	  multibyte_safe_strncpy(temp_str, mkr->kr.password, len);
+      jp_charset_p2j(temp_str, len);
       gtk_entry_set_text(GTK_ENTRY(entry_password), temp_str); 
       free(temp_str);
    } else {
@@ -991,8 +1029,11 @@ static void cb_clist_selection(GtkWidget      *clist,
 			   gtk_text_get_length(GTK_TEXT(text_note)));
 #endif
    if (mkr->kr.note) {
-      temp_str = strdup(mkr->kr.note);
-      jp_charset_p2j((unsigned char *)temp_str, strlen(mkr->kr.note)+1);
+	  int len;
+
+	  temp_str = malloc((len = strlen(mkr->kr.note)*2+1));
+	  multibyte_safe_strncpy(temp_str, mkr->kr.note, len);
+      jp_charset_p2j(temp_str, len);
 #ifdef ENABLE_GTK2
       gtk_text_buffer_set_text(GTK_TEXT_BUFFER(text_note_buffer), temp_str, -1);
 #else
@@ -1028,6 +1069,17 @@ static void cb_category(GtkWidget *item, unsigned int value)
 
    switch (menu) {
     case KEYRING_CAT1:
+      // remember the previously used category
+      for (old_category=0; old_category<NUM_KEYRING_CAT_ITEMS; old_category++) {
+#ifdef ENABLE_GTK2
+	 if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item_category1[old_category])))
+	   break;
+#else
+	 if (GTK_CHECK_MENU_ITEM(menu_item_category1[old_category])->active)
+	   break;
+#endif
+      }
+
       b=dialog_save_changed_record(clist, record_changed);
       if (b==DIALOG_SAID_1) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
@@ -1088,7 +1140,7 @@ static int make_menu(char *items[], int menu_index, GtkWidget **Poption_menu,
 
    gtk_widget_show(option_menu);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /* 
@@ -1102,7 +1154,6 @@ static void make_menus()
    int buf_size;
    int i, count;
    char all[]="All";
-   GtkWidget *menu_item_category1[17];
    char *categories[18];
      
    jp_logf(JP_LOG_DEBUG, "KeyRing: make_menus\n");
@@ -1127,7 +1178,7 @@ static void make_menus()
       if (ai.name[i][0]=='\0') {
 	 continue;
       }
-      jp_charset_p2j((unsigned char *)ai.name[i], 16);
+      jp_charset_p2j(ai.name[i], 16);
       categories[count+1]=ai.name[i];
       glob_category_number_from_menu_item[count++]=i;
    }
@@ -1209,7 +1260,7 @@ static int dialog_password(GtkWindow *main_window, char *ascii_password, int rea
    int ret;
 
    if (!ascii_password) {
-      return -1;
+      return EXIT_FAILURE;
    }
    ascii_password[0]='\0';
    ret = 2; 
@@ -1267,13 +1318,21 @@ static int dialog_password(GtkWindow *main_window, char *ascii_password, int rea
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 2);
 
    /* Buttons */
+#ifdef ENABLE_STOCK_BUTTONS
+   button = gtk_button_new_from_stock(GTK_STOCK_OK);
+#else
    button = gtk_button_new_with_label(_("OK"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_dialog_button),
 		      GINT_TO_POINTER(DIALOG_SAID_1));
    gtk_box_pack_start(GTK_BOX(hbox1), button, TRUE, TRUE, 1);
 
+#ifdef ENABLE_STOCK_BUTTONS
+   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
    button = gtk_button_new_with_label(_("Cancel"));
+#endif
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_dialog_button),
 		      GINT_TO_POINTER(DIALOG_SAID_2));
@@ -1325,22 +1384,22 @@ static int check_for_db()
    if (!home) { /* Not home; */
       home = getenv("HOME");
       if (!home) {
-	 jp_logf(JP_LOG_WARN, "Can't get HOME environment variable\n");
-	 return -1;
+	 jp_logf(JP_LOG_WARN, _("Can't get HOME environment variable\n"));
+	 return EXIT_FAILURE;
       }
    }
    if (strlen(home)>(max_size-strlen(file)-strlen("/.jpilot/")-2)) {
-      jp_logf(JP_LOG_WARN, "Your HOME environment variable is too long for me\n");
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Your HOME environment variable is too long for me\n"));
+      return EXIT_FAILURE;
    }
    sprintf(full_name, "%s/.jpilot/%s", home, file);
    if (stat(full_name, &buf)) {
-      jp_logf(JP_LOG_FATAL, "KeyRing: file %s not found.\n", full_name);
-      jp_logf(JP_LOG_FATAL, "KeyRing: Try Syncing.\n", full_name);
-      return -1;
+      jp_logf(JP_LOG_FATAL, _("KeyRing: file %s not found.\n"), full_name);
+      jp_logf(JP_LOG_FATAL, _("KeyRing: Try Syncing.\n"), full_name);
+      return EXIT_FAILURE;
    }
 		 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1357,7 +1416,7 @@ int verify_pasword(char *ascii_password)
    jp_logf(JP_LOG_DEBUG, "KeyRing: verify_pasword\n");
 
    if (check_for_db()) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    password_not_correct = 1;
@@ -1365,6 +1424,8 @@ int verify_pasword(char *ascii_password)
    /* This function takes care of reading the Database for us */
    records=NULL;
    num = jp_read_DB_files("Keys-Gtkr", &records);
+   if (-1 == num)
+     return 0;
 
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
@@ -1392,7 +1453,7 @@ int verify_pasword(char *ascii_password)
    }
    jp_free_DB_records(&records);
    if (password_not_correct) return 1;
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1404,7 +1465,7 @@ int verify_pasword(char *ascii_password)
 int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
 {
    GtkWidget *vbox1, *vbox2;
-   GtkWidget *temp_hbox;
+   GtkWidget *hbox_temp;
    GtkWidget *button;
    GtkWidget *label;
 #ifndef ENABLE_GTK2
@@ -1412,6 +1473,8 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
 #endif
    GtkWidget *table;
    GtkWindow *w;
+   GtkWidget *separator;
+   long ivalue;
    time_t ltime;
    struct tm *now;
    char ascii_password[PASSWD_LEN];
@@ -1419,17 +1482,14 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
    int password_not_correct;
    char *titles[2];
    int retry;
+   int cycle_category = FALSE;
 
    titles[0] = _("Name"); titles[1] = _("Account");
    
    jp_logf(JP_LOG_DEBUG, "KeyRing: plugin gui started, unique_id=%d\n", unique_id);
 
-   if (unique_id) {
-      show_category = CATEGORY_ALL;
-   }
-
    if (check_for_db()) {
-      return -1;
+      return EXIT_FAILURE;
    }
    
    /* Find the main window from some widget */
@@ -1464,13 +1524,20 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
       }
       memset(ascii_password, 0, PASSWD_LEN-1);
    }
+   else
+      cycle_category = TRUE;
+
+   /* called to display the result of a search */
+   if (unique_id) {
+      show_category = CATEGORY_ALL;
+      cycle_category = FALSE;
+   }
 
    /* plug entered with correct password */
    plugin_last_time = time(NULL);
    plugin_active = TRUE;
 
    record_changed=CLEAR_FLAG;
-   show_category = CATEGORY_ALL;
    clist_row_selected = 0;
 
    time(&ltime);
@@ -1480,22 +1547,38 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
    jp_logf(JP_LOG_DEBUG, "KeyRing: calling make_menus\n");
    make_menus();
    
+   pane = gtk_hpaned_new();
+   get_pref(PREF_KEYRING_PANE, &ivalue, NULL);
+   gtk_paned_set_position(GTK_PANED(pane), ivalue + PANE_CREEP);
+
+   gtk_box_pack_start(GTK_BOX(hbox), pane, TRUE, TRUE, 5);
+
    /* left and right main boxes */
    vbox1 = gtk_vbox_new(FALSE, 0);
    vbox2 = gtk_vbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(hbox), vbox1, TRUE, TRUE, 5);
-   gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 5);
+   gtk_paned_pack1(GTK_PANED(pane), vbox1, TRUE, FALSE);
+   gtk_paned_pack2(GTK_PANED(pane), vbox2, TRUE, FALSE);
 
    gtk_widget_set_usize(GTK_WIDGET(vbox1), 0, 230);
    gtk_widget_set_usize(GTK_WIDGET(vbox2), 0, 230);
 
+   /* Make accelerators for some buttons window */
+#ifndef ENABLE_STOCK_BUTTONS
+   accel_group = gtk_accel_group_new();
+#ifdef ENABLE_GTK2
+   gtk_window_add_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(vbox)), accel_group);
+#else
+   gtk_accel_group_attach(accel_group, GTK_OBJECT(gtk_widget_get_toplevel(vbox)));
+#endif
+#endif
+
    /* Make a temporary hbox */
-   temp_hbox = gtk_hbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(vbox1), temp_hbox, FALSE, FALSE, 0);
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox1), hbox_temp, FALSE, FALSE, 0);
    
    label = gtk_label_new(_("Category: "));
-   gtk_box_pack_start(GTK_BOX(temp_hbox), label, FALSE, FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(temp_hbox), menu_category1, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), menu_category1, TRUE, TRUE, 0);
 
    
    /* Scrolled Window */
@@ -1524,44 +1607,47 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
    /* Right half of screen */
    /* -------------------- */
    
-   temp_hbox = gtk_hbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(vbox2), temp_hbox, FALSE, FALSE, 0);
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
    /* Add record button */
-   delete_record_button = gtk_button_new_with_label(_("Delete"));
+   CREATE_BUTTON(delete_record_button, _("Delete"), DELETE, _("Delete the selected record"), GDK_d, GDK_CONTROL_MASK, "Ctrl+D");
    gtk_signal_connect(GTK_OBJECT(delete_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_delete),
 		      GINT_TO_POINTER(DELETE_FLAG));
-   gtk_box_pack_start(GTK_BOX(temp_hbox), delete_record_button, TRUE, TRUE, 0);
 
-   copy_record_button = gtk_button_new_with_label(_("Copy"));
-   gtk_box_pack_start(GTK_BOX(temp_hbox), copy_record_button, TRUE, TRUE, 0);
+   CREATE_BUTTON(copy_record_button, _("Copy"), COPY, _("Copy the selected record"), GDK_o, GDK_CONTROL_MASK, "Ctrl+O");
    gtk_signal_connect(GTK_OBJECT(copy_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(COPY_FLAG));
 
-   new_record_button = gtk_button_new_with_label(_("New Record"));
-   gtk_box_pack_start(GTK_BOX(temp_hbox), new_record_button, TRUE, TRUE, 0);
+   CREATE_BUTTON(new_record_button, _("New Record"), NEW, _("Add a new record"), GDK_n, GDK_CONTROL_MASK, "Ctrl+N")
    gtk_signal_connect(GTK_OBJECT(new_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(CLEAR_FLAG));
 
-   add_record_button = gtk_button_new_with_label(_("Add Record"));
-   gtk_box_pack_start(GTK_BOX(temp_hbox), add_record_button, TRUE, TRUE, 0);
+   CREATE_BUTTON(add_record_button, _("Add Record"), ADD, _("Add the new record"), GDK_Return, GDK_CONTROL_MASK, "Ctrl+Enter")
    gtk_signal_connect(GTK_OBJECT(add_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(NEW_FLAG));
+#ifndef ENABLE_STOCK_BUTTONS
    gtk_widget_set_name(GTK_WIDGET(GTK_LABEL(GTK_BIN(add_record_button)->child)),
 		       "label_high");
+#endif
 
-   apply_record_button = gtk_button_new_with_label(_("Apply Changes"));
-   gtk_box_pack_start(GTK_BOX(temp_hbox), apply_record_button, TRUE, TRUE, 0);
+   CREATE_BUTTON(apply_record_button, _("Apply Changes"), APPLY, _("Commit the modifications"), GDK_Return, GDK_CONTROL_MASK, "Ctrl+Enter")
    gtk_signal_connect(GTK_OBJECT(apply_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(MODIFY_FLAG));
+#ifndef ENABLE_STOCK_BUTTONS
    gtk_widget_set_name(GTK_WIDGET(GTK_LABEL(GTK_BIN(apply_record_button)->child)),
 		       "label_high");
+#endif
    
+   /*Separator */
+   separator = gtk_hseparator_new();
+   gtk_box_pack_start(GTK_BOX(vbox2), separator, FALSE, FALSE, 5);
+
    /* Table */
    table = gtk_table_new(4, 10, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table),0);
@@ -1605,8 +1691,8 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
    label = gtk_label_new(_("Note"));
    gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
-   temp_hbox = gtk_hbox_new(FALSE, 0);
-   gtk_box_pack_start(GTK_BOX(vbox2), temp_hbox, TRUE, TRUE, 0);
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, TRUE, TRUE, 0);
 
 #ifdef ENABLE_GTK2
    text_note = gtk_text_view_new();
@@ -1619,14 +1705,14 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 1);
    gtk_container_add(GTK_CONTAINER(scrolled_window), text_note);
-   gtk_box_pack_start_defaults(GTK_BOX(temp_hbox), scrolled_window);
+   gtk_box_pack_start_defaults(GTK_BOX(hbox_temp), scrolled_window);
 #else
    text_note = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(text_note), TRUE);
    gtk_text_set_word_wrap(GTK_TEXT(text_note), TRUE);
    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(text_note)->vadj);
-   gtk_box_pack_start(GTK_BOX(temp_hbox), text_note, TRUE, TRUE, 0);
-   gtk_box_pack_start(GTK_BOX(temp_hbox), vscrollbar, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), text_note, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), vscrollbar, FALSE, FALSE, 0);
 #endif
 
    gtk_widget_show_all(hbox);
@@ -1635,6 +1721,18 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
    gtk_widget_hide(add_record_button);
    gtk_widget_hide(apply_record_button);
 
+   if (cycle_category) {
+      old_category++;
+
+      if (NULL == menu_item_category1[old_category]) {
+	 old_category = 0;
+	 show_category = CATEGORY_ALL;
+      }
+
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item_category1[old_category]), TRUE);
+      gtk_option_menu_set_history(GTK_OPTION_MENU(menu_category1), old_category);
+   }
+
    jp_logf(JP_LOG_DEBUG, "KeyRing: calling display_records\n");
 
    display_records();
@@ -1645,7 +1743,7 @@ int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id)
       keyring_find(unique_id);
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1675,7 +1773,28 @@ int plugin_gui_cleanup() {
    }
    plugin_active = FALSE;
 
-   return 0;
+   /* Remove the accelerators */
+#ifndef ENABLE_STOCK_BUTTONS
+#ifdef ENABLE_GTK2
+   gtk_window_remove_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(pane)), accel_group);
+#else
+   gtk_accel_group_detach(accel_group, GTK_OBJECT(gtk_widget_get_toplevel(pane)));
+#endif
+#endif
+
+   /* the pane may not exist.
+	* if the wrong password is entered the gui is not built */
+   if (pane)
+   {
+#ifdef ENABLE_GTK2
+	   set_pref(PREF_KEYRING_PANE, gtk_paned_get_position(GTK_PANED(pane)), NULL, TRUE);
+#else
+	   set_pref(PREF_KEYRING_PANE, GTK_PANED(pane)->handle_xpos, NULL, TRUE);
+#endif
+	   pane = NULL;
+   }
+
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1692,7 +1811,7 @@ int plugin_startup(jp_startup_info *info)
 	 jp_logf(JP_LOG_DEBUG, "KeyRing: base_dir = [%s]\n", info->base_dir);
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1702,7 +1821,7 @@ int plugin_startup(jp_startup_info *info)
 int plugin_pre_sync(void)
 {
    jp_logf(JP_LOG_DEBUG, "KeyRing: plugin_pre_sync\n");
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1714,7 +1833,7 @@ int plugin_pre_sync(void)
 int plugin_sync(int sd)
 {
    jp_logf(JP_LOG_DEBUG, "KeyRing: plugin_sync\n");
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int plugin_help(char **text, int *width, int *height)
@@ -1735,7 +1854,7 @@ int plugin_help(char **text, int *width, int *height)
    *height = 0;
    *width = 0;
    
-   return 0;
+   return EXIT_SUCCESS;
 }
 	 
 /*
@@ -1744,7 +1863,7 @@ int plugin_help(char **text, int *width, int *height)
 int plugin_post_sync(void)
 {
    jp_logf(JP_LOG_DEBUG, "KeyRing: plugin_post_sync\n");
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1753,7 +1872,7 @@ int plugin_post_sync(void)
 int plugin_exit_cleanup(void)
 {
    jp_logf(JP_LOG_DEBUG, "KeyRing: plugin_exit_cleanup\n");
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 static int add_search_result(const char *line, int unique_id, struct search_result **sr)
@@ -1763,7 +1882,7 @@ static int add_search_result(const char *line, int unique_id, struct search_resu
    jp_logf(JP_LOG_DEBUG, "KeyRing: add_search_result for [%s]\n", line);
    temp_sr=malloc(sizeof(struct search_result));
    if (!temp_sr) {
-      return -1;
+      return EXIT_FAILURE;
    }
    temp_sr->next=NULL;
    temp_sr->unique_id=unique_id;
@@ -1774,7 +1893,7 @@ static int add_search_result(const char *line, int unique_id, struct search_resu
       (*sr)->next=temp_sr;
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1803,6 +1922,8 @@ int plugin_search(const char *search_string, int case_sense,
 
    /* This function takes care of reading the Database for us */
    num = jp_read_DB_files("Keys-Gtkr", &records);
+   if (-1 == num)
+     return 0;
 
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
@@ -1873,19 +1994,18 @@ int plugin_search(const char *search_string, int case_sense,
 
 static int keyring_find(int unique_id)
 {
-   int r, found_at, total_count;
+   int r, found_at;
    
    jp_logf(JP_LOG_DEBUG, "KeyRing: keyring_find\n");
 
    r = clist_find_id(clist,
 		     unique_id,
-		     &found_at,
-		     &total_count);
+		     &found_at);
    if (r) {
       cb_clist_selection(clist, found_at, 0, (gpointer)455, NULL);
       gtk_clist_select_row(GTK_CLIST(clist), found_at, 0);
       gtk_clist_moveto(GTK_CLIST(clist), found_at, 0, 0.5, 0.0);
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
