@@ -1,4 +1,7 @@
-/* address.c
+/* $Id: address.c,v 1.38 2005/09/26 03:27:59 judd Exp $ */
+
+/*******************************************************************************
+ * address.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #include "config.h"
 #include "i18n.h"
 #include <stdlib.h>
@@ -48,7 +52,7 @@ int print_address_list(AddressList **al)
 
    for (prev_al=NULL, temp_al=*al; temp_al;
 	prev_al=temp_al, temp_al=temp_al->next) {
-      jp_logf(JP_LOG_FILE | JP_LOG_STDOUT, "entry[0]=[%s]\n", temp_al->ma.a.entry[0]);
+      jp_logf(JP_LOG_FILE | JP_LOG_STDOUT, "entry[0]=[%s]\n", temp_al->maddr.addr.entry[0]);
    }
 }
 #endif
@@ -64,8 +68,8 @@ int address_compare(const void *v1, const void *v2)
    al1=(AddressList **)v1;
    al2=(AddressList **)v2;
 
-   a1=&((*al1)->ma.a);
-   a2=&((*al2)->ma.a);
+   a1=&((*al1)->maddr.addr);
+   a2=&((*al2)->maddr.addr);
 
    if (glob_sort_rule & SORT_BY_COMPANY) {
       sort1=2; /*company */
@@ -95,7 +99,7 @@ int address_compare(const void *v1, const void *v2)
 	 if (a1->entry[sort1] && a1->entry[sort2]) {
 	    if ((str1 = (char *)malloc(strlen(a1->entry[sort1])+strlen(a1->entry[sort2])+1)) == NULL) {
 	       return 0;
-	    }	      
+	    }
 	    strcpy(str1, a1->entry[sort1]);
 	    strcat(str1, a1->entry[sort2]);
 	 }
@@ -137,7 +141,7 @@ int address_compare(const void *v1, const void *v2)
 	 if ((!a2->entry[sort1]) && a2->entry[sort2]) {
 	    if ((str2 = (char *)malloc(strlen(a2->entry[sort2])+1)) == NULL) {
 	       return 0;
-	    }	      
+	    }
 	    strcpy(str2, a2->entry[sort2]);
 	 }
       } else if (a2->entry[sort3]) {
@@ -239,7 +243,7 @@ int address_compare(const void *v1, const void *v2)
    return i;
 }
 
-/* 
+/*
  * sort_order: 0=descending,  1=ascending
  */
 int address_sort(AddressList **al, int sort_order)
@@ -257,7 +261,7 @@ int address_sort(AddressList **al, int sort_order)
 
    if (count<2) {
       /* We don't have to sort less than 2 items */
-      return 0;
+      return EXIT_SUCCESS;
    }
 
    get_address_app_info(&ai);
@@ -266,7 +270,7 @@ int address_sort(AddressList **al, int sort_order)
       glob_sort_rule = !(ai.sortByCompany & SORT_BY_COMPANY);
    }
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set == CHAR_SET_JAPANESE) {
+   if (char_set == CHAR_SET_JAPANESE || char_set == CHAR_SET_SJIS_UTF) {
       glob_sort_rule = glob_sort_rule | SORT_JAPANESE;
    } else {
       glob_sort_rule = glob_sort_rule & (SORT_JAPANESE-1);
@@ -281,8 +285,8 @@ int address_sort(AddressList **al, int sort_order)
    /* Allocate an array to be qsorted */
    sort_al = calloc(count, sizeof(AddressList *));
    if (!sort_al) {
-      jp_logf(JP_LOG_WARN, "address_sort(): Out of Memory\n");
-      return 0;
+      jp_logf(JP_LOG_WARN, "address_sort(): %s\n", _("Out of memory"));
+      return EXIT_FAILURE;
    }
 
    /* Set our array to be a list of pointers to the nodes in the linked list */
@@ -311,34 +315,53 @@ int address_sort(AddressList **al, int sort_order)
 
    free(sort_al);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
-int pc_address_write(struct Address *a, PCRecType rt, unsigned char attrib,
+int pc_address_write(struct Address *addr, PCRecType rt, unsigned char attrib,
 		     unsigned int *unique_id)
 {
+#ifndef PILOT_LINK_0_12
    char record[65536];
-   int rec_len,i;
+   int rec_len;
+#else /* PILOT_LINK_0_12 */
+   pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
+   int i;
    buf_rec br;
    long char_set;
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set != CHAR_SET_LATIN1) {
       for (i = 0; i < 19; i++) {
-	 if (a->entry[i]) charset_j2p((unsigned char *)a->entry[i], strlen(a->entry[i])+1, char_set);
+	 if (addr->entry[i]) charset_j2p(addr->entry[i], strlen(addr->entry[i])+1, char_set);
       }
    }
 
-   rec_len = pack_Address(a, (unsigned char *)record, 65535);
+#ifndef PILOT_LINK_0_12
+   rec_len = pack_Address(addr, (unsigned char *)record, 65535);
    if (!rec_len) {
       PRINT_FILE_LINE;
       jp_logf(JP_LOG_WARN, "pack_Address %s\n", _("error"));
-      return -1;
+      return EXIT_FAILURE;
    }
+#else /* PILOT_LINK_0_12 */
+   RecordBuffer = pi_buffer_new(0);
+   if (pack_Address(addr, RecordBuffer, address_v1) == -1) {
+      PRINT_FILE_LINE;
+      jp_logf(JP_LOG_WARN, "pack_Address %s\n", _("error"));
+      return EXIT_FAILURE;
+   }
+#endif /* PILOT_LINK_0_12 */
    br.rt=rt;
    br.attrib = attrib;
+#ifndef PILOT_LINK_0_12
    br.buf = record;
    br.size = rec_len;
+#else /* PILOT_LINK_0_12 */
+   br.buf = RecordBuffer->data;
+   br.size = RecordBuffer->used;
+#endif /* PILOT_LINK_0_12 */
    /* Keep unique ID intact */
    if (unique_id) {
       br.unique_id = *unique_id;
@@ -351,7 +374,10 @@ int pc_address_write(struct Address *a, PCRecType rt, unsigned char attrib,
       *unique_id = br.unique_id;
    }
 
-   return 0;
+#ifdef PILOT_LINK_0_12
+   pi_buffer_free(RecordBuffer);
+#endif
+   return EXIT_SUCCESS;
 }
 
 void free_AddressList(AddressList **al)
@@ -359,7 +385,7 @@ void free_AddressList(AddressList **al)
    AddressList *temp_al, *temp_al_next;
 
    for (temp_al = *al; temp_al; temp_al=temp_al_next) {
-      free_Address(&(temp_al->ma.a));
+      free_Address(&(temp_al->maddr.addr));
       temp_al_next = temp_al->next;
       free(temp_al);
    }
@@ -383,36 +409,33 @@ int get_address_app_info(struct AddressAppInfo *ai)
       free(buf);
    }
    if ((num<0) || (rec_size<=0)) {
-      jp_logf(JP_LOG_WARN, _("Error reading %s\n"), "AddressDB.pdb");
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Error reading file: %s\n"), "AddressDB.pdb");
+      return EXIT_FAILURE;
    }
 
-   get_pref(PREF_CHAR_SET, &char_set, NULL); 
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set != CHAR_SET_LATIN1) {
       /* Convert to host character set */
       int i;
-      for (i = 0; i < CATCOUNT; i++)
-	if (ai->category.name[i][0] != '\0') {
-	   charset_p2j((unsigned char *)ai->category.name[i], HOSTCATLTH, char_set);
-	}
+
       for (i = 0; i < 19 + 3; i++)
 	if (ai->labels[i][0] != '\0') {
-	   charset_p2j((unsigned char *)ai->labels[i],16, char_set);
+	   charset_p2j(ai->labels[i],16, char_set);
 	}
       for (i = 0; i < 8; i++)
 	if (ai->phoneLabels[i][0] != '\0') {
-	   charset_p2j((unsigned char *)ai->phoneLabels[i],16, char_set);
+	   charset_p2j(ai->phoneLabels[i],16, char_set);
 	}
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int get_addresses(AddressList **address_list, int sort_order)
 {
    return get_addresses2(address_list, sort_order, 1, 1, 1, CATEGORY_ALL);
 }
-/* 
+/*
  * sort_order: 0=descending,  1=ascending
  * modified, deleted and private, 0 for no, 1 for yes, 2 for use prefs
  */
@@ -422,13 +445,16 @@ int get_addresses2(AddressList **address_list, int sort_order,
    GList *records;
    GList *temp_list;
    int recs_returned, i, num;
-   struct Address a;
+   struct Address addr;
    AddressList *temp_a_list;
    long keep_modified, keep_deleted;
    int keep_priv;
    long char_set;
    buf_rec *br;
    char *buf;
+#ifdef PILOT_LINK_0_12
+   pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
 
    jp_logf(JP_LOG_DEBUG, "get_addresses2()\n");
    if (modified==2) {
@@ -451,6 +477,9 @@ int get_addresses2(AddressList **address_list, int sort_order,
    recs_returned = 0;
 
    num = jp_read_DB_files("AddressDB", &records);
+   if (-1 == num)
+     return 0;
+
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
       records = temp_list;
@@ -465,20 +494,35 @@ int get_addresses2(AddressList **address_list, int sort_order,
 	 continue;
       }
 
-      if ( ((br->rt==DELETED_PALM_REC) && (!keep_deleted)) ||
-	  ((br->rt==MODIFIED_PALM_REC) && (!keep_modified)) ) {
+      if ( ((br->rt==DELETED_PALM_REC)  && (!keep_deleted)) ||
+	   ((br->rt==DELETED_PC_REC)    && (!keep_deleted)) ||
+	   ((br->rt==MODIFIED_PALM_REC) && (!keep_modified)) ) {
 	 continue;
       }
-      if ((keep_priv != SHOW_PRIVATES) && 
+      if ((keep_priv != SHOW_PRIVATES) &&
 	  (br->attrib & dlpRecAttrSecret)) {
 	 continue;
       }
 
-      num = unpack_Address(&a, br->buf, br->size);
+#ifndef PILOT_LINK_0_12
+      num = unpack_Address(&addr, br->buf, br->size);
+#else /* PILOT_LINK_0_12 */
+      RecordBuffer = pi_buffer_new(br->size);
+      memcpy(RecordBuffer->data, br->buf, br->size);
+      RecordBuffer->used = br->size;
+#endif /* PILOT_LINK_0_12 */
 
+#ifndef PILOT_LINK_0_12
       if (num <= 0) {
 	 continue;
       }
+#else /* PILOT_LINK_0_12 */
+      if (unpack_Address(&addr, RecordBuffer, address_v1) == -1) {
+	 pi_buffer_free(RecordBuffer);
+	 continue;
+      }
+      pi_buffer_free(RecordBuffer);
+#endif /* PILOT_LINK_0_12 */
 
       if ( ((br->attrib & 0x0F) != category) && category != CATEGORY_ALL) {
 	 continue;
@@ -487,36 +531,32 @@ int get_addresses2(AddressList **address_list, int sort_order,
       get_pref(PREF_CHAR_SET, &char_set, NULL);
       if (char_set != CHAR_SET_LATIN1) {
 	 for (i = 0; i < 19; i++) {
-	    if ((a.entry[i] != NULL) && (a.entry[i][0] != '\0')) {
-/* JPA use new conversion routines
-	       if ((buf = (char *)realloc(buf, strlen(a.entry[i])*2+1)) != NULL) {
-		  strcpy(buf, a.entry[i]);
-		  charset_p2j((unsigned char *)buf, strlen(a.entry[i])*2+1, char_set);
-*/
-               buf = charset_p2newj((unsigned char *)a.entry[i], strlen(a.entry[i])+1, char_set);
+	    if ((addr.entry[i] != NULL) && (addr.entry[i][0] != '\0')) {
+               buf = charset_p2newj(addr.entry[i], strlen(addr.entry[i])+1, char_set);
                if (buf) {
-		  if (strlen(buf) > strlen(a.entry[i])) {
-		     free(a.entry[i]);
-		     a.entry[i] = strdup(buf);
+		  if (strlen(buf) > strlen(addr.entry[i])) {
+		     free(addr.entry[i]);
+		     addr.entry[i] = strdup(buf);
+		     free(buf);
 		  } else {
-		     multibyte_safe_strncpy(a.entry[i], buf, strlen(a.entry[i])+1);
+		     multibyte_safe_strncpy(addr.entry[i], buf, strlen(addr.entry[i])+1);
+		     free(buf);
 		  }
 	       }
 	    }
 	 }
-	 free(buf);
       }
 
       temp_a_list = malloc(sizeof(AddressList));
       if (!temp_a_list) {
-	 jp_logf(JP_LOG_WARN, "get_addresses2(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "get_addresses2(): %s\n", _("Out of memory"));
 	 break;
       }
-      memcpy(&(temp_a_list->ma.a), &a, sizeof(struct Address));
+      memcpy(&(temp_a_list->maddr.addr), &addr, sizeof(struct Address));
       temp_a_list->app_type = ADDRESS;
-      temp_a_list->ma.rt = br->rt;
-      temp_a_list->ma.attrib = br->attrib;
-      temp_a_list->ma.unique_id = br->unique_id;
+      temp_a_list->maddr.rt = br->rt;
+      temp_a_list->maddr.attrib = br->attrib;
+      temp_a_list->maddr.unique_id = br->unique_id;
       temp_a_list->next = *address_list;
       *address_list = temp_a_list;
       recs_returned++;
