@@ -1,6 +1,9 @@
-/* alarms.c
+/* $Id: alarms.c,v 1.29 2005/10/26 04:27:22 rikster5 Exp $ */
+
+/*******************************************************************************
+ * alarms.c
  * A module of J-Pilot http://jpilot.org
- * 
+ *
  * Copyright (C) 2000-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 /*
  * The PalmOS datebook will alarm on private records even when they are hidden
@@ -39,7 +42,7 @@
 
 /* This is how often to check for alarms in seconds */
 /* Every call takes CPU time(not much), so you may want it to be greater */
-#define ALARM_INTERVAL 1
+#define ALARM_INTERVAL 10
 
 
 #define PREV_ALARM_MASK 1
@@ -172,7 +175,7 @@ int dialog_alarm(char *title, char *frame_text,
 
    /* Prevent alarms from going crazy and using all resources */
    if (total_alarm_windows>20) {
-      return -1;
+      return EXIT_FAILURE;
    }
    total_alarm_windows++;
 #ifdef ALARMS_DEBUG
@@ -250,12 +253,12 @@ int dialog_alarm(char *title, char *frame_text,
       Pdata->unique_id = unique_id;
       Pdata->remind_time = remind_time;
       /* Set the default button pressed to OK */
-      Pdata->button_hit = DIALOG_SAID_1;     
+      Pdata->button_hit = DIALOG_SAID_1;
       Pdata->remind_entry=remind_entry;
       Pdata->radio1=radio1;
       Pdata->radio2=radio2;
    }
-   gtk_object_set_data(GTK_OBJECT(alarm_dialog), "alarm", Pdata);                    
+   gtk_object_set_data(GTK_OBJECT(alarm_dialog), "alarm", Pdata);
 
    get_pref(PREF_REMIND_IN, NULL, &pref_entry);
    gtk_entry_set_text(GTK_ENTRY(remind_entry), pref_entry);
@@ -274,7 +277,7 @@ int dialog_alarm(char *title, char *frame_text,
 
    gtk_widget_show_all(alarm_dialog);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 /*
  * End Alarm GUI
@@ -346,7 +349,7 @@ void alarms_add_to_list(unsigned int unique_id,
 
    temp_alarm = malloc(sizeof(struct jp_alarms));
    if (!temp_alarm) {
-      jp_logf(JP_LOG_WARN, "alarms_add_to_list: Out of memory\n");
+      jp_logf(JP_LOG_WARN, "alarms_add_to_list: %s\n", _("Out of memory"));
       return;
    }
    temp_alarm->unique_id = unique_id;
@@ -433,7 +436,7 @@ void alarms_write_file(void)
 
    out=jp_open_home_file(EPN".alarms.tmp", "w");
    if (!out) {
-      jp_logf(JP_LOG_WARN, "Could not open "EPN".alarms.tmp file\n");
+      jp_logf(JP_LOG_WARN, _("Unable to open %s%s file\n"), EPN, ".alarms.tmp");
       return;
    }
    fail=0;
@@ -488,7 +491,7 @@ void make_command_safe(char *command)
  * Do alarm setting (play sound, or whatever).
  * if user postpones then put in postponed alarm list.
  */
-int alarms_do_one(struct Appointment *a,
+int alarms_do_one(struct Appointment *appt,
 		  unsigned long unique_id,
 		  time_t t_alarm,
 		  AlarmType type)
@@ -505,7 +508,6 @@ int alarms_do_one(struct Appointment *a,
    char date_str[50];
    char command[1024];
    char *reason;
-   long ivalue;
    long wants_windows;
    long do_command;
    const char *pref_date;
@@ -517,7 +519,7 @@ int alarms_do_one(struct Appointment *a,
 
    switch (type) {
     case ALARM_NONE:
-      return 0;
+      return EXIT_SUCCESS;
     case ALARM_NEW:
       reason=_("Appointment Reminder");
       break;
@@ -530,29 +532,29 @@ int alarms_do_one(struct Appointment *a,
     default:
       reason=_("Appointment");
    }
-   get_pref(PREF_SHORTDATE, &ivalue, &pref_date);
+   get_pref(PREF_SHORTDATE, NULL, &pref_date);
    get_pref_time_no_secs(pref_time);
 
    Pnow = localtime(&t_alarm);
 
    strftime(date_str, sizeof(date_str), pref_date, Pnow);
-   tm_copy_with_dst_adj(&begin, &(a->begin));
+   tm_copy_with_dst_adj(&begin, &(appt->begin));
    strftime(time1_str, sizeof(time1_str), pref_time, &begin);
-   tm_copy_with_dst_adj(&end, &(a->end));
+   tm_copy_with_dst_adj(&end, &(appt->end));
    strftime(time2_str, sizeof(time2_str), pref_time, &end);
    g_snprintf(time_str, sizeof(time_str), "%s %s-%s\n", date_str, time1_str, time2_str);
    desc_str[0]='\0';
    note_str[0]='\0';
-   if (a->description) {
-      strncpy(desc_str, a->description, sizeof(desc_str));
+   if (appt->description) {
+      strncpy(desc_str, appt->description, sizeof(desc_str));
       desc_str[sizeof(desc_str)-1]='\0';
    }
-   if (a->note) {
-      strncpy(note_str, a->note, sizeof(note_str));
+   if (appt->note) {
+      strncpy(note_str, appt->note, sizeof(note_str));
       note_str[sizeof(note_str)-1]='\0';
    }
 
-   get_pref(PREF_ALARM_COMMAND, &ivalue, &pref_command);
+   get_pref(PREF_ALARM_COMMAND, NULL, &pref_command);
    get_pref(PREF_DO_ALARM_COMMAND, &do_command, NULL);
 #ifdef ALARMS_DEBUG
    printf("pref_command = [%s]\n", pref_command);
@@ -602,19 +604,19 @@ int alarms_do_one(struct Appointment *a,
       command[sizeof(command)-2]='\0';
 
       make_command_safe(command);
-      jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "executing command = [%s]\n", command);
+      jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, _("executing command = [%s]\n"), command);
       system(command);
    }
 
    get_pref(PREF_OPEN_ALARM_WINDOWS, &wants_windows, NULL);
 
    if (wants_windows) {
-      return dialog_alarm("J-Pilot Alarm", reason,
+      return dialog_alarm(_("J-Pilot Alarm"), reason,
 			  time_str, desc_str, note_str,
 			  unique_id,
 			  t_alarm);
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 
@@ -660,11 +662,11 @@ gint cb_timer_alarms(gpointer data)
       printf("alarm_advance=%ld\n", temp_alarm->alarm_advance);
 #endif
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-	 if (temp_al->ma.unique_id == temp_alarm->unique_id) {
+	 if (temp_al->mappt.unique_id == temp_alarm->unique_id) {
 #ifdef ALARMS_DEBUG
-	    printf("%s\n", temp_al->ma.a.description);
+	    printf("%s\n", temp_al->mappt.appt.description);
 #endif
-	    alarms_do_one(&(temp_al->ma.a),
+	    alarms_do_one(&(temp_al->mappt.appt),
 			  temp_alarm->unique_id,
 			  temp_alarm->event_time,
 			  ALARM_MISSED);
@@ -673,7 +675,7 @@ gint cb_timer_alarms(gpointer data)
       }
       /* Be careful, this modifies the list we are parsing and
        removes the current node */
-      alarms_remove_from_to_list(temp_al->ma.unique_id);
+      alarms_remove_from_to_list(temp_al->mappt.unique_id);
    }
 
    if (next_alarm) {
@@ -684,15 +686,15 @@ gint cb_timer_alarms(gpointer data)
 	 }
 	 for (temp_alarm=next_alarm; temp_alarm; temp_alarm=ta_next) {
 	    for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-	       if (temp_al->ma.unique_id == temp_alarm->unique_id) {
+	       if (temp_al->mappt.unique_id == temp_alarm->unique_id) {
 #ifdef ALARMS_DEBUG
 		  printf("** next unique_id=%d\n", temp_alarm->unique_id);
 		  printf("** next type=%s\n", print_type(temp_alarm->type));
 		  printf("** next event_time=%s\n", print_date(temp_alarm->event_time));
 		  printf("** next alarm_advance=%ld\n", temp_alarm->alarm_advance);
-		  printf("** next %s\n", temp_al->ma.a.description);
-#endif		  
-		  alarms_do_one(&(temp_al->ma.a),
+		  printf("** next %s\n", temp_al->mappt.appt.description);
+#endif
+		  alarms_do_one(&(temp_al->mappt.appt),
 				temp_alarm->unique_id,
 				temp_alarm->event_time,
 				ALARM_NEW);
@@ -722,26 +724,26 @@ gint cb_timer_alarms(gpointer data)
 
 /*
  * This routine takes time (t) and either advances t to the next
- * occurence of a repeating appointment, or the previous occurence
- * 
+ * occurrence of a repeating appointment, or the previous occurrence
+ *
  * a is the appointment passed in
  * t is an in/out param.
  * fdom is first day of month (Sunday being 0)
  * ndim is the number of days in the month
  * forward_or_backward should be -1 for backward or 1 for forward.
  */
-int forward_backward_in_appt_time(const struct Appointment *a,
+int forward_backward_in_appt_time(const struct Appointment *appt,
 				  struct tm *t,
 				  int fdom, int ndim,
 				  int forward_or_backward)
 {
    int count, dow, freq;
 
-   freq = a->repeatFrequency;
+   freq = appt->repeatFrequency;
 
    /* Go forward in time */
    if (forward_or_backward==1) {
-      switch (a->repeatType) {
+      switch (appt->repeatType) {
        case repeatNone:
 	 break;
        case repeatDaily:
@@ -749,19 +751,19 @@ int forward_backward_in_appt_time(const struct Appointment *a,
        case repeatWeekly:
 	 for (count=0, dow=t->tm_wday; count<14; count++) {
 	    add_days_to_date(t, 1);
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	    printf("fpn: weekly forward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
 #endif
 	    dow++;
 	    if (dow==7) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	       printf("fpn: dow==7\n");
 #endif
 	       add_days_to_date(t, (freq-1)*7);
 	       dow=0;
 	    }
-	    if (a->repeatDays[dow]) {
-#ifdef ALARMS_DEBUG      
+	    if (appt->repeatDays[dow]) {
+#ifdef ALARMS_DEBUG
 	       printf("fpn: repeatDay[dow] dow=%d\n", dow);
 #endif
 	       break;
@@ -771,25 +773,25 @@ int forward_backward_in_appt_time(const struct Appointment *a,
        case repeatMonthlyByDay:
 	 add_months_to_date(t, freq);
 	 get_month_info(t->tm_mon, 1, t->tm_year, &fdom, &ndim);
-	 t->tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
+	 t->tm_mday=((appt->repeatDay+7-fdom)%7) - ((appt->repeatDay)%7) + appt->repeatDay + 1;
 	 if (t->tm_mday > ndim-1) {
 	    t->tm_mday -= 7;
 	 }
 	 break;
        case repeatMonthlyByDate:
-	 t->tm_mday=a->begin.tm_mday;
+	 t->tm_mday=appt->begin.tm_mday;
 	 add_months_to_date(t, freq);
 	 break;
        case repeatYearly:
-	 t->tm_mday=a->begin.tm_mday;
+	 t->tm_mday=appt->begin.tm_mday;
 	 add_years_to_date(t, freq);
 	 break;
       }/*switch */
-      return 0;
+      return EXIT_SUCCESS;
    }
    /* Go back in time */
    if (forward_or_backward==-1) {
-      switch (a->repeatType) {
+      switch (appt->repeatType) {
        case repeatNone:
 	 break;
        case repeatDaily:
@@ -802,14 +804,14 @@ int forward_backward_in_appt_time(const struct Appointment *a,
 #endif
 	    dow--;
 	    if (dow==-1) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	       printf("fpn: dow==-1\n");
 #endif
 	       sub_days_from_date(t, (freq-1)*7);
 	       dow=6;
 	    }
-	    if (a->repeatDays[dow]) {
-#ifdef ALARMS_DEBUG      
+	    if (appt->repeatDays[dow]) {
+#ifdef ALARMS_DEBUG
 	       printf("fpn: repeatDay[dow] dow=%d\n", dow);
 #endif
 	       break;
@@ -819,25 +821,25 @@ int forward_backward_in_appt_time(const struct Appointment *a,
        case repeatMonthlyByDay:
 	 sub_months_from_date(t, freq);
 	 get_month_info(t->tm_mon, 1, t->tm_year, &fdom, &ndim);
-	 t->tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
+	 t->tm_mday=((appt->repeatDay+7-fdom)%7) - ((appt->repeatDay)%7) + appt->repeatDay + 1;
 	 if (t->tm_mday > ndim-1) {
 	    t->tm_mday -= 7;
 	 }
 	 break;
        case repeatMonthlyByDate:
-	 t->tm_mday=a->begin.tm_mday;
+	 t->tm_mday=appt->begin.tm_mday;
 	 sub_months_from_date(t, freq);
 	 break;
        case repeatYearly:
-	 t->tm_mday=a->begin.tm_mday;
+	 t->tm_mday=appt->begin.tm_mday;
 	 sub_years_from_date(t, freq);
 	 break;
       }/*switch */
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
-static int find_prev_next(struct Appointment *a,
+static int find_prev_next(struct Appointment *appt,
 			  int adv,
 			  struct tm *date1,
 			  struct tm *date2,
@@ -880,11 +882,11 @@ static int find_prev_next(struct Appointment *a,
    memset(tm_next, 0, sizeof(*tm_next));
 
    memset(&t, 0, sizeof(t));
-   t.tm_year=a->begin.tm_year;
-   t.tm_mon=a->begin.tm_mon;
-   t.tm_mday=a->begin.tm_mday;
-   t.tm_hour=a->begin.tm_hour;
-   t.tm_min=a->begin.tm_min;
+   t.tm_year=appt->begin.tm_year;
+   t.tm_mon=appt->begin.tm_mon;
+   t.tm_mday=appt->begin.tm_mday;
+   t.tm_hour=appt->begin.tm_hour;
+   t.tm_min=appt->begin.tm_min;
    t.tm_isdst=-1;
 
    mktime(&t);
@@ -896,20 +898,20 @@ static int find_prev_next(struct Appointment *a,
 	}
 #endif
    freq = 0;
-   switch (a->repeatType) {
+   switch (appt->repeatType) {
     case repeatNone:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatNone\n");
 #endif
-      t_alarm=mktime_dst_adj(&(a->begin)) - adv;
+      t_alarm=mktime_dst_adj(&(appt->begin)) - adv;
       if ((t_alarm < t2) && (t_alarm > t1)) {
-	 memcpy(tm_prev, &(a->begin), sizeof(struct tm));
+	 memcpy(tm_prev, &(appt->begin), sizeof(struct tm));
 	 *prev_found=1;
 #ifdef ALARMS_DEBUG
 	 printf("fpn: prev_found none\n");
 #endif
       } else if (t_alarm > t2) {
-	 memcpy(tm_next, &(a->begin), sizeof(struct tm));
+	 memcpy(tm_next, &(appt->begin), sizeof(struct tm));
 	 *next_found=1;
 #ifdef ALARMS_DEBUG
 	 printf("fpn: next_found none\n");
@@ -921,8 +923,8 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatDaily\n");
 #endif
-      freq = a->repeatFrequency;
-      t_interval = a->repeatFrequency * 86400;
+      freq = appt->repeatFrequency;
+      t_interval = appt->repeatFrequency * 86400;
       if (t_interval==0) t_interval=1;
       t_alarm = mktime_dst_adj(&t);
       if ((t2 + adv) > t_alarm) {
@@ -938,24 +940,24 @@ static int find_prev_next(struct Appointment *a,
       Pnow = localtime(&t_future);
       memcpy(tm_next, Pnow, sizeof(struct tm));
       forward=backward=0;
-      /* since the code above disregarged DST in the above calcs,
+      /* since the code above disregarded DST in the above calcs,
        * we have to try to correct for it here */
-      if (tm_prev->tm_hour<a->begin.tm_hour) {
+      if (tm_prev->tm_hour<appt->begin.tm_hour) {
 	 t_past+=3600; /* 1 hour for dst */
 	 Pnow = localtime(&t_past);
 	 memcpy(tm_prev, Pnow, sizeof(struct tm));
       }
-      if (tm_prev->tm_hour>a->begin.tm_hour) {
+      if (tm_prev->tm_hour>appt->begin.tm_hour) {
 	 t_past-=3600; /* 1 hour for dst */
 	 Pnow = localtime(&t_past);
 	 memcpy(tm_prev, Pnow, sizeof(struct tm));
       }
-      if (tm_next->tm_hour<a->begin.tm_hour) {
+      if (tm_next->tm_hour<appt->begin.tm_hour) {
 	 t_future+=3600; /* 1 hour for dst */
 	 Pnow = localtime(&t_future);
 	 memcpy(tm_next, Pnow, sizeof(struct tm));
       }
-      if (tm_next->tm_hour>a->begin.tm_hour) {
+      if (tm_next->tm_hour>appt->begin.tm_hour) {
 	 t_future-=3600; /* 1 hour for dst */
 	 Pnow = localtime(&t_future);
 	 memcpy(tm_next, Pnow, sizeof(struct tm));
@@ -978,9 +980,9 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatWeekly\n");
 #endif
-      freq = a->repeatFrequency;
+      freq = appt->repeatFrequency;
       /* Check for future start date */
-      begin_days = dateToDays(&(a->begin));
+      begin_days = dateToDays(&(appt->begin));
       days = dateToDays(&t);
       if (days > begin_days) {
 	 /* appt started in past, get to current date sooner */
@@ -990,16 +992,16 @@ static int find_prev_next(struct Appointment *a,
       }
       t.tm_isdst=-1;
       mktime(&t);
-      begin_days = dateToDays(&(a->begin));
+      begin_days = dateToDays(&(appt->begin));
       days = dateToDays(&t);
 #ifdef ALARMS_DEBUG
       printf("fpn: begin_days %d days %d\n", begin_days, days);
-      printf("fpn: t.tm_wday %d a->begin.tm_wday %d\n", t.tm_wday, a->begin.tm_wday);
+      printf("fpn: t.tm_wday %d appt->begin.tm_wday %d\n", t.tm_wday, appt->begin.tm_wday);
 #endif
       get_pref(PREF_FDOW, &fdow, NULL);
 
       /* Offset is how many weeks we are off of an iteration */
-      offset = ((int)((days - t.tm_wday) - (begin_days - a->begin.tm_wday))/7)%freq;
+      offset = ((int)((days - t.tm_wday) - (begin_days - appt->begin.tm_wday))/7)%freq;
 #ifdef ALARMS_DEBUG
       printf("fpn: offset %d\n", offset);
 #endif
@@ -1010,7 +1012,7 @@ static int find_prev_next(struct Appointment *a,
       }
       found=0;
       for (count=0, i=t.tm_wday; i>=0; i--, count++) {
-	 if (a->repeatDays[i]) {
+	 if (appt->repeatDays[i]) {
 	    sub_days_from_date(&t, count);
 	    found=1;
 #ifdef ALARMS_DEBUG
@@ -1025,7 +1027,7 @@ static int find_prev_next(struct Appointment *a,
       }
       if (!found) {
 	 for (count=0, i=t.tm_wday; i<7; i++, count++) {
-	    if (a->repeatDays[i]) {
+	    if (appt->repeatDays[i]) {
 	       add_days_to_date(&t, count);
 	       found=1;
 	       break;
@@ -1038,27 +1040,27 @@ static int find_prev_next(struct Appointment *a,
       printf("fpn: repeatMonthlyByDay\n");
 #endif
       /* Check for future start date */
-      begin_days = dateToDays(&(a->begin));
+      begin_days = dateToDays(&(appt->begin));
       days = dateToDays(&t);
       if (days > begin_days) {
 	 /* appt started in past, get to current date sooner */
 	 t.tm_mon=date2->tm_mon;
 	 t.tm_year=date2->tm_year;
       }
-      freq = a->repeatFrequency;
+      freq = appt->repeatFrequency;
 #ifdef ALARMS_DEBUG
       printf("fpn: freq=%d\n", freq);
 #endif
-      offset = ((t.tm_year - a->begin.tm_year)*12 +
-		(t.tm_mon - a->begin.tm_mon))%(a->repeatFrequency);
+      offset = ((t.tm_year - appt->begin.tm_year)*12 +
+		(t.tm_mon - appt->begin.tm_mon))%(appt->repeatFrequency);
       /* This will adjust for leap year, and exceeding the end of the month */
-      if (((t.tm_year - a->begin.tm_year)*12 + (t.tm_mon - a->begin.tm_mon)) < 0) {
+      if (((t.tm_year - appt->begin.tm_year)*12 + (t.tm_mon - appt->begin.tm_mon)) < 0) {
 	 add_months_to_date(&t, offset);
       } else {
 	 sub_months_from_date(&t, offset);
       }
       get_month_info(t.tm_mon, 1, t.tm_year, &fdom, &ndim);
-      t.tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
+      t.tm_mday=((appt->repeatDay+7-fdom)%7) - ((appt->repeatDay)%7) + appt->repeatDay + 1;
 #ifdef ALARMS_DEBUG
       printf("fpn: %02d/01/%02d, fdom=%d\n", t.tm_mon+1, t.tm_year+1900, fdom);
       printf("fpn: mday = %d\n", t.tm_mday);
@@ -1079,7 +1081,7 @@ static int find_prev_next(struct Appointment *a,
       printf("fpn: repeatMonthlyByDate\n");
 #endif
       /* Check for future start date */
-      begin_days = dateToDays(&(a->begin));
+      begin_days = dateToDays(&(appt->begin));
       days = dateToDays(&t);
       if (days > begin_days) {
 	 /* appt started in past, get to current date sooner */
@@ -1087,11 +1089,11 @@ static int find_prev_next(struct Appointment *a,
 	 t.tm_year=date2->tm_year;
       }
 
-      freq = a->repeatFrequency;
-      offset = ((t.tm_year - a->begin.tm_year)*12 +
-		(t.tm_mon - a->begin.tm_mon))%(a->repeatFrequency);
+      freq = appt->repeatFrequency;
+      offset = ((t.tm_year - appt->begin.tm_year)*12 +
+		(t.tm_mon - appt->begin.tm_mon))%(appt->repeatFrequency);
       /* This will adjust for leap year, and exceeding the end of the month */
-      if (((t.tm_year - a->begin.tm_year)*12 + (t.tm_mon - a->begin.tm_mon)) < 0) {
+      if (((t.tm_year - appt->begin.tm_year)*12 + (t.tm_mon - appt->begin.tm_mon)) < 0) {
 	 add_months_to_date(&t, offset);
       } else {
 	 sub_months_from_date(&t, offset);
@@ -1101,20 +1103,20 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatYearly\n");
 #endif
-      begin_days = dateToDays(&(a->begin));
+      begin_days = dateToDays(&(appt->begin));
       days = dateToDays(&t);
       if (days > begin_days) {
 	 /* appt started in past, get to current date sooner */
 	 t.tm_year=date2->tm_year;
       }
-      freq = a->repeatFrequency;
-      offset = (t.tm_year - a->begin.tm_year)%(a->repeatFrequency);
-#ifdef ALARMS_DEBUG      
-      printf("fpn: (%d - %d)%%%d\n", t.tm_year,a->begin.tm_year,a->repeatFrequency);
+      freq = appt->repeatFrequency;
+      offset = (t.tm_year - appt->begin.tm_year)%(appt->repeatFrequency);
+#ifdef ALARMS_DEBUG
+      printf("fpn: (%d - %d)%%%d\n", t.tm_year,appt->begin.tm_year,appt->repeatFrequency);
       printf("fpn: *** years offset = %d\n", offset);
 #endif
       /* This will adjust for leap year, and exceeding the end of the month */
-      if ((t.tm_year - a->begin.tm_year) < 0) {
+      if ((t.tm_year - appt->begin.tm_year) < 0) {
 	 add_years_to_date(&t, offset);
       } else {
 	 sub_years_from_date(&t, offset);
@@ -1126,9 +1128,9 @@ static int find_prev_next(struct Appointment *a,
    while(forward || backward) {
       safety_counter++;
       if (safety_counter > 3000) {
-	 jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "find_prev_next():infinite loop, breaking\n");
-	 if (a->description) {
-	    jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "desc=[%s]\n", a->description);
+	 jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "find_prev_next(): %s\n", _("infinite loop, breaking\n"));
+	 if (appt->description) {
+	    jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "desc=[%s]\n", appt->description);
 	 }
 	 break;
       }
@@ -1144,10 +1146,10 @@ static int find_prev_next(struct Appointment *a,
 #endif
       /* Check for exceptions */
       found_exception=0;
-      for (i=0; i<a->exceptions; i++) {
-	 if ((t.tm_mday==a->exception[i].tm_mday) &&
-	     (t.tm_mon==a->exception[i].tm_mon) &&
-	     (t.tm_year==a->exception[i].tm_year)
+      for (i=0; i<appt->exceptions; i++) {
+	 if ((t.tm_mday==appt->exception[i].tm_mday) &&
+	     (t.tm_mon==appt->exception[i].tm_mon) &&
+	     (t.tm_year==appt->exception[i].tm_year)
 	     ) {
 	    found_exception=1;
 	    break;
@@ -1155,25 +1157,25 @@ static int find_prev_next(struct Appointment *a,
       }
       if (found_exception) {
 	 if (forward) {
-	    forward_backward_in_appt_time(a, &t, fdom, ndim, 1);
+	    forward_backward_in_appt_time(appt, &t, fdom, ndim, 1);
 	    continue;
 	 }
 	 if (backward) {
-	    forward_backward_in_appt_time(a, &t, fdom, ndim, -1);
+	    forward_backward_in_appt_time(appt, &t, fdom, ndim, -1);
 	    continue;
 	 }
       }
       /* See that we aren't before the begin date */
-      t_begin = mktime_dst_adj(&(a->begin));
+      t_begin = mktime_dst_adj(&(appt->begin));
       if (t_temp < t_begin - adv) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	 printf("fpn: before begin date\n");
 #endif
 	 backward=0;
       }
       /* If the appointment has an end date, see that we are not past it */
-      if (!(a->repeatForever)) {
-	 t_end = mktime_dst_adj(&(a->repeatEnd));
+      if (!(appt->repeatForever)) {
+	 t_end = mktime_dst_adj(&(appt->repeatEnd));
 	 if (t_temp > t_end) {
 #ifdef ALARMS_DEBUG
 	    printf("fpn: after end date\n");
@@ -1186,27 +1188,27 @@ static int find_prev_next(struct Appointment *a,
 	 memcpy(tm_next, &t, sizeof(t));
 	 *next_found=1;
 	 forward=0;
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	 printf("fpn: next found\n");
 #endif
       } else {
 	 memcpy(tm_prev, &t, sizeof(t));
 	 *prev_found=1;
 	 backward=0;
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	 printf("fpn: prev_found\n");
 #endif
       }
       if (forward) {
-	 forward_backward_in_appt_time(a, &t, fdom, ndim, 1);
+	 forward_backward_in_appt_time(appt, &t, fdom, ndim, 1);
 	 continue;
       }
       if (backward) {
-	 forward_backward_in_appt_time(a, &t, fdom, ndim, -1);
+	 forward_backward_in_appt_time(appt, &t, fdom, ndim, -1);
 	 continue;
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 
@@ -1238,7 +1240,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 
    jp_logf(JP_LOG_DEBUG, "alarms_find_next()\n");
 
-   if (glob_skip_all_alarms) return 0;
+   if (glob_skip_all_alarms) return EXIT_SUCCESS;
 
    if (!date1_in) {
       time(&ltime);
@@ -1302,20 +1304,20 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       /*
        * No alarm, skip
        */
-      if (!temp_al->ma.a.alarm) {
+      if (!temp_al->mappt.appt.alarm) {
 	 continue;
       }
-#ifdef ALARMS_DEBUG      
-      printf("\n[%s]\n", temp_al->ma.a.description);
+#ifdef ALARMS_DEBUG
+      printf("\n[%s]\n", temp_al->mappt.appt.description);
 #endif
       /*
        * See if the appointment starts before date1
        * and is a non repeating appointment
        */
-      if (temp_al->ma.a.repeatType == repeatNone) {
-	 t_alarm = mktime_dst_adj(&(temp_al->ma.a.begin));
+      if (temp_al->mappt.appt.repeatType == repeatNone) {
+	 t_alarm = mktime_dst_adj(&(temp_al->mappt.appt.begin));
 	 if (t_alarm < t1) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	    printf("non repeat before t1, t_alarm<t1, %ld<%ld\n",t_alarm,t1);
 #endif
 	    continue;
@@ -1323,50 +1325,50 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       }
 
       /* If the appointment has an end date, see that we are not past it */
-      if (!(temp_al->ma.a.repeatForever)) {
-	 t_end = mktime_dst_adj(&(temp_al->ma.a.repeatEnd));
+      if (!(temp_al->mappt.appt.repeatForever)) {
+	 t_end = mktime_dst_adj(&(temp_al->mappt.appt.repeatEnd));
 	 /* We need to add 24 hours to the end date to make it inclusive */
 	 t_end += 86400;
-	 t_begin = mktime_dst_adj(&(temp_al->ma.a.begin));
+	 t_begin = mktime_dst_adj(&(temp_al->mappt.appt.begin));
 	 if (t_end < t2) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	    printf("past end date\n");
 #endif
 	    continue;
 	 }
       }
 
-#ifdef ALARMS_DEBUG      
-      printf("alarm advance %d ", temp_al->ma.a.advance);
+#ifdef ALARMS_DEBUG
+      printf("alarm advance %d ", temp_al->mappt.appt.advance);
 #endif
       adv = 0;
-      if (temp_al->ma.a.advanceUnits == advMinutes) {
-#ifdef ALARMS_DEBUG      
+      if (temp_al->mappt.appt.advanceUnits == advMinutes) {
+#ifdef ALARMS_DEBUG
 	 printf("minutes\n");
 #endif
-	 adv = temp_al->ma.a.advance*60;
+	 adv = temp_al->mappt.appt.advance*60;
       }
-      if (temp_al->ma.a.advanceUnits == advHours) {
-#ifdef ALARMS_DEBUG      
+      if (temp_al->mappt.appt.advanceUnits == advHours) {
+#ifdef ALARMS_DEBUG
 	 printf("hours\n");
 #endif
-	 adv = temp_al->ma.a.advance*3600;
+	 adv = temp_al->mappt.appt.advance*3600;
       }
-      if (temp_al->ma.a.advanceUnits == advDays) {
-#ifdef ALARMS_DEBUG      
+      if (temp_al->mappt.appt.advanceUnits == advDays) {
+#ifdef ALARMS_DEBUG
 	 printf("days\n");
 #endif
-	 adv = temp_al->ma.a.advance*86400;
+	 adv = temp_al->mappt.appt.advance*86400;
       }
 
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
       printf("adv=%ld\n", adv);
 #endif
 
       t_prev=t_future=t_interval=0;
       prev_found=next_found=0;
 
-      find_prev_next(&(temp_al->ma.a),
+      find_prev_next(&(temp_al->mappt.appt),
 		     adv,
 		     &date1,
 		     &date2,
@@ -1377,7 +1379,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       t_prev=mktime_dst_adj(&tm_prev);
       t_future=mktime_dst_adj(&tm_next);
 
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
       printf("adv = %ld\n", adv);
 #endif
       /*
@@ -1386,23 +1388,23 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 
       if (prev_found) {
 	 if (t_prev - adv < t1) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	    printf("failed prev is before t1\n");
 #endif
 	    prev_found=0;
 	 }
 	 if (t_prev - adv > t2) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	    printf("failed prev is after t2\n");
 #endif
 	    continue;
 	 }
       }
       if (next_found) {
-	 if (!(temp_al->ma.a.repeatForever)) {
-	    t_end = mktime_dst_adj(&(temp_al->ma.a.repeatEnd));
+	 if (!(temp_al->mappt.appt.repeatForever)) {
+	    t_end = mktime_dst_adj(&(temp_al->mappt.appt.repeatEnd));
 	    if (t_future > t_end) {
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	       printf("failed future is after t_end\n");
 #endif
 	       next_found=0;
@@ -1415,18 +1417,18 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       printf("t_prev=   %s\n", prev_found ? print_date(t_prev):"None");
       printf("t_future= %s\n", next_found ? print_date(t_future):"None");
       printf("alarm me= %s\n", next_found ? print_date(t_future):"None");
-      printf("desc=[%s]\n", temp_al->ma.a.description);
+      printf("desc=[%s]\n", temp_al->mappt.appt.description);
 #endif
       if (!soonest_only) {
 	 if (prev_found) {
-	    alarms_add_to_list(temp_al->ma.unique_id, ALARM_MISSED, t_prev, adv);
+	    alarms_add_to_list(temp_al->mappt.unique_id, ALARM_MISSED, t_prev, adv);
 	 }
       }
       if (next_found) {
 	 add_a_next=0;
 	 if (next_alarm==NULL) {
 	    add_a_next=1;
-	 } else if 
+	 } else if
 	   (t_future - adv <= next_alarm->event_time - next_alarm->alarm_advance) {
 	      add_a_next=1;
 	      if (t_future - adv < next_alarm->event_time - next_alarm->alarm_advance) {
@@ -1445,7 +1447,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	    if (ta) {
 	       ta->next = next_alarm;
 	       next_alarm = ta;
-	       next_alarm->unique_id = temp_al->ma.unique_id;
+	       next_alarm->unique_id = temp_al->mappt.unique_id;
 	       next_alarm->type = ALARM_NEW;
 	       next_alarm->event_time = t_future;
 	       next_alarm->alarm_advance = adv;
@@ -1455,7 +1457,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    }
    free_AppointmentList(&a_list);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1488,14 +1490,14 @@ int alarms_init(unsigned char skip_past_alarms,
    }
    if (skip_all_alarms) {
       alarms_write_file();
-      return 0;
+      return EXIT_SUCCESS;
    }
 
    found_uptodate=0;
    in=jp_open_home_file(EPN".alarms", "r");
    if (!in) {
-      jp_logf(JP_LOG_WARN, "Could not open "EPN".alarms file\n");
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Unable to open %s.alarms file\n"), EPN);
+      return EXIT_FAILURE;
    }
 
    while (!feof(in)) {
@@ -1544,7 +1546,11 @@ int alarms_init(unsigned char skip_past_alarms,
 
    alarms_find_next(&tm1, &now, FALSE);
 
+   /* Pop up reminder windows for expired alarms immediately
+    * rather than waiting ALARM_INTERVAL seconds and then doing it */
+   cb_timer_alarms(NULL);
+
    gtk_timeout_add(ALARM_INTERVAL*1000, cb_timer_alarms, NULL);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
