@@ -1,4 +1,7 @@
-/* datebook.c
+/* $Id: datebook.c,v 1.48 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * datebook.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #include "config.h"
 #include "i18n.h"
 #include <stdlib.h>
@@ -39,37 +43,31 @@
 #include "libplugin.h"
 #include "password.h"
 
-#ifndef FALSE
-#define FALSE 0
-#endif
-#ifndef TRUE
-#define TRUE 1
-#endif
-
 #define DATEBOOK_EOF 7
 
 int datebook_compare(const void *v1, const void *v2)
 {
    AppointmentList **al1, **al2;
-   struct Appointment *a1, *a2;
+   struct Appointment *appt1, *appt2;
 
    al1=(AppointmentList **)v1;
    al2=(AppointmentList **)v2;
 
-   a1=&((*al1)->ma.a);
-   a2=&((*al2)->ma.a);
+   appt1=&((*al1)->mappt.appt);
+   appt2=&((*al2)->mappt.appt);
 
-   if ((a1->event) || (a2->event)) {
-      return a2->event-a1->event;
+   if ((appt1->event) || (appt2->event)) {
+      return appt2->event-appt1->event;
    }
 
    /* Jim Rees pointed out my sorting error */
-   /* return ((a1->begin.tm_hour*60 + a1->begin.tm_min) > */
-   return ((a1->begin.tm_hour*60 + a1->begin.tm_min) -
-	   (a2->begin.tm_hour*60 + a2->begin.tm_min));
+   /* return ((appt1->begin.tm_hour*60 + appt1->begin.tm_min) > */
+   return ((appt1->begin.tm_hour*60 + appt1->begin.tm_min) -
+	   (appt2->begin.tm_hour*60 + appt2->begin.tm_min));
 }
 
-static int datebook_sort(AppointmentList **al)
+int datebook_sort(AppointmentList **al,
+                  int (*compare_func)(const void*, const void*))
 {
    AppointmentList *temp_al;
    AppointmentList **sort_al;
@@ -82,14 +80,14 @@ static int datebook_sort(AppointmentList **al)
 
    if (count<2) {
       /* We don't have to sort less than 2 items */
-      return 0;
+      return EXIT_SUCCESS;
    }
 
    /* Allocate an array to be qsorted */
    sort_al = calloc(count, sizeof(AppointmentList *));
    if (!sort_al) {
-      jp_logf(JP_LOG_WARN, "datebook_sort(): Out of Memory\n");
-      return 0;
+      jp_logf(JP_LOG_WARN, "datebook_sort(): %s\n", _("Out of memory"));
+      return EXIT_FAILURE;
    }
 
    /* Set our array to be a list of pointers to the nodes in the linked list */
@@ -98,7 +96,7 @@ static int datebook_sort(AppointmentList **al)
    }
 
    /* qsort them */
-   qsort(sort_al, count, sizeof(AppointmentList *), datebook_compare);
+   qsort(sort_al, count, sizeof(AppointmentList *), compare_func);
 
    /* Put the linked list in the order of the array */
    sort_al[count-1]->next = NULL;
@@ -110,58 +108,58 @@ static int datebook_sort(AppointmentList **al)
 
    free(sort_al);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 #ifdef ENABLE_DATEBK
-int db3_hack_date(struct Appointment *a, struct tm *today)
+int db3_hack_date(struct Appointment *appt, struct tm *today)
 {
    int t1, t2;
 
    if (today==NULL) {
-      return 0;
+      return EXIT_SUCCESS;
    }
-   if (!a->note) {
-      return 0;
+   if (!appt->note) {
+      return EXIT_SUCCESS;
    }
-   if (strlen(a->note) > 8) {
-      if ((a->note[0]=='#') && (a->note[1]=='#')) {
-	 if (a->note[2]=='f') {
+   if (strlen(appt->note) > 8) {
+      if ((appt->note[0]=='#') && (appt->note[1]=='#')) {
+	 if (appt->note[2]=='f' || appt->note[2]=='F') {
 	    /* Check to see if its in the future */
-	    t1 = a->begin.tm_mday + a->begin.tm_mon*31 + a->begin.tm_year*372;
+	    t1 = appt->begin.tm_mday + appt->begin.tm_mon*31 + appt->begin.tm_year*372;
 	    t2 = today->tm_mday + today->tm_mon*31 + today->tm_year*372;
-	    if (t1 > t2) return 0;
+	    if (t1 > t2) return EXIT_SUCCESS;
 	    /* We found some silly hack, so we lie about the date */
-	    /*memcpy(&(a->begin), today, sizeof(struct tm));*/
-	    /*memcpy(&(a->end), today, sizeof(struct tm));*/
-	    a->begin.tm_mday = today->tm_mday;
-	    a->begin.tm_mon = today->tm_mon;
-	    a->begin.tm_year = today->tm_year;
-	    a->begin.tm_wday = today->tm_wday;
-	    a->begin.tm_yday = today->tm_yday;
-	    a->begin.tm_isdst = today->tm_isdst;
-	    a->end.tm_mday = today->tm_mday;
-	    a->end.tm_mon = today->tm_mon;
-	    a->end.tm_year = today->tm_year;
-	    a->end.tm_wday = today->tm_wday;
-	    a->end.tm_yday = today->tm_yday;
-	    a->end.tm_isdst = today->tm_isdst;
+	    /*memcpy(&(appt->begin), today, sizeof(struct tm));*/
+	    /*memcpy(&(appt->end), today, sizeof(struct tm));*/
+	    appt->begin.tm_mday = today->tm_mday;
+	    appt->begin.tm_mon = today->tm_mon;
+	    appt->begin.tm_year = today->tm_year;
+	    appt->begin.tm_wday = today->tm_wday;
+	    appt->begin.tm_yday = today->tm_yday;
+	    appt->begin.tm_isdst = today->tm_isdst;
+	    appt->end.tm_mday = today->tm_mday;
+	    appt->end.tm_mon = today->tm_mon;
+	    appt->end.tm_year = today->tm_year;
+	    appt->end.tm_wday = today->tm_wday;
+	    appt->end.tm_yday = today->tm_yday;
+	    appt->end.tm_isdst = today->tm_isdst;
 	    /* If the appointment has an end date, and today is past the end
 	     * date, because of this hack we would never be able to view
 	     * it anymore (or delete it).
 	     */
-	    if (!(a->repeatForever)) {
-	       if (compareTimesToDay(today, &(a->repeatEnd))==1) {
+	    if (!(appt->repeatForever)) {
+	       if (compareTimesToDay(today, &(appt->repeatEnd))==1) {
 		  /* end date is before start date, illegal appointment */
-		  /* make it legal, by only floating upto the end date */
-		  memcpy(&(a->begin), &(a->repeatEnd), sizeof(struct tm));
-		  memcpy(&(a->end), &(a->repeatEnd), sizeof(struct tm));
+		  /* make it legal, by only floating up to the end date */
+		  memcpy(&(appt->begin), &(appt->repeatEnd), sizeof(struct tm));
+		  memcpy(&(appt->end), &(appt->repeatEnd), sizeof(struct tm));
 	       }
 	    }
 	 }
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /* Note should be pretty much validated by now */
@@ -170,12 +168,14 @@ void db3_fill_struct(char *note, int type, struct db4_struct *db4)
    /* jp_logf(JP_LOG_WARN, "db3_fill_struct()\n"); */
    switch (note[2]) {
     case 'c':
+    case 'C':
       db4->floating_event=DB3_FLOAT_COMPLETE;
       break;
     case 'd':
       db4->floating_event=DB3_FLOAT_DONE;
       break;
     case 'f':
+    case 'F':
       db4->floating_event=DB3_FLOAT;
       break;
    }
@@ -206,7 +206,7 @@ void db3_fill_struct(char *note, int type, struct db4_struct *db4)
       db4->link = DB3_LINK_YES;
    }
    /* bytes 11-14 I don't understand yet */
-   /* bytes 15-18 lower 6 bits make 24 bit numer (not sure of byte order) */
+   /* bytes 15-18 lower 6 bits make 24 bit number (not sure of byte order) */
    db4->custom_alarm_sound = ((note[14] & 0x3F) << 18) +
      ((note[15] & 0x3F) << 12) +
      ((note[16] & 0x3F) << 6) +
@@ -219,7 +219,7 @@ void db3_fill_struct(char *note, int type, struct db4_struct *db4)
  * Parses the note tag and looks for db3 or db4 tags.
  * returns -1: error, 0: false, 1: true
  * for true: will in db4
- * 
+ *
  * db4 can be passed in NULL for just checking for datebk tags.
  */
 int db3_parse_tag(char *note, int *type, struct db4_struct *db4)
@@ -228,11 +228,11 @@ int db3_parse_tag(char *note, int *type, struct db4_struct *db4)
    /* NULL means any character is allowed */
    char *allowed[]={
       "#", "#", /* First 2 characters are # */
-	"@fcd", /* f floating, c completed, d done */
+	"@FfCcd", /* F or f floating, C or c completed, d done */
 	"@blL", /* b bold, l large, L large bold */
 	"@ABCDEFGHIJKLMNO", /* Category */
 	"@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_abcdefghijklmnopqrst", /* icon */
-	"@s", /* Spans midnight */ 
+	"@s", /* Spans midnight */
 	NULL, /* Lower 2 digits of next 2 chars are time zone */
 	NULL,
 	"@l\n", /* l link, EOL for datebk3 tags */
@@ -293,10 +293,10 @@ int db3_parse_tag(char *note, int *type, struct db4_struct *db4)
       }
       if (allowed[i]==NULL) continue;
       if (!strchr(allowed[i], note[i])) {
-	 return 0;	 
+	 return 0;
       }
    }
-   
+
    /* If we made it this far then its a db4 tag */
    *type=DB3_TAG_TYPE_DB4;
    if (note[i]) {
@@ -311,30 +311,44 @@ int db3_parse_tag(char *note, int *type, struct db4_struct *db4)
 }
 #endif
 
-int pc_datebook_write(struct Appointment *a, PCRecType rt,
+int pc_datebook_write(struct Appointment *appt, PCRecType rt,
 		      unsigned char attrib, unsigned int *unique_id)
 {
+#ifndef PILOT_LINK_0_12
    unsigned char record[65536];
    int rec_len;
+#else /* PILOT_LINK_0_12 */
+   pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
    buf_rec br;
    long char_set;
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set != CHAR_SET_LATIN1) {
-      if (a->description) charset_j2p((unsigned char *)a->description, strlen(a->description)+1, char_set);
-      if (a->note) charset_j2p((unsigned char *)a->note, strlen(a->note)+1, char_set);
+      if (appt->description) charset_j2p(appt->description, strlen(appt->description)+1, char_set);
+      if (appt->note) charset_j2p(appt->note, strlen(appt->note)+1, char_set);
    }
 
-   rec_len = pack_Appointment(a, record, 65535);
+#ifndef PILOT_LINK_0_12
+   rec_len = pack_Appointment(appt, record, 65535);
    if (!rec_len) {
+#else /* PILOT_LINK_0_12 */
+   RecordBuffer = pi_buffer_new(0);
+   if (pack_Appointment(appt, RecordBuffer, datebook_v1) == -1) {
+#endif /* PILOT_LINK_0_12 */
       PRINT_FILE_LINE;
       jp_logf(JP_LOG_WARN, "pack_Appointment %s\n", _("error"));
-      return -1;
+      return EXIT_FAILURE;
    }
    br.rt=rt;
    br.attrib = attrib;
+#ifndef PILOT_LINK_0_12
    br.buf = record;
    br.size = rec_len;
+#else /* PILOT_LINK_0_12 */
+   br.buf = RecordBuffer->data;
+   br.size = RecordBuffer->used;
+#endif /* PILOT_LINK_0_12 */
    /* Keep unique ID intact */
    if ((unique_id) && (*unique_id!=0)) {
       br.unique_id = *unique_id;
@@ -347,14 +361,18 @@ int pc_datebook_write(struct Appointment *a, PCRecType rt,
       *unique_id = br.unique_id;
    }
 
-   return 0;
+#ifdef PILOT_LINK_0_12
+   pi_buffer_free(RecordBuffer);
+#endif
+
+   return EXIT_SUCCESS;
 }
 
 void free_AppointmentList(AppointmentList **al)
 {
    AppointmentList *temp_al, *temp_al_next;
    for (temp_al = *al; temp_al; temp_al=temp_al_next) {
-      free_Appointment(&(temp_al->ma.a));
+      free_Appointment(&(temp_al->mappt.appt));
       temp_al_next = temp_al->next;
       free(temp_al);
    }
@@ -369,15 +387,15 @@ int datebook_copy_appointment(struct Appointment *a1,
 {
    *a2=malloc(sizeof(struct Appointment));
    if (!(*a2)) {
-      jp_logf(JP_LOG_WARN, "datebook_copy_appointment(): Out of memory\n");
-      return -1;
+      jp_logf(JP_LOG_WARN, "datebook_copy_appointment(): %s\n", _("Out of memory"));
+      return EXIT_FAILURE;
    }
    memcpy(*a2, a1, sizeof(struct Appointment));
 
    (*a2)->exception = (struct tm *)malloc(a1->exceptions * sizeof(struct tm));
    if (!(*a2)->exception) {
-      jp_logf(JP_LOG_WARN, "datebook_copy_appointment(): Out of memory 2\n");
-      return -1;
+      jp_logf(JP_LOG_WARN, "datebook_copy_appointment(): %s 2\n", _("Out of memory"));
+      return EXIT_FAILURE;
    }
    memcpy((*a2)->exception, a1->exception, a1->exceptions * sizeof(struct tm));
 
@@ -388,7 +406,7 @@ int datebook_copy_appointment(struct Appointment *a1,
       (*a2)->note=strdup(a1->note);
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 
@@ -396,24 +414,24 @@ int datebook_copy_appointment(struct Appointment *a1,
 /* Mon is 0-11 */
 /* Day is 1-31 */
 /* */
-int datebook_add_exception(struct Appointment *a, int year, int mon, int day)
+int datebook_add_exception(struct Appointment *appt, int year, int mon, int day)
 {
    struct tm *new_exception, *Ptm;
 
-   if (a->exceptions==0) {
-      a->exception=NULL;
+   if (appt->exceptions==0) {
+      appt->exception=NULL;
    }
 
-   new_exception = malloc((a->exceptions + 1) * sizeof(struct tm));
+   new_exception = malloc((appt->exceptions + 1) * sizeof(struct tm));
    if (!new_exception) {
-      jp_logf(JP_LOG_WARN, "datebook_add_exception(): Out of memory\n");
-      return -1;
+      jp_logf(JP_LOG_WARN, "datebook_add_exception(): %s\n", _("Out of memory"));
+      return EXIT_FAILURE;
    }
-   memcpy(new_exception, a->exception, (a->exceptions) * sizeof(struct tm));
-   free(a->exception);
-   a->exceptions++;
-   a->exception = new_exception;
-   Ptm = &(a->exception[a->exceptions - 1]);
+   memcpy(new_exception, appt->exception, (appt->exceptions) * sizeof(struct tm));
+   free(appt->exception);
+   appt->exceptions++;
+   appt->exception = new_exception;
+   Ptm = &(appt->exception[appt->exceptions - 1]);
    Ptm->tm_year = year;
    Ptm->tm_mon = mon;
    Ptm->tm_mday = day;
@@ -422,25 +440,7 @@ int datebook_add_exception(struct Appointment *a, int year, int mon, int day)
    Ptm->tm_sec = 0;
    Ptm->tm_isdst = -1;
    mktime(Ptm);
-   return 0;
-}
-
-int dateToSecs(struct tm *tm1)
-{
-   time_t t1;
-   struct tm *gmt;
-   struct tm tm2;
-   static time_t adj = -1;
-
-   memcpy(&tm2, tm1, sizeof(struct tm));
-   tm2.tm_isdst = 0;
-   tm2.tm_hour=0;
-   t1 = mktime(&tm2);
-   if (-1 == adj) {
-      gmt = gmtime(&t1);
-      adj = t1 - mktime(gmt);
-   }
-   return (t1+adj);
+   return EXIT_SUCCESS;
 }
 
 int dateToDays(struct tm *tm1)
@@ -490,7 +490,7 @@ int compareTimesToDay(struct tm *tm1, struct tm *tm2)
    return 0;
 }
 
-unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
+unsigned int isApptOnDate(struct Appointment *appt, struct tm *date)
 {
 /*   long fdow; */
    unsigned int ret;
@@ -503,7 +503,6 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
    };
    int exception_days;
    static int days, begin_days;
-   static struct tm cached_date;
 
    /* jp_logf(JP_LOG_DEBUG, "isApptOnDate\n"); */
 
@@ -513,13 +512,6 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
       return FALSE;
    }
 
-   /* To try to speed things up */
-   if (memcmp(date, &cached_date, sizeof(struct tm))) {
-      memcpy(&cached_date, date, sizeof(struct tm));
-   } else {
-      days=0;
-      begin_days=0;
-   }
    /* Leap year */
    if ((date->tm_year%4 == 0) &&
        !(((date->tm_year+1900)%100==0) && ((date->tm_year+1900)%400!=0))
@@ -528,7 +520,7 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
    }
 
    /* See if the appointment starts after date */
-   r = compareTimesToDay(&(a->begin), date);
+   r = compareTimesToDay(&(appt->begin), date);
    if (r == 1) {
       return FALSE;
    }
@@ -536,77 +528,71 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
       ret = TRUE;
    }
    /* If the appointment has an end date, see that we are not past it */
-   if (!(a->repeatForever)) {
-      r = compareTimesToDay(&(a->repeatEnd), date);
+   if (!(appt->repeatForever)) {
+      r = compareTimesToDay(&(appt->repeatEnd), date);
       if (r == 2) {
 	 return FALSE;
       }
    }
 
-   switch (a->repeatType) {
+   switch (appt->repeatType) {
     case repeatNone:
       break;
     case repeatDaily:
       /* See if this appt repeats on this day */
-      if (!begin_days) {
-	 begin_days = dateToDays(&(a->begin));
-      }
-      if (!days) {
-	 days = dateToDays(date);
-      }
-      ret = (((days - begin_days)%(a->repeatFrequency))==0);
+      begin_days = dateToDays(&(appt->begin));
+      days = dateToDays(date);
+
+      ret = (((days - begin_days)%(appt->repeatFrequency))==0);
       break;
     case repeatWeekly:
       get_month_info(date->tm_mon, date->tm_mday, date->tm_year, &dow, &ndim);
       /* See if the appointment repeats on this day */
       /*
-      if (a->repeatWeekstart > 1) {
-	 a->repeatWeekstart = 1;
+      if (appt->repeatWeekstart > 1) {
+	 appt->repeatWeekstart = 1;
       }
-      if (a->repeatWeekstart < 0) {
-	 a->repeatWeekstart = 0;
+      if (appt->repeatWeekstart < 0) {
+	 appt->repeatWeekstart = 0;
       }
       */
-      /*if (!(a->repeatDays[dow + a->repeatWeekstart])) {*/
-      if (!(a->repeatDays[dow])) {
+      /*if (!(appt->repeatDays[dow + appt->repeatWeekstart])) {*/
+      if (!(appt->repeatDays[dow])) {
 	 ret = FALSE;
 	 break;
       }
       /*See if we are in a week that is repeated in */
-      if (!begin_days) {
-	 begin_days = dateToDays(&(a->begin));
+      begin_days = dateToDays(&(appt->begin));
+      days = dateToDays(date);
+
+      /* In repeatWeekly Palm treats the week as running Monday-Sunday [0-6]
+       * This contrasts with the C time structures which run Sunday-Sat[0-6]
+       * The date calculation requires switching between the two.
+       * Palm tm structure = C tm structure -1 with wraparound for Sunday */
+      if (appt->begin.tm_wday == 0)
+      {
+	 ret = (days - begin_days) + 6;
+      } else {
+	 ret = (days - begin_days) + (appt->begin.tm_wday - 1);
       }
-      if (!days) {
-	 days = dateToDays(date);
-      }
-      /* get_pref(PREF_FDOW, &fdow, NULL); */
-      /* Note: Palm Bug?  I think the palm does this wrong.
-       * I prefer this way of doing it so that you can have appts repeating
-       * from Wed->Tue, for example.  The palms way prevents this */
-      /* ret = (((int)((days - begin_days - fdow)/7))%(a->repeatFrequency)==0);*/
-      /* But, here is the palm way */
-      /* ret = (((int)((days-begin_days+a->begin.tm_wday-fdow)/7))
-	     %(a->repeatFrequency)==0); */
-      /* The above seemed to be wrong for fdow=1 and dow=0 appointment */
-      ret = (((int)((days-begin_days+a->begin.tm_wday)/7))
-	     %(a->repeatFrequency)==0);
+      ret = ((((int)ret/7) % appt->repeatFrequency) == 0);
       break;
     case repeatMonthlyByDay:
       /* See if we are in a month that is repeated in */
-      ret = (((date->tm_year - a->begin.tm_year)*12 +
-       (date->tm_mon - a->begin.tm_mon))%(a->repeatFrequency)==0);
+      ret = (((date->tm_year - appt->begin.tm_year)*12 +
+       (date->tm_mon - appt->begin.tm_mon))%(appt->repeatFrequency)==0);
       if (!ret) {
 	 break;
       }
       /* If the days of the week match - good */
       /* e.g. Monday or Thur, etc. */
-      if (a->repeatDay%7 != date->tm_wday) {
+      if (appt->repeatDay%7 != date->tm_wday) {
 	 ret = FALSE;
 	 break;
       }
       /* Are they both in the same week in the month */
       /* e.g. The 3rd Mon, or the 2nd Fri, etc. */
-      week1 = a->repeatDay/7;
+      week1 = appt->repeatDay/7;
       week2 = (date->tm_mday - 1)/7;
       if (week1 != week2) {
 	 ret = FALSE;
@@ -621,58 +607,56 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
       break;
     case repeatMonthlyByDate:
       /* See if we are in a repeating month */
-      ret = (((date->tm_year - a->begin.tm_year)*12 +
-       (date->tm_mon - a->begin.tm_mon))%(a->repeatFrequency) == 0);
+      ret = (((date->tm_year - appt->begin.tm_year)*12 +
+       (date->tm_mon - appt->begin.tm_mon))%(appt->repeatFrequency) == 0);
       if (!ret) {
 	 break;
       }
       /* See if this is the date that the appt repeats on */
-      if (date->tm_mday == a->begin.tm_mday) {
+      if (date->tm_mday == appt->begin.tm_mday) {
 	 ret = TRUE;
 	 break;
       }
       /* If appt occurs after the last day of the month and this date */
       /* is the last day of the month then it occurs today */
-      ret = ((a->begin.tm_mday > days_in_month[date->tm_mon]) &&
+      ret = ((appt->begin.tm_mday > days_in_month[date->tm_mon]) &&
 	     (date->tm_mday == days_in_month[date->tm_mon]));
       break;
     case repeatYearly:
-      if ((date->tm_year - a->begin.tm_year)%(a->repeatFrequency) != 0) {
+      if ((date->tm_year - appt->begin.tm_year)%(appt->repeatFrequency) != 0) {
 	 ret = FALSE;
 	 break;
       }
-      if ((date->tm_mday == a->begin.tm_mday) &&
-	  (date->tm_mon == a->begin.tm_mon)) {
+      if ((date->tm_mday == appt->begin.tm_mday) &&
+	  (date->tm_mon == appt->begin.tm_mon)) {
 	 ret = TRUE;
 	 break;
       }
       /* Take care of Feb 29th (Leap Day) */
-      if ((a->begin.tm_mon == 1) && (a->begin.tm_mday == 29) &&
+      if ((appt->begin.tm_mon == 1) && (appt->begin.tm_mday == 29) &&
 	(date->tm_mon == 1) && (date->tm_mday == 28)) {
 	 ret = TRUE;
 	 break;
-      }   
+      }
       break;
     default:
-      jp_logf(JP_LOG_WARN, "unknown repeatType (%d) found in DatebookDB\n",
-	   a->repeatType);
+      jp_logf(JP_LOG_WARN, _("Unknown repeatType (%d) found in DatebookDB\n"),
+	   appt->repeatType);
       ret = FALSE;
    }/*switch */
 
-   if (ret) {
-      /* Check for exceptions */
-      for (i=0; i<a->exceptions; i++) {
+   /* Check for exceptions */
+   if (ret && appt->exceptions) {
+      days = dateToDays(date);
+      for (i=0; i<appt->exceptions; i++) {
 #ifdef JPILOT_DEBUG
-	 jp_logf(JP_LOG_DEBUG, "exception %d mon %d\n", i, a->exception[i].tm_mon);
-	 jp_logf(JP_LOG_DEBUG, "exception %d day %d\n", i, a->exception[i].tm_mday);
-	 jp_logf(JP_LOG_DEBUG, "exception %d year %d\n", i, a->exception[i].tm_year);
-	 jp_logf(JP_LOG_DEBUG, "exception %d yday %d\n", i, a->exception[i].tm_yday);
+	 jp_logf(JP_LOG_DEBUG, "exception %d mon %d\n", i, appt->exception[i].tm_mon);
+	 jp_logf(JP_LOG_DEBUG, "exception %d day %d\n", i, appt->exception[i].tm_mday);
+	 jp_logf(JP_LOG_DEBUG, "exception %d year %d\n", i, appt->exception[i].tm_year);
+	 jp_logf(JP_LOG_DEBUG, "exception %d yday %d\n", i, appt->exception[i].tm_yday);
 	 jp_logf(JP_LOG_DEBUG, "today is yday %d\n", date->tm_yday);
 #endif
-	 exception_days = dateToDays(&(a->exception[i]));
-	 if (!days) {
-	    days = dateToDays(date);
-	 }
+	 exception_days = dateToDays(&(appt->exception[i]));
 	 if (exception_days == days) {
 	    ret = FALSE;
 	    break;
@@ -685,10 +669,9 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
 
 int get_datebook_app_info(struct AppointmentAppInfo *ai)
 {
-   int num,i;
+   int num;
    int rec_size;
    unsigned char *buf;
-   long char_set;
 
    memset(ai, 0, sizeof(*ai));
    /* Put at least one entry in there */
@@ -700,19 +683,11 @@ int get_datebook_app_info(struct AppointmentAppInfo *ai)
       free(buf);
    }
    if ((num<0) || (rec_size<=0)) {
-      jp_logf(JP_LOG_WARN, _("Error reading %s\n"), "DatebookDB.pdb");
-      return -1;
-   }
-   get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_LATIN1) {
-      for (i = 0; i < 16; i++) {
-	 if (ai->category.name[i][0] != '\0') {
-	    charset_p2j((unsigned char *)ai->category.name[i], 16, char_set);
-	 }
-      }
+      jp_logf(JP_LOG_WARN, _("Error reading file: %s\n"), "DatebookDB.pdb");
+      return EXIT_FAILURE;
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int weed_datebook_list(AppointmentList **al, int mon, int year,
@@ -752,20 +727,20 @@ int weed_datebook_list(AppointmentList **al, int mon, int year,
     * search though it ~30 times.
     */
    for (prev_al=NULL, tal=*al; tal; tal = next_al) {
-      if (skip_privates && (tal->ma.attrib & dlpRecAttrSecret)) {
+      if (skip_privates && (tal->mappt.attrib & dlpRecAttrSecret)) {
 	 next_al=tal->next;
 	 continue;
       }
       trash_it=0;
       /* See if the appointment starts after the last day of the month */
-      r = compareTimesToDay(&(tal->ma.a.begin), &tm_ldom);
+      r = compareTimesToDay(&(tal->mappt.appt.begin), &tm_ldom);
       if (r == 1) {
 	 trash_it=1;
 	 goto trash;
       }
       /* If the appointment has an end date, see if it ended before the 1st */
-      if (!(tal->ma.a.repeatForever)) {
-	 r = compareTimesToDay(&(tal->ma.a.repeatEnd), &tm_fdom);
+      if (!(tal->mappt.appt.repeatForever)) {
+	 r = compareTimesToDay(&(tal->mappt.appt.repeatEnd), &tm_fdom);
 	 if (r == 2) {
 	    trash_it=1;
 	    goto trash;
@@ -773,14 +748,14 @@ int weed_datebook_list(AppointmentList **al, int mon, int year,
       }
       /* No repeat */
       /* See if its a non-repeating appointment that is this month/year */
-      if (tal->ma.a.repeatType==repeatNone) {
-	 if ((tal->ma.a.begin.tm_year==year) &&
-	     (tal->ma.a.begin.tm_mon==mon)) {
-	    tm_test.tm_mday=tal->ma.a.begin.tm_mday;
+      if (tal->mappt.appt.repeatType==repeatNone) {
+	 if ((tal->mappt.appt.begin.tm_year==year) &&
+	     (tal->mappt.appt.begin.tm_mon==mon)) {
+	    tm_test.tm_mday=tal->mappt.appt.begin.tm_mday;
 	    mktime(&tm_test);
 	    /* Go ahead and mark this day (highlight it) */
-	    if (isApptOnDate(&(tal->ma.a), &tm_test)) {
-	       *mask = *mask | (1 << ((tal->ma.a.begin.tm_mday)-1));
+	    if (isApptOnDate(&(tal->mappt.appt), &tm_test)) {
+	       *mask = *mask | (1 << ((tal->mappt.appt.begin.tm_mday)-1));
 	    }
 	 } else {
 	    trash_it=1;
@@ -789,59 +764,55 @@ int weed_datebook_list(AppointmentList **al, int mon, int year,
       }
       /* Daily */
       /* See if this daily appt repeats this month or not */
-      if (tal->ma.a.repeatType==repeatDaily) {
-	 begin_days = dateToDays(&(tal->ma.a.begin));
-	 ret = ((days - begin_days)%(tal->ma.a.repeatFrequency));
+      if (tal->mappt.appt.repeatType==repeatDaily) {
+	 begin_days = dateToDays(&(tal->mappt.appt.begin));
+	 ret = ((days - begin_days)%(tal->mappt.appt.repeatFrequency));
 	 if ((ret>31) || (ret<-31)) {
 	    trash_it=1;
 	    goto trash;
 	 }
       }
       /* Weekly */
-      if (tal->ma.a.repeatType==repeatWeekly) {
-	 begin_days = dateToDays(&(tal->ma.a.begin));
+      if (tal->mappt.appt.repeatType==repeatWeekly) {
+	 begin_days = dateToDays(&(tal->mappt.appt.begin));
 	 /* Note: Palm Bug?  I think the palm does this wrong.
 	  * I prefer this way of doing it so that you can have appts repeating
 	  * from Wed->Tue, for example.  The palms way prevents this */
-	 /* ret = (((int)((days - begin_days - fdow)/7))%(a->repeatFrequency)==0);*/
+	 /* ret = (((int)((days - begin_days - fdow)/7))%(appt->repeatFrequency)==0);*/
 	 /* But, here is the palm way */
-	 ret = (((int)((days-begin_days+tal->ma.a.begin.tm_wday-fdow)/7))
-		%(tal->ma.a.repeatFrequency));
+	 ret = (((int)((days-begin_days+tal->mappt.appt.begin.tm_wday-fdow)/7))
+		%(tal->mappt.appt.repeatFrequency));
 	 if ((ret>6) || (ret<-6)) {
 	    trash_it=1;
 	    goto trash;
 	 }
       }
       /* Monthly */
-      if ((tal->ma.a.repeatType==repeatMonthlyByDay) ||
-	  (tal->ma.a.repeatType==repeatMonthlyByDate)) {
+      if ((tal->mappt.appt.repeatType==repeatMonthlyByDay) ||
+	  (tal->mappt.appt.repeatType==repeatMonthlyByDate)) {
 	 /* See if we are in a month that is repeated in */
-	 ret = (((year - tal->ma.a.begin.tm_year)*12 +
-		 (mon - tal->ma.a.begin.tm_mon))%(tal->ma.a.repeatFrequency)==0);
+	 ret = (((year - tal->mappt.appt.begin.tm_year)*12 +
+		 (mon - tal->mappt.appt.begin.tm_mon))%(tal->mappt.appt.repeatFrequency)==0);
 	 if (!ret) {
 	    trash_it=1;
 	    goto trash;
 	 }
-	 if (tal->ma.a.repeatType==repeatMonthlyByDay) {
-	    tm_test.tm_mday=tal->ma.a.begin.tm_mday;
+	 if (tal->mappt.appt.repeatType==repeatMonthlyByDay) {
+	    tm_test.tm_mday=tal->mappt.appt.begin.tm_mday;
 	    mktime(&tm_test);
-	    if (isApptOnDate(&(tal->ma.a), &tm_test)) {
-	       *mask = *mask | (1 << ((tal->ma.a.begin.tm_mday)-1));
-	       trash_it=1;
-	       goto trash;
+	    if (isApptOnDate(&(tal->mappt.appt), &tm_test)) {
+	       *mask = *mask | (1 << ((tal->mappt.appt.begin.tm_mday)-1));
 	    }
 	 }
       }
       /* Yearly */
       /* See if its a yearly appointment that does reoccur on this month */
-      if (tal->ma.a.repeatType==repeatYearly) {
-	 if ((tal->ma.a.begin.tm_mon==mon)) {
-	    tm_test.tm_mday=tal->ma.a.begin.tm_mday;
+      if (tal->mappt.appt.repeatType==repeatYearly) {
+	 if ((tal->mappt.appt.begin.tm_mon==mon)) {
+	    tm_test.tm_mday=tal->mappt.appt.begin.tm_mday;
 	    mktime(&tm_test);
-	    if (isApptOnDate(&(tal->ma.a), &tm_test)) {
-	       *mask = *mask | (1 << ((tal->ma.a.begin.tm_mday)-1));
-	       /* trash_it=1; */
-	       /* goto trash; */
+	    if (isApptOnDate(&(tal->mappt.appt), &tm_test)) {
+	       *mask = *mask | (1 << ((tal->mappt.appt.begin.tm_mday)-1));
 	    }
 	 } else {
 	    trash_it=1;
@@ -857,14 +828,14 @@ int weed_datebook_list(AppointmentList **al, int mon, int year,
 	    *al=tal->next;
 	 }
 	 next_al=tal->next;
-	 free_Appointment(&(tal->ma.a));
+	 free_Appointment(&(tal->mappt.appt));
 	 free(tal);
       } else {
 	 prev_al=tal;
 	 next_al=tal->next;
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int appointment_on_day_list(int mon, int year, int *mask)
@@ -904,8 +875,8 @@ int appointment_on_day_list(int mon, int year, int *mask)
       mktime(&tm_dom);
 
       for (tal=al; tal; tal = tal->next) {
-	 if (skip_privates && (tal->ma.attrib & dlpRecAttrSecret)) continue;
-	 if (isApptOnDate(&(tal->ma.a), &tm_dom)) {
+	 if (skip_privates && (tal->mappt.attrib & dlpRecAttrSecret)) continue;
+	 if (isApptOnDate(&(tal->mappt.appt), &tm_dom)) {
 	    *mask = *mask | bit;
 	    break;
 	 }
@@ -913,7 +884,7 @@ int appointment_on_day_list(int mon, int year, int *mask)
    }
    free_AppointmentList(&al);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int get_days_appointments(AppointmentList **appointment_list, struct tm *now,
@@ -932,7 +903,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
    GList *records;
    GList *temp_list;
    int recs_returned, i, num;
-   struct Appointment a;
+   struct Appointment appt;
    AppointmentList *temp_a_list;
    long keep_modified, keep_deleted;
    int keep_priv;
@@ -940,6 +911,9 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
    long char_set;
    char *buf;
    size_t len;
+#ifdef PILOT_LINK_0_12
+   pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
 #ifdef ENABLE_DATEBK
    long use_db3_tags;
    time_t ltime;
@@ -976,6 +950,9 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
    recs_returned = 0;
 
    num = jp_read_DB_files("DatebookDB", &records);
+   if (-1 == num)
+     return 0;
+
    if (total_records) *total_records = num;
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
@@ -991,60 +968,74 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
 	 continue;
       }
 
-      if ( ((br->rt==DELETED_PALM_REC) && (!keep_deleted)) ||
-	  ((br->rt==MODIFIED_PALM_REC) && (!keep_modified)) ) {
+      if ( ((br->rt==DELETED_PALM_REC)  && (!keep_deleted)) ||
+	   ((br->rt==DELETED_PC_REC)    && (!keep_deleted)) ||
+	   ((br->rt==MODIFIED_PALM_REC) && (!keep_modified)) ) {
 	 continue;
       }
-      if ((keep_priv != SHOW_PRIVATES) && 
+      if ((keep_priv != SHOW_PRIVATES) &&
 	  (br->attrib & dlpRecAttrSecret)) {
 	 continue;
       }
 
-      a.exception=NULL;
-      a.description=NULL;
-      a.note=NULL;
-      num = unpack_Appointment(&a, br->buf, br->size);
+      appt.exception=NULL;
+      appt.description=NULL;
+      appt.note=NULL;
+#ifndef PILOT_LINK_0_12
+      num = unpack_Appointment(&appt, br->buf, br->size);
+#endif /* ! PILOT_LINK_0_12 */
 
+#ifndef PILOT_LINK_0_12
       if (num <= 0) {
+#else /* PILOT_LINK_0_12 */
+      RecordBuffer = pi_buffer_new(br->size);
+      memcpy(RecordBuffer->data, br->buf, br->size);
+      RecordBuffer->used = br->size;
+      if (unpack_Appointment(&appt, RecordBuffer, datebook_v1) == -1) {
+	 pi_buffer_free(RecordBuffer);
+#endif /* PILOT_LINK_0_12 */
 	 continue;
       }
+#ifdef PILOT_LINK_0_12
+      pi_buffer_free(RecordBuffer);
+#endif /* PILOT_LINK_0_12 */
 
 #ifdef ENABLE_DATEBK
       if (use_db3_tags) {
-	 db3_hack_date(&a, &today);
+	 db3_hack_date(&appt, &today);
       }
 #endif
       if (now!=NULL) {
-	 if (!isApptOnDate(&a, now)) {
-	    free_Appointment(&a);
+	 if (!isApptOnDate(&appt, now)) {
+	    free_Appointment(&appt);
 	    continue;
 	 }
       }
 
       get_pref(PREF_CHAR_SET, &char_set, NULL);
 
-      if (a.description) {
-	 if ((buf = (char *)malloc((len = strlen(a.description)*2+1))) != NULL) {
-	    multibyte_safe_strncpy(buf, a.description, len);
-	    charset_p2j((unsigned char *)buf, len, char_set);
-	    if (strlen(buf) > strlen(a.description)) {
-	       free(a.description);
-	       a.description = strdup(buf);
+      if (appt.description) {
+	 if ((buf = (char *)malloc((len = strlen(appt.description)*2+1))) != NULL) {
+	    multibyte_safe_strncpy(buf, appt.description, len);
+	    charset_p2j(buf, len, char_set);
+	    if (strlen(buf) > strlen(appt.description)) {
+	       free(appt.description);
+	       appt.description = strdup(buf);
 	    } else {
-	       multibyte_safe_strncpy(a.description, buf, strlen(a.description)+1);
+	       multibyte_safe_strncpy(appt.description, buf, strlen(appt.description)+1);
 	    }
 	    free(buf);
 	 }
       }
-      if (a.note) {
-	 if ((buf = (char *) malloc((len = strlen(a.note)*2+1))) != NULL) {
-	    multibyte_safe_strncpy(buf, a.note, len);
-	    charset_p2j((unsigned char *)buf, len, char_set);
-	    if (strlen(buf) > strlen(a.note)) {
-	       free(a.note);
-	       a.note = strdup(buf);
+      if (appt.note) {
+	 if ((buf = (char *) malloc((len = strlen(appt.note)*2+1))) != NULL) {
+	    multibyte_safe_strncpy(buf, appt.note, len);
+	    charset_p2j(buf, len, char_set);
+	    if (strlen(buf) > strlen(appt.note)) {
+	       free(appt.note);
+	       appt.note = strdup(buf);
 	    } else {
-	       multibyte_safe_strncpy(a.note, buf, strlen(a.note)+1);
+	       multibyte_safe_strncpy(appt.note, buf, strlen(appt.note)+1);
 	    }
 	    free(buf);
 	 }
@@ -1052,15 +1043,15 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
 
       temp_a_list = malloc(sizeof(AppointmentList));
       if (!temp_a_list) {
-	 jp_logf(JP_LOG_WARN, "get_days_appointments(): Out of memory\n");
-	 free_Appointment(&a);
+	 jp_logf(JP_LOG_WARN, "get_days_appointments(): %s\n", _("Out of memory"));
+	 free_Appointment(&appt);
 	 break;
       }
-      memcpy(&(temp_a_list->ma.a), &a, sizeof(struct Appointment));
+      memcpy(&(temp_a_list->mappt.appt), &appt, sizeof(struct Appointment));
       temp_a_list->app_type = DATEBOOK;
-      temp_a_list->ma.rt = br->rt;
-      temp_a_list->ma.attrib = br->attrib;
-      temp_a_list->ma.unique_id = br->unique_id;
+      temp_a_list->mappt.rt = br->rt;
+      temp_a_list->mappt.attrib = br->attrib;
+      temp_a_list->mappt.unique_id = br->unique_id;
       temp_a_list->next = *appointment_list;
       *appointment_list = temp_a_list;
       recs_returned++;
@@ -1068,7 +1059,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
 
    jp_free_DB_records(&records);
 
-   datebook_sort(appointment_list);
+   datebook_sort(appointment_list, datebook_compare);
 
    jp_logf(JP_LOG_DEBUG, "Leaving get_days_appointments()\n");
 
