@@ -1,3 +1,25 @@
+/* $Id: japanese.c,v 1.10 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * japanese.c
+ * A module of J-Pilot http://jpilot.org
+ *
+ * Copyright (C) 1999 by Hiroshi Kawashima
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ ******************************************************************************/
+
 /*
 	Japanization library
 	Convert Palm <-> Unix Japanese Code, ie:
@@ -24,12 +46,12 @@ void multibyte_safe_strncpy(char *dst, char *src, size_t max_len);
 #define isEuc(c) \
     (0xa0 < ((unsigned char) (c)) && ((unsigned char) (c)) < 0xff)
 
-/* convert SJIS char to EUC char 
+/* convert SJIS char to EUC char
 
    this does not support Machine dependent codes.
    args: hi: first byte of sjis char.
          lo: second byte of sjis char.
-   return:   euc char in 16bit value. 
+   return:   euc char in 16bit value.
 
  */
 static unsigned int SjisToEuc(unsigned char hi, unsigned char lo)
@@ -41,20 +63,20 @@ static unsigned int SjisToEuc(unsigned char hi, unsigned char lo)
             (lo + (lo >= 0x7f ? 0x60 : 0x61));
 }
 
-/* 
+/*
    args: source char pointer, destination source pointer, a length of srting
-   Length include null termination.  
+   Length include null termination.
    return the pointer of nul termination code.
  */
-unsigned char *Sjis2EucCpy(unsigned char *dest, unsigned char *src, int max_len)
+char *Sjis2EucCpy(char *dest, char *src, int max_len)
 {
     unsigned char *p, *q;
     unsigned char hi, lo;
     unsigned int w;
     int n = 0;
 
-    p = src;
-    q = dest;
+    p = (unsigned char *)src;
+    q = (unsigned char *)dest;
     while ((*p) && (n < max_len-2)) {
         if (isSjis1stByte(*p)) {
   	    hi = *p++;
@@ -69,7 +91,7 @@ unsigned char *Sjis2EucCpy(unsigned char *dest, unsigned char *src, int max_len)
 	    n += 2;
 	} else if ((*p) & 0x80) {	            /* irregular japanese char */
 	    p++;                                    /* ??abort and return NULL?? */
-	    /* discard it */ 
+	    /* discard it */
 	} else {
 	    *q++ = *p++;
 	    n++;
@@ -81,26 +103,26 @@ unsigned char *Sjis2EucCpy(unsigned char *dest, unsigned char *src, int max_len)
     } else {
 	    *q = '\0';
     }
-    return q;
+    return (char *)q;
 }
 
-/* 
+/*
    convert strings from Sjis to EUC.
-   max_len includes null termiantion.
+   max_len includes null termination.
    size of buf must be more than max_len.
 
 */
 
-void Sjis2Euc(unsigned char *buf, int max_len)
+void Sjis2Euc(char *buf, int max_len)
 {
-	unsigned char *dst;
+	char *dst;
 
 	if (buf == NULL) return;
-	if ((dst = (unsigned char *)malloc(max_len)) != NULL) { 
+	if ((dst = (char *)malloc(max_len)) != NULL) {
                             /* assign buffer for destination. */
-		if (Sjis2EucCpy(dst, buf, max_len) != NULL) { 
+		if (Sjis2EucCpy(dst, buf, max_len) != NULL) {
 			multibyte_safe_strncpy(buf, dst, max_len);
-			buf[max_len-1] = '\0';  /* i am a paranoire B-) */
+			buf[max_len-1] = '\0';  /* i am a paranoid B-) */
 		}
 		free(dst);
 	}
@@ -108,25 +130,25 @@ void Sjis2Euc(unsigned char *buf, int max_len)
 
 /*
    convert strings from Sjis to EUC with buffer extension.
-   max_len includes null termiantion.
+   max_len includes null termination.
    size of buf must be more than max_len.
 
 */
 
-void Sjis2Euc_x(unsigned char *buf, int max_len)
+void Sjis2Euc_x(char *buf, int max_len)
 {
-    unsigned char *dst;
-    unsigned char *p;
-    
+    char *dst;
+    char *p;
+
     if (buf == NULL) return;
-    if ((dst = (unsigned char *)malloc(max_len*2)) == NULL) return; /* assign buffer for destination. */
-    if ((p = Sjis2EucCpy(dst, buf, max_len*2)) != NULL) { 
+    if ((dst = (char *)malloc(max_len*2)) == NULL) return; /* assign buffer for destination. */
+    if ((p = Sjis2EucCpy(dst, buf, max_len*2)) != NULL) {
 	if (strlen(dst) > strlen(buf)) {
 	    free(buf);
 	    buf = strdup(dst);
 	} else {
 	    multibyte_safe_strncpy(buf, dst, max_len);
-	    buf[max_len-1] = '\0';  /* i am a paranoire B-) */
+	    buf[max_len-1] = '\0';  /* i am a paranoid B-) */
 	}
     }
     free(dst);
@@ -136,7 +158,7 @@ void Sjis2Euc_x(unsigned char *buf, int max_len)
    Convert one char from euc to sjis.
    args:  hi:  first byte of euc code.
           lo:  second byte of euc code.
-   return:  16bit value of sjis char code. 
+   return:  16bit value of sjis char code.
  */
 static unsigned int EucToSjis(unsigned char hi, unsigned char lo)
 {
@@ -147,20 +169,20 @@ static unsigned int EucToSjis(unsigned char hi, unsigned char lo)
     return ((hi / 2 + (hi < 0xdf ? 0x30 : 0x70)) << 8) | (lo - 2);
 }
 
-/* 
-   Convert string from euc to sjis with coping to another buffer. 
-   Theoritically, strlen(EUC) >= strlen(SJIS), 
+/*
+   Convert string from euc to sjis with coping to another buffer.
+   Theoretically, strlen(EUC) >= strlen(SJIS),
     then it is ok that dest == src.
  */
-unsigned char *Euc2SjisCpy(unsigned char *dest, unsigned char *src, int max_len)
+char *Euc2SjisCpy(char *dest, char *src, int max_len)
 {
-    unsigned char *p, *q; 
+    unsigned char *p, *q;
     unsigned char hi, lo;
     unsigned int w;
     int n = 0;
 
-    p = src;   
-    q = dest;
+    p = (unsigned char *)src;
+    q = (unsigned char *)dest;
     while ((*p) && (n < max_len-2)) {
 	if (isEucKana(*p)) {      /* euc kana(2byte) -> sjis(1byte) */
 	    p++;
@@ -193,29 +215,29 @@ unsigned char *Euc2SjisCpy(unsigned char *dest, unsigned char *src, int max_len)
   this function exists for symmetry.
  */
 
-void Euc2Sjis(unsigned char *buf, int max_len)
+void Euc2Sjis(char *buf, int max_len)
 {
 	if (buf == NULL) return;
 	if (max_len <= 0) return;
 	(void *)Euc2SjisCpy(buf, buf, max_len);
 }
 
-/* 
+/*
    convert strings from Sjis to EUC.
-   max_len includes null termiantion.
+   max_len includes null termination.
    size of buf must be more than max_len.
 
 */
 
-void jp_Sjis2Euc(unsigned char *buf, int max_len)
+void jp_Sjis2Euc(char *buf, int max_len)
 {
-	unsigned char dst[65536];
+	char dst[65536];
 
 	if (buf == NULL) return;
 	if (max_len > 0xFFFF) max_len = 0xFFFF;
-	if (Sjis2EucCpy(dst, buf, max_len) != NULL) { 
+	if (Sjis2EucCpy(dst, buf, max_len) != NULL) {
 		multibyte_safe_strncpy(buf, dst, max_len);
-		buf[max_len-1] = '\0';  /* i am a paranoire B-) */
+		buf[max_len-1] = '\0';  /* i am a paranoid B-) */
 	}
 }
 
