@@ -1,4 +1,7 @@
-/* libplugin.c
+/* $Id: libplugin.c,v 1.24 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * libplugin.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -28,6 +32,7 @@
 #include <glib.h>
 
 #include "libplugin.h"
+#include "i18n.h"
 
 void jp_init()
 {
@@ -82,11 +87,23 @@ const char *jp_strstr(const char *haystack, const char *needle, int case_sense)
    }
 }
 
+void jp_pack_htonl(unsigned char *dest, unsigned long l)
+{
+    dest[3]=l & 0xFF;
+    dest[2]=l>>8 & 0xFF;
+    dest[1]=l>>16 & 0xFF;
+    dest[0]=l>>24 & 0xFF;
+}
+
+void jp_unpack_ntohl(unsigned long *l, unsigned char *src)
+{
+    *l=src[0]<<24 | src[1]<<16 | src[2]<<8 | src[3];
+}
+
 static int pack_header(PC3RecordHeader *header, unsigned char *packed_header)
 {
    unsigned char *p;
    unsigned long l;
-   unsigned long len;
 
    l=0;
    p=packed_header;
@@ -99,47 +116,24 @@ static int pack_header(PC3RecordHeader *header, unsigned char *packed_header)
     * unsigned long rt;
     * unsigned char attrib;
     */
-   len = sizeof(unsigned long) +
-     sizeof(unsigned long) +
-     sizeof(unsigned long) +
-     sizeof(unsigned long) +
-     sizeof(unsigned long) +
-     sizeof(unsigned char);
+    /* 4 + 4 + 4 + 4 + 4 + 1 */
+   header->header_len = 21;
 
    header->header_version = 2;
 
-   header->header_len = len;
+   jp_pack_htonl(p, header->header_len);
+   jp_pack_htonl(p+4, header->header_version);
+   jp_pack_htonl(p+8, header->rec_len);
+   jp_pack_htonl(p+12, header->unique_id);
+   jp_pack_htonl(p+16, header->rt);
+   memcpy(p+20, &header->attrib, 1);
 
-   l=htonl(header->header_len);
-   memcpy(p, &l, sizeof(l));
-   p+=sizeof(l);
-
-   l=htonl(header->header_version);
-   memcpy(p, &l, sizeof(l));
-   p+=sizeof(l);
-
-   l=htonl(header->rec_len);
-   memcpy(p, &l, sizeof(l));
-   p+=sizeof(l);
-
-   l=htonl(header->unique_id);
-   memcpy(p, &l, sizeof(l));
-   p+=sizeof(l);
-
-   l=htonl(header->rt);
-   memcpy(p, &l, sizeof(l));
-   p+=sizeof(l);
-
-   memcpy(p, &header->attrib, sizeof(unsigned char));
-   p+=sizeof(unsigned char);
-
-   return len;
+   return header->header_len;
 }
 
 static int unpack_header(PC3RecordHeader *header, unsigned char *packed_header)
 {
    unsigned char *p;
-   unsigned long l;
 
    /*
     * Header structure:
@@ -152,60 +146,43 @@ static int unpack_header(PC3RecordHeader *header, unsigned char *packed_header)
     */
    p = packed_header;
 
-   memcpy(&l, p, sizeof(l));
-   header->header_len=ntohl(l);
-   p+=sizeof(l);
-
-   memcpy(&l, p, sizeof(l));
-   header->header_version=ntohl(l);
-   p+=sizeof(l);
+   jp_unpack_ntohl(&(header->header_len), p);
+   jp_unpack_ntohl(&(header->header_version), p+4);
 
    if (header->header_version > 2) {
-      jp_logf(JP_LOG_WARN, "Unknown PC header version = %d\n", header->header_version);
+      jp_logf(JP_LOG_WARN, _("Unknown PC header version = %d\n"), header->header_version);
    }
 
-   memcpy(&l, p, sizeof(l));
-   header->rec_len=ntohl(l);
-   p+=sizeof(l);
+   jp_unpack_ntohl(&(header->rec_len), p+8);
+   jp_unpack_ntohl(&(header->unique_id), p+12);
+   jp_unpack_ntohl(&(header->rt), p+16);
+   header->attrib = p[20];
 
-   memcpy(&l, p, sizeof(l));
-   header->unique_id=ntohl(l);
-   p+=sizeof(l);
-
-   memcpy(&l, p, sizeof(l));
-   header->rt=ntohl(l);
-   p+=sizeof(l);
-
-   memcpy(&(header->attrib), p, sizeof(unsigned char));
-   p+=sizeof(unsigned char);
-
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /* FIXME: Add jp_ and document. */
 int read_header(FILE *pc_in, PC3RecordHeader *header)
 {
-   unsigned long l, len;
    unsigned char packed_header[256];
    int num;
 
-   num = fread(&l, sizeof(l), 1, pc_in);
+   num = fread(packed_header, 4, 1, pc_in);
    if (feof(pc_in)) {
       return JPILOT_EOF;
-   }      
+   }
    if (num!=1) {
       return num;
    }
-   memcpy(packed_header, &l, sizeof(l));
-   len=ntohl(l);
-   if (len > sizeof(packed_header)-1) {
-      jp_logf(JP_LOG_WARN, "read_header() error\n");
-      return -1;
+   jp_unpack_ntohl(&(header->header_len), packed_header);
+   if (header->header_len > sizeof(packed_header)-1) {
+      jp_logf(JP_LOG_WARN, "read_header() %s\n", _("error"));
+      return EXIT_FAILURE;
    }
-   num = fread(packed_header+sizeof(l), len-sizeof(l), 1, pc_in);
+   num = fread(packed_header+4, (header->header_len)-4, 1, pc_in);
    if (feof(pc_in)) {
       return JPILOT_EOF;
-   }      
+   }
    if (num!=1) {
       return num;
    }
@@ -230,6 +207,8 @@ int write_header(FILE *pc_out, PC3RecordHeader *header)
    len = pack_header(header, packed_header);
    if (len>0) {
       fwrite(packed_header, len, 1, pc_out);
+   } else {
+      jp_logf(JP_LOG_WARN, "%s:%d pack_header returned error\n", __FILE__, __LINE__);
    }
 
    return len;
@@ -254,7 +233,7 @@ int jp_edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
 }
 
 /*
- * file must not be open elsewhere when this is called 
+ * file must not be open elsewhere when this is called
  * the first line in file is 0
  */
 int jp_install_remove_line(int deleted_line)
@@ -268,14 +247,14 @@ int jp_install_remove_line(int deleted_line)
    in = jp_open_home_file(EPN"_to_install", "r");
    if (!in) {
       jp_logf(JP_LOG_DEBUG, "failed opening install_file\n");
-      return -1;
+      return EXIT_FAILURE;
    }
 
    out = jp_open_home_file(EPN"_to_install.tmp", "w");
    if (!out) {
       fclose(in);
       jp_logf(JP_LOG_DEBUG, "failed opening install_file.tmp\n");
-      return -1;
+      return EXIT_FAILURE;
    }
 
    for (line_count=0; (!feof(in)); line_count++) {
@@ -297,7 +276,7 @@ int jp_install_remove_line(int deleted_line)
 
    rename_file(EPN"_to_install.tmp", EPN"_to_install");
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int jp_install_append_line(char *line)
@@ -307,17 +286,17 @@ int jp_install_append_line(char *line)
 
    out = jp_open_home_file(EPN"_to_install", "a");
    if (!out) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    r = fprintf(out, "%s\n", line);
    if (r==EOF) {
       fclose(out);
-      return -1;
+      return EXIT_FAILURE;
    }
    fclose(out);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*returns 1 if found */
@@ -381,7 +360,7 @@ int jp_free_DB_records(GList **br_list)
    g_list_free(*br_list);
    *br_list=NULL;
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /* These next 2 functions were copied from pi-file.c in the pilot-link app */
@@ -438,7 +417,7 @@ static int raw_header_to_header(RawDBHeader *rdbh, DBHeader *dbh)
    dbh->next_record_list_id = bytes_to_bin(rdbh->next_record_list_id, 4);
    dbh->number_of_records = bytes_to_bin(rdbh->number_of_records, 2);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
@@ -451,7 +430,7 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
    char PDB_name[FILENAME_MAX];
 
    if ((!buf_size) || (!buf)) {
-      return -1;
+      return EXIT_FAILURE;
    }
    *buf = NULL;
    *buf_size=0;
@@ -459,50 +438,50 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
    g_snprintf(PDB_name, sizeof(PDB_name), "%s.pdb", DB_name);
    in = jp_open_home_file(PDB_name, "r");
    if (!in) {
-      jp_logf(JP_LOG_WARN, "%s:%d Error opening %s\n", __FILE__, __LINE__, PDB_name);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("%s:%d Error opening file: %s\n"), __FILE__, __LINE__, PDB_name);
+      return EXIT_FAILURE;
    }
    num = fread(&rdbh, sizeof(RawDBHeader), 1, in);
    if (num != 1) {
       if (ferror(in)) {
-	 jp_logf(JP_LOG_WARN, "%s:%d Error reading %s\n", __FILE__, __LINE__, PDB_name);
+	 jp_logf(JP_LOG_WARN, _("%s:%d Error reading file: %s\n"), __FILE__, __LINE__, PDB_name);
 	 fclose(in);
-	 return -1;
+	 return EXIT_FAILURE;
       }
       if (feof(in)) {
 	 fclose(in);
 	 return JPILOT_EOF;
-      }      
+      }
    }
    raw_header_to_header(&rdbh, &dbh);
 
    num = get_app_info_size(in, &rec_size);
    if (num) {
       fclose(in);
-      return -1;
+      return EXIT_FAILURE;
    }
 
    fseek(in, dbh.app_info_offset, SEEK_SET);
    *buf=malloc(rec_size);
    if (!(*buf)) {
-      jp_logf(JP_LOG_WARN, "jp_get_app_info(): Out of memory\n");
+      jp_logf(JP_LOG_WARN, "jp_get_app_info(): %s\n", _("Out of memory"));
       fclose(in);
-      return -1;
+      return EXIT_FAILURE;
    }
    num = fread(*buf, rec_size, 1, in);
    if (num != 1) {
       if (ferror(in)) {
 	 fclose(in);
 	 free(*buf);
-	 jp_logf(JP_LOG_WARN, "%s:%d Error reading %s\n", __FILE__, __LINE__, PDB_name);
-	 return -1;
+	 jp_logf(JP_LOG_WARN, _("%s:%d Error reading file: %s\n"), __FILE__, __LINE__, PDB_name);
+	 return EXIT_FAILURE;
       }
    }
    fclose(in);
 
    *buf_size=rec_size;
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -515,34 +494,34 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
    char PC_name[FILENAME_MAX];
 
    if (br==NULL) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    g_snprintf(PC_name, sizeof(PC_name), "%s.pc3", DB_name);
 
    if ((br->rt==DELETED_PALM_REC) || (br->rt==MODIFIED_PALM_REC)) {
-      jp_logf(JP_LOG_INFO, "This record is already deleted.\n"
-		  "It is scheduled to be deleted from the Palm on the next sync.\n");
-      return 0;
+      jp_logf(JP_LOG_INFO, _("This record is already deleted.\n"
+		  "It is scheduled to be deleted from the Palm on the next sync.\n"));
+      return EXIT_SUCCESS;
    }
    switch (br->rt) {
     case NEW_PC_REC:
     case REPLACEMENT_PALM_REC:
       pc_in=jp_open_home_file(PC_name, "r+");
       if (pc_in==NULL) {
-	 jp_logf(JP_LOG_WARN, "Couldn't open PC records file\n");
-	 return -1;
+	 jp_logf(JP_LOG_WARN, _("Unable to open PC records file\n"));
+	 return EXIT_FAILURE;
       }
       while(!feof(pc_in)) {
 	 read_header(pc_in, &header);
 	 if (feof(pc_in)) {
-	    jp_logf(JP_LOG_WARN, "couldn't find record to delete\n");
+	    jp_logf(JP_LOG_WARN, _("Couldn't find record to delete\n"));
 	    fclose(pc_in);
-	    return -1;
+	    return EXIT_FAILURE;
 	 }
 	 if (header.header_version==2) {
 	    /* Keep unique ID intact */
-	    if ((header.unique_id==br->unique_id) && 
+	    if ((header.unique_id==br->unique_id) &&
 		((header.rt==NEW_PC_REC) || (header.rt==REPLACEMENT_PALM_REC))
 		) {
 	       if (fseek(pc_in, -header.header_len, SEEK_CUR)) {
@@ -552,24 +531,24 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
 	       write_header(pc_in, &header);
 	       jp_logf(JP_LOG_DEBUG, "record deleted\n");
 	       fclose(pc_in);
-	       return 0;
+	       return EXIT_SUCCESS;
 	    }
 	 } else {
-	    jp_logf(JP_LOG_WARN, "unknown header version %d\n", header.header_version);
+	    jp_logf(JP_LOG_WARN, _("Unknown header version %d\n"), header.header_version);
 	 }
 	 if (fseek(pc_in, header.rec_len, SEEK_CUR)) {
 	    jp_logf(JP_LOG_WARN, "fseek failed\n");
 	 }
       }
       fclose(pc_in);
-      return -1;
+      return EXIT_FAILURE;
 
     case PALM_REC:
-      jp_logf(JP_LOG_DEBUG, "Deleteing Palm ID %d\n", br->unique_id);
+      jp_logf(JP_LOG_DEBUG, "Deleting Palm ID %d\n", br->unique_id);
       pc_in=jp_open_home_file(PC_name, "a");
       if (pc_in==NULL) {
-	 jp_logf(JP_LOG_WARN, "Couldn't open PC records file\n");
-	 return -1;
+	 jp_logf(JP_LOG_WARN, _("Couldn't open PC records file\n"));
+	 return EXIT_FAILURE;
       }
       header.unique_id=br->unique_id;
       if (flag==MODIFY_FLAG) {
@@ -595,7 +574,7 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
       break;
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -625,8 +604,8 @@ int jp_pc_write(char *DB_name, buf_rec *br)
 
    out = jp_open_home_file(PC_name, "a");
    if (!out) {
-      jp_logf(JP_LOG_WARN, "Error opening %s\n", PC_name);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Error opening file: %s\n"), PC_name);
+      return EXIT_FAILURE;
    }
 
    header.rec_len=br->size;
@@ -639,7 +618,7 @@ int jp_pc_write(char *DB_name, buf_rec *br)
 
    fclose(out);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 static int pc_read_next_rec(FILE *in, buf_rec *br)
@@ -654,7 +633,7 @@ static int pc_read_next_rec(FILE *in, buf_rec *br)
    num = read_header(in, &header);
    if (num < 1) {
       if (ferror(in)) {
-	 jp_logf(JP_LOG_WARN, "Error reading pc file 1\n");
+	 jp_logf(JP_LOG_WARN, _("Error reading PC file 1\n"));
 	 return JPILOT_EOF;
       }
       if (feof(in)) {
@@ -664,13 +643,13 @@ static int pc_read_next_rec(FILE *in, buf_rec *br)
    rec_len = header.rec_len;
    record = malloc(rec_len);
    if (!record) {
-      jp_logf(JP_LOG_WARN, "pc_read_next_rec(): Out of memory\n");
+      jp_logf(JP_LOG_WARN, "pc_read_next_rec(): %s\n", _("Out of memory"));
       return JPILOT_EOF;
    }
    num = fread(record, rec_len, 1, in);
    if (num != 1) {
       if (ferror(in)) {
-	 jp_logf(JP_LOG_WARN, "Error reading pc file 2\n");
+	 jp_logf(JP_LOG_WARN, _("Error reading PC file 2\n"));
 	 free(record);
 	 return JPILOT_EOF;
       }
@@ -681,7 +660,7 @@ static int pc_read_next_rec(FILE *in, buf_rec *br)
    br->buf = record;
    br->size = rec_len;
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int jp_read_DB_files(char *DB_name, GList **records)
@@ -717,20 +696,20 @@ int jp_read_DB_files(char *DB_name, GList **records)
    g_snprintf(PC_name, sizeof(PC_name), "%s.pc3", DB_name);
    in = jp_open_home_file(PDB_name, "r");
    if (!in) {
-      jp_logf(JP_LOG_WARN, "Error opening %s\n", PDB_name);
+      jp_logf(JP_LOG_WARN, _("Error opening file: %s\n"), PDB_name);
       return -1;
    }
    /*Read the database header */
    num = fread(&rdbh, sizeof(RawDBHeader), 1, in);
    if (num != 1) {
       if (ferror(in)) {
-	 jp_logf(JP_LOG_WARN, "Error reading %s\n", PDB_name);
+	 jp_logf(JP_LOG_WARN, _("Error reading file: %s\n"), PDB_name);
 	 fclose(in);
 	 return -1;
       }
       if (feof(in)) {
 	 return JPILOT_EOF;
-      }      
+      }
    }
    raw_header_to_header(&rdbh, &dbh);
 #ifdef JPILOT_DEBUG
@@ -748,12 +727,12 @@ int jp_read_DB_files(char *DB_name, GList **records)
       num = fread(&rh, sizeof(record_header), 1, in);
       if (num != 1) {
 	 if (ferror(in)) {
-	    jp_logf(JP_LOG_WARN, "Error reading %s\n", PDB_name);
+	    jp_logf(JP_LOG_WARN, _("Error reading file: %s\n"), PDB_name);
 	    break;
 	 }
 	 if (feof(in)) {
 	    return JPILOT_EOF;
-	 }      
+	 }
       }
 
       offset = ((rh.Offset[0]*256+rh.Offset[1])*256+rh.Offset[2])*256+rh.Offset[3];
@@ -771,7 +750,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
 #endif
       temp_mem_rh = (mem_rec_header *)malloc(sizeof(mem_rec_header));
       if (!temp_mem_rh) {
-	 jp_logf(JP_LOG_WARN, "jp_read_DB_files(): Out of memory 1\n");
+	 jp_logf(JP_LOG_WARN, "jp_read_DB_files(): %s 1\n", _("Out of memory"));
 	 break;
       }
       temp_mem_rh->next = NULL;
@@ -842,7 +821,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	 }
 	 if ((num < 1)) {
 	    if (ferror(in)) {
-	       jp_logf(JP_LOG_WARN, "Error reading %s 5\n", PDB_name);
+	       jp_logf(JP_LOG_WARN, _("Error reading %s 5\n"), PDB_name);
 	       free(buf);
 	       break;
 	    }
@@ -850,7 +829,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
 
 	 temp_br = malloc(sizeof(buf_rec));
 	 if (!temp_br) {
-	    jp_logf(JP_LOG_WARN, "jp_read_DB_files(): Out of memory 2\n");
+	    jp_logf(JP_LOG_WARN, "jp_read_DB_files(): %s 2\n", _("Out of memory"));
 	    break;
 	 }
 	 temp_br->rt = PALM_REC;
@@ -884,14 +863,15 @@ int jp_read_DB_files(char *DB_name, GList **records)
    pc_in = jp_open_home_file(PC_name, "r");
    if (pc_in==NULL) {
       jp_logf(JP_LOG_DEBUG, "jp_open_home_file failed: %s\n", PC_name);
-      return 0;
+      return -1;
    }
 
    while(!feof(pc_in)) {
       temp_br_used = 0;
       temp_br = malloc(sizeof(buf_rec));
       if (!temp_br) {
-	 jp_logf(JP_LOG_WARN, "jp_read_DB_files(): Out of memory 3\n");
+	 jp_logf(JP_LOG_WARN, "jp_read_DB_files(): %s 3\n", _("Out of memory"));
+	 recs_returned = -1;
 	 break;
       }
       r = pc_read_next_rec(pc_in, temp_br);
@@ -899,10 +879,9 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	 free(temp_br);
 	 break;
       }
-      if ((temp_br->rt!=DELETED_PC_REC)
-	  &&(temp_br->rt!=DELETED_PALM_REC)
-	  &&(temp_br->rt!=MODIFIED_PALM_REC)
-	  &&(temp_br->rt!=DELETED_DELETED_PALM_REC)) {
+      if (temp_br->rt!=DELETED_PALM_REC  &&
+	  temp_br->rt!=MODIFIED_PALM_REC &&
+	  temp_br->rt!=DELETED_DELETED_PALM_REC) {
 
 	 /* g_list_append parses the list to get to the end on every call.
 	  * To speed it up we have to give the last record
