@@ -1,4 +1,7 @@
-/* libplugin.h
+/* $Id: libplugin.h,v 1.20 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * libplugin.h
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #ifndef __LIBPLUGIN_H__
 #define __LIBPLUGIN_H__
 
@@ -108,12 +112,13 @@ typedef struct {
 int get_next_unique_pc_id(unsigned int *next_unique_id);
 
 /* used for jp_delete_record */
-#define CLEAR_FLAG  1
-#define CANCEL_FLAG 2
-#define DELETE_FLAG 3
-#define MODIFY_FLAG 4
-#define NEW_FLAG    5
-#define COPY_FLAG   6
+#define CLEAR_FLAG    1
+#define CANCEL_FLAG   2
+#define DELETE_FLAG   3
+#define MODIFY_FLAG   4
+#define NEW_FLAG      5
+#define COPY_FLAG     6
+#define UNDELETE_FLAG 7
 
 #define CLIST_DEL_RED 0xCCCC
 #define CLIST_DEL_GREEN 0xCCCC
@@ -142,7 +147,7 @@ int get_next_unique_pc_id(unsigned int *next_unique_id);
 #define DIALOG_SAID_3        456
 #define DIALOG_SAID_CANCEL   456
 
-#define JP_LOG_DEBUG  1    /*debugging info for programers, and bug reports */
+#define JP_LOG_DEBUG  1    /*debugging info for programmers, and bug reports */
 #define JP_LOG_INFO   2    /*info, and misc messages */
 #define JP_LOG_WARN   4    /*worse messages */
 #define JP_LOG_FATAL  8    /*even worse messages */
@@ -190,8 +195,6 @@ struct search_result
    struct search_result *next;
 };
 
-/* void free_buf_rec_list(GList **br_list); */
-
 int plugin_get_name(char *name, int len);
 int plugin_get_menu_name(char *name, int len);
 int plugin_get_db_name(char *db_name, int len);
@@ -238,8 +241,8 @@ int jp_edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
 /*************************************
  * convert char code
  *************************************/
-extern void jp_charset_j2p(unsigned char *buf, int max_len);
-extern void jp_charset_p2j(unsigned char *buf, int max_len);
+extern void jp_charset_j2p(char *buf, int max_len);
+extern void jp_charset_p2j(char *buf, int max_len);
 
 /* file must not be open elsewhere when this is called, the first line is 0 */
 int jp_install_remove_line(int deleted_line);
@@ -322,7 +325,7 @@ typedef struct {
 /*   INTTYPE or CHARTYPE, this is the type of value that the pref is when
  *   it is read from, or written to a file.
  *   i.e., For some of my menus I have file type of int and usertype
- *   of char.  I want to use char, except I don't store the char becuase
+ *   of char.  I want to use char, except I don't store the char because
  *   of translations, so I store 3 for the 3rd option.  It also allows
  *   predefined allowed values for strings instead of anything goes. */
 /* long ivalue; */
