@@ -1,6 +1,9 @@
-/* memo_gui.c
+/* $Id: memo_gui.c,v 1.85 2005/10/24 19:15:41 judd Exp $ */
+
+/*******************************************************************************
+ * memo_gui.c
  * A module of J-Pilot http://jpilot.org
- * 
+ *
  * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,13 +18,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
 #include <sys/stat.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <time.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,6 +37,7 @@
 #include "print.h"
 #include "memo.h"
 #include "export.h"
+#include "stock_buttons.h"
 
 
 #define MEMO_MAX_COLUMN_LEN 80
@@ -44,6 +49,10 @@
 #define DISCONNECT_SIGNALS 401
 
 extern GtkTooltips *glob_tooltips;
+#ifdef ENABLE_GTK2
+extern gchar *glob_gtk_clipboard_text;
+extern gchar *glob_x11_clipboard_text;
+#endif
 
 struct MemoAppInfo memo_app_info;
 static int memo_category = CATEGORY_ALL;
@@ -54,7 +63,6 @@ static GtkWidget *memo_text;
 static GObject   *memo_text_buffer;
 #endif
 static GtkWidget *private_checkbox;
-static GtkWidget *memo32_checkbox;
 /*Need one extra for the ALL category */
 static GtkWidget *memo_cat_menu_item1[NUM_MEMO_CAT_ITEMS+1];
 static GtkWidget *memo_cat_menu_item2[NUM_MEMO_CAT_ITEMS];
@@ -66,7 +74,9 @@ static GtkWidget *new_record_button;
 static GtkWidget *apply_record_button;
 static GtkWidget *add_record_button;
 static GtkWidget *delete_record_button;
+static GtkWidget *undelete_record_button;
 static GtkWidget *copy_record_button;
+static GtkWidget *cancel_record_button;
 static int record_changed;
 static int clist_hack;
 
@@ -79,7 +89,7 @@ static void connect_changed_signals(int con_or_dis);
 static int memo_find();
 int memo_get_details(struct Memo *new_memo, unsigned char *attrib);
 static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
-			      MemoList *memo_list, int category, int main);
+			      MemoList **memo_list, int category, int main);
 static void cb_add_new_record(GtkWidget *widget, gpointer data);
 
 static void
@@ -95,16 +105,18 @@ set_new_button_to(int new_state)
       gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
       clist_hack=TRUE;
       /* The line selected on the clist becomes unhighlighted, so we do this */
-      gtk_clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
+      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
       gtk_widget_show(apply_record_button);
+      gtk_widget_show(cancel_record_button);
       gtk_widget_hide(delete_record_button);
       break;
     case NEW_FLAG:
       gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
       clist_hack=TRUE;
       /* The line selected on the clist becomes unhighlighted, so we do this */
-      gtk_clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
+      clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
       gtk_widget_show(add_record_button);
+      gtk_widget_show(cancel_record_button);
       gtk_widget_hide(copy_record_button);
       gtk_widget_hide(delete_record_button);
       break;
@@ -115,23 +127,39 @@ set_new_button_to(int new_state)
       gtk_widget_show(copy_record_button);
       gtk_widget_show(delete_record_button);
       break;
+    case UNDELETE_FLAG:
+      gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
+      clist_hack=FALSE;
+      gtk_widget_show(undelete_record_button);
+      gtk_widget_hide(delete_record_button);
+      break;
+
     default:
       return;
    }
    switch (record_changed) {
     case MODIFY_FLAG:
       gtk_widget_hide(apply_record_button);
+      gtk_widget_hide(cancel_record_button);
       gtk_widget_show(copy_record_button);
       gtk_widget_show(delete_record_button);
       break;
     case NEW_FLAG:
       gtk_widget_hide(add_record_button);
+      gtk_widget_hide(cancel_record_button);
       gtk_widget_show(copy_record_button);
       gtk_widget_show(delete_record_button);
       break;
     case CLEAR_FLAG:
-      gtk_widget_hide(new_record_button);
-      gtk_widget_hide(delete_record_button);
+      if (new_state != UNDELETE_FLAG)
+      {
+         gtk_widget_hide(new_record_button);
+         gtk_widget_hide(delete_record_button);
+      }
+      break;
+    case UNDELETE_FLAG:
+      gtk_widget_hide(undelete_record_button);
+      gtk_widget_show(delete_record_button);
       break;
    }
    record_changed=new_state;
@@ -144,12 +172,18 @@ cb_record_changed(GtkWidget *widget,
    jp_logf(JP_LOG_DEBUG, "cb_record_changed\n");
    if (record_changed==CLEAR_FLAG) {
       connect_changed_signals(DISCONNECT_SIGNALS);
-      if (((GtkCList *)clist)->rows > 0) {
+      if (GTK_CLIST(clist)->rows > 0) {
 	 set_new_button_to(MODIFY_FLAG);
       } else {
 	 set_new_button_to(NEW_FLAG);
       }
    }
+   else if (record_changed==UNDELETE_FLAG)
+   {
+      jp_logf(JP_LOG_INFO|JP_LOG_GUI,
+              _("This record is deleted.\n"
+	        "Undelete it or copy it to make changes.\n"));
+   }
 }
 
 static void connect_changed_signals(int con_or_dis)
@@ -185,7 +219,7 @@ static void connect_changed_signals(int con_or_dis)
       for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
 	 if (memo_cat_menu_item2[i]) {
 	    gtk_signal_disconnect_by_func(GTK_OBJECT(memo_cat_menu_item2[i]),
-					  GTK_SIGNAL_FUNC(cb_record_changed), NULL);     
+					  GTK_SIGNAL_FUNC(cb_record_changed), NULL);
 	 }
       }
 #ifdef ENABLE_GTK2
@@ -213,7 +247,7 @@ int memo_print()
    if (this_many==1) {
       mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
       if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
-	 return -1;
+	 return EXIT_FAILURE;
       }
       memcpy(&(memo_list1.mmemo), mmemo, sizeof(MyMemo));
       memo_list1.next=NULL;
@@ -232,7 +266,7 @@ int memo_print()
       free_MemoList(&memo_list);
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -264,8 +298,8 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 
    in=fopen(file_path, "r");
    if (!in) {
-      jp_logf(JP_LOG_WARN, _("Could not open file %s\n"), file_path);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), file_path);
+      return EXIT_FAILURE;
    }
 
    /* TEXT */
@@ -281,13 +315,20 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 	 if (text_len+len>65534) {
 	    len=65534-text_len;
 	    line[len]='\0';
-	    jp_logf(JP_LOG_WARN, "Memo text > 65535, truncating\n");
+	    jp_logf(JP_LOG_WARN, _("Memo text > 65535, truncating\n"));
 	    strcat(text, line);
 	    break;
 	 }
 	 text_len+=len;
 	 strcat(text, line);
       }
+
+#ifdef ENABLE_GTK2
+      /* convert to valid UTF-8 and recalculate the length */
+      jp_charset_p2j(text, sizeof(text));
+      text_len = strlen(text);
+#endif
+
 #ifdef JPILOT_DEBUG
       printf("text=[%s]\n", text);
       printf("text_len=%d\n", text_len);
@@ -304,7 +345,7 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 			    &new_cat_num);
       if ((ret==DIALOG_SAID_IMPORT_ALL) || (ret==DIALOG_SAID_IMPORT_YES)) {
 	 pc_memo_write(&new_memo, NEW_PC_REC, attrib, NULL);
-	 jp_logf(JP_LOG_WARN, "Imported Memo %s\n", file_path);
+	 jp_logf(JP_LOG_WARN, _("Imported Memo %s\n"), file_path);
       }
    }
 
@@ -377,7 +418,7 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
       if (dat_check_if_dat_file(in)!=DAT_MEMO_FILE) {
 	 jp_logf(JP_LOG_WARN, _("File doesn't appear to be memopad.dat format\n"));
 	 fclose(in);
-	 return 1;
+	 return EXIT_FAILURE;
       }
       memolist=NULL;
       dat_get_memos(in, &memolist, &cai);
@@ -442,7 +483,7 @@ int memo_import_callback(GtkWidget *parent_window, const char *file_path, int ty
 
    memo_refresh();
    fclose(in);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int memo_import(GtkWidget *window)
@@ -461,7 +502,7 @@ int memo_import(GtkWidget *window)
    };
 
    import_gui(window, pane, type_desc, type_int, memo_import_callback);
-   return 0;
+   return EXIT_SUCCESS;
 }
 /*
  * End Import Code
@@ -482,8 +523,8 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    const char *short_date;
    time_t ltime;
    struct tm *now;
-   char *button_text[]={gettext_noop("OK")};
-   char *button_overwrite_text[]={gettext_noop("Yes"), gettext_noop("No")};
+   char *button_text[]={N_("OK")};
+   char *button_overwrite_text[]={N_("Yes"), N_("No")};
    char text[1024];
    char str1[256], str2[256];
    char pref_time[40];
@@ -496,7 +537,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	 g_snprintf(text, sizeof(text), _("%s is a directory"), filename);
 	 dialog_generic(GTK_WINDOW(export_window),
 			0, 0, _("Error Opening File"),
-			"Directory", text, 1, button_text);
+			_("Directory"), text, 1, button_text);
 	 return;
       }
       g_snprintf(text,sizeof(text), _("Do you want to overwrite file %s?"), filename);
@@ -510,10 +551,10 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 
    out = fopen(filename, "w");
    if (!out) {
-      g_snprintf(text,sizeof(text), "Error Opening File: %s", filename);
+      g_snprintf(text,sizeof(text), _("Error opening file: %s"), filename);
       dialog_generic(GTK_WINDOW(export_window),
 		     0, 0, _("Error Opening File"),
-		     "Filename", text, 1, button_text);
+		     _("Filename"), text, 1, button_text);
       return;
    }
 
@@ -521,10 +562,10 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    for (num=0, temp_list=list; temp_list; temp_list = temp_list->next, num++);
 
    for (i=0, temp_list=list; temp_list; temp_list = temp_list->next, i++) {
-      mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), (int) temp_list->data);
+      mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), GPOINTER_TO_INT(temp_list->data));
       if (!mmemo) {
 	 continue;
-	 jp_logf(JP_LOG_WARN, "Can't export memo %d\n", (long) temp_list->data + 1);
+	 jp_logf(JP_LOG_WARN, _("Can't export memo %d\n"), (long) temp_list->data + 1);
       }
       switch (type) {
        case EXPORT_TYPE_CSV:
@@ -539,7 +580,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	 csv_text=malloc(len);
 	 if (!csv_text) {
 	    continue;
-	    jp_logf(JP_LOG_WARN, "Can't export memo %d\n", (long) temp_list->data + 1);
+	    jp_logf(JP_LOG_WARN, _("Can't export memo %d\n"), (long) temp_list->data + 1);
 	 }
 	 str_to_csv_str(csv_text, memo_app_info.category.name[mmemo->attrib & 0x0F]);
 	 fprintf(out, "\"%s\",", csv_text);
@@ -568,7 +609,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	 fprintf(out, "\n----- End of Memo -----\n\n");
 	 break;
        default:
-	 jp_logf(JP_LOG_WARN, "Unknown export type\n");
+	 jp_logf(JP_LOG_WARN, _("Unknown export type\n"));
       }
    }
 
@@ -580,7 +621,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 
 static void cb_memo_update_clist(GtkWidget *clist, int category)
 {
-   memo_update_clist(clist, NULL, export_memo_list, category, FALSE);
+   memo_update_clist(clist, NULL, &export_memo_list, category, FALSE);
 }
 
 
@@ -617,7 +658,7 @@ int memo_export(GtkWidget *window)
 	      cb_memo_export_ok
 	      );
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -633,7 +674,7 @@ static int find_sorted_cat(int cat)
  	 return i;
       }
    }
-   return -1;
+   return EXIT_FAILURE;
 }
 
 void cb_delete_memo(GtkWidget *widget,
@@ -652,8 +693,7 @@ void cb_delete_memo(GtkWidget *widget,
    get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set != CHAR_SET_LATIN1) {
       if (mmemo->memo.text)
-	charset_j2p((unsigned char *)mmemo->memo.text,
-		    strlen(mmemo->memo.text)+1, char_set);
+	charset_j2p(mmemo->memo.text, strlen(mmemo->memo.text)+1, char_set);
    }
 
    /* Do masking like Palm OS 3.5 */
@@ -676,9 +716,57 @@ void cb_delete_memo(GtkWidget *widget,
       }
    }
 
+   if (flag == DELETE_FLAG) {
+      memo_clist_redraw();
+   }
+}
+
+void cb_undelete_memo(GtkWidget *widget,
+		    gpointer   data)
+{
+   MyMemo *mmemo;
+   int flag;
+   int show_priv;
+
+   mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
+   if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
+      return;
+   }
+
+   /* Do masking like Palm OS 3.5 */
+   show_priv = show_privates(GET_PRIVATES);
+   if ((show_priv != SHOW_PRIVATES) &&
+       (mmemo->attrib & dlpRecAttrSecret)) {
+      return;
+   }
+   /* End Masking */
+
+   jp_logf(JP_LOG_DEBUG, "mmemo->unique_id = %d\n",mmemo->unique_id);
+   jp_logf(JP_LOG_DEBUG, "mmemo->rt = %d\n",mmemo->rt);
+
+   flag = GPOINTER_TO_INT(data);
+   if (flag==UNDELETE_FLAG) {
+      if (mmemo->rt == DELETED_PALM_REC ||
+          mmemo->rt == DELETED_PC_REC)
+      {
+	 undelete_pc_record(MEMO, mmemo, flag);
+      }
+      /* Possible later addition of undelete for modified records
+      else if (mmemo->rt == MODIFIED_PALM_REC)
+      {
+	 cb_add_new_record(widget, GINT_TO_POINTER(COPY_FLAG));
+      }
+      */
+   }
+
    memo_clist_redraw();
 }
 
+static void cb_cancel(GtkWidget *widget, gpointer data)
+{
+   set_new_button_to(CLEAR_FLAG);
+   memo_refresh();
+}
 
 static void cb_category(GtkWidget *item, int selection)
 {
@@ -691,9 +779,10 @@ static void cb_category(GtkWidget *item, int selection)
       }
 
       memo_category = selection;
+      clist_row_selected = 0;
       jp_logf(JP_LOG_DEBUG, "cb_category() cat=%d\n", memo_category);
       memo_clear_details();
-      memo_update_clist(clist, category_menu1, glob_memo_list, memo_category, TRUE);
+      memo_update_clist(clist, category_menu1, &glob_memo_list, memo_category, TRUE);
       jp_logf(JP_LOG_DEBUG, "Leaving cb_category()\n");
    }
 }
@@ -729,7 +818,7 @@ static int memo_clear_details()
    }
    sorted_position = find_sorted_cat(new_cat);
    if (sorted_position<0) {
-      jp_logf(JP_LOG_WARN, "Category is not legal\n");
+      jp_logf(JP_LOG_WARN, _("Category is not legal\n"));
    } else {
       gtk_check_menu_item_set_active
 	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[sorted_position]), TRUE);
@@ -738,7 +827,7 @@ static int memo_clear_details()
 
    set_new_button_to(CLEAR_FLAG);
    jp_logf(JP_LOG_DEBUG, "Leaving memo_clear_details()\n");
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int memo_get_details(struct Memo *new_memo, unsigned char *attrib)
@@ -771,14 +860,7 @@ int memo_get_details(struct Memo *new_memo, unsigned char *attrib)
    if (GTK_TOGGLE_BUTTON(private_checkbox)->active) {
       *attrib |= dlpRecAttrSecret;
    }
-   return 0;
-}
-
-static void cb_use_memo32(GtkWidget *widget,
-			  gpointer   data)
-{
-   set_pref(PREF_MEMO32_MODE, GTK_TOGGLE_BUTTON(memo32_checkbox)->active, NULL, TRUE);
-   cb_app_button(NULL, GINT_TO_POINTER(REDRAW));
+   return EXIT_SUCCESS;
 }
 
 static void cb_add_new_record(GtkWidget *widget, gpointer data)
@@ -796,8 +878,7 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
    unique_id=0;
 
    /* Do masking like Palm OS 3.5 */
-   if ((GPOINTER_TO_INT(data)==COPY_FLAG) || 
-       (GPOINTER_TO_INT(data)==MODIFY_FLAG)) {
+   if ((flag==COPY_FLAG) || (flag==MODIFY_FLAG)) {
       show_priv = show_privates(GET_PRIVATES);
       mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
       if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
@@ -826,8 +907,10 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
       if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
 	 return;
       }
-      if ((mmemo->rt==DELETED_PALM_REC) || (mmemo->rt==MODIFIED_PALM_REC)) {
-	 jp_logf(JP_LOG_INFO, "You can't modify a record that is deleted\n");
+      if ((mmemo->rt==DELETED_PALM_REC) ||
+	  (mmemo->rt==DELETED_PC_REC)   ||
+          (mmemo->rt==MODIFIED_PALM_REC)) {
+	 jp_logf(JP_LOG_INFO, _("You can't modify a record that is deleted\n"));
 	 return;
       }
    }
@@ -880,8 +963,12 @@ static void cb_edit_cats(GtkWidget *widget, gpointer data)
    char pdb_name[FILENAME_MAX];
    char full_name[FILENAME_MAX];
    unsigned char buffer[65536];
-   int num, r;
+   int num;
+#ifdef PILOT_LINK_0_12
+   size_t size;
+#else
    int size;
+#endif
    void *buf;
    long ivalue;
    struct pi_file *pf;
@@ -902,11 +989,11 @@ static void cb_edit_cats(GtkWidget *widget, gpointer data)
    memset(&ai, 0, sizeof(ai));
 
    pf = pi_file_open(full_name);
-   r = pi_file_get_app_info(pf, &buf, &size);
+   pi_file_get_app_info(pf, &buf, &size);
 
    num = unpack_MemoAppInfo(&ai, buf, size);
    if (num <= 0) {
-      jp_logf(JP_LOG_WARN, _("Error reading %s\n"), pdb_name);
+      jp_logf(JP_LOG_WARN, _("Error reading file: %s\n"), pdb_name);
       return;
    }
 
@@ -939,16 +1026,13 @@ static void cb_clist_selection(GtkWidget      *clist,
    /* HACK, see clist hack explanation in memo_gui.c */
    if (clist_hack) {
       keep=record_changed;
-      gtk_clist_select_row(GTK_CLIST(clist), clist_row_selected, column);
+      clist_select_row(GTK_CLIST(clist), clist_row_selected, column);
       b=dialog_save_changed_record(pane, record_changed);
       if (b==DIALOG_SAID_1) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
       set_new_button_to(CLEAR_FLAG);
-      /* This doesn't cause an event to occur, it does highlight
-       * the line, so we do the next call also */
-      gtk_clist_select_row(GTK_CLIST(clist), row, column);
-      cb_clist_selection(clist, row, column, GINT_TO_POINTER(1), NULL);
+      clist_select_row(GTK_CLIST(clist), row, column);
       return;
    }
 
@@ -959,7 +1043,18 @@ static void cb_clist_selection(GtkWidget      *clist,
       return;
    }
 
-   set_new_button_to(CLEAR_FLAG);
+   if (mmemo->rt == DELETED_PALM_REC ||
+      (mmemo->rt == DELETED_PC_REC))
+      /* Possible later addition of undelete code for modified deleted records
+         || mmemo->rt == MODIFIED_PALM_REC
+      */
+   {
+      set_new_button_to(UNDELETE_FLAG);
+   }
+   else
+   {
+      set_new_button_to(CLEAR_FLAG);
+   }
 
    connect_changed_signals(DISCONNECT_SIGNALS);
 
@@ -981,7 +1076,7 @@ static void cb_clist_selection(GtkWidget      *clist,
    count--;
 
    if (sorted_position<0) {
-      jp_logf(JP_LOG_WARN, "Category is not legal\n");
+      jp_logf(JP_LOG_WARN, _("Category is not legal\n"));
    } else {
       gtk_check_menu_item_set_active
 	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[sorted_position]), TRUE);
@@ -1008,11 +1103,38 @@ static void cb_clist_selection(GtkWidget      *clist,
    connect_changed_signals(CONNECT_SIGNALS);
 }
 
-static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
-			      MemoList *memo_list, int category, int main)
+static gboolean cb_key_pressed_left_side(GtkWidget   *widget, 
+                                         GdkEventKey *event,
+                                         gpointer     next_widget)
 {
-   int num_entries, entries_shown, i;
-   int row_count;
+   if (event->keyval == GDK_Return) {
+      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+      gtk_widget_grab_focus(GTK_WIDGET(next_widget));
+      return TRUE;
+   }
+
+   return FALSE;
+}
+
+static gboolean cb_key_pressed_right_side(GtkWidget   *widget, 
+                                          GdkEventKey *event,
+                                          gpointer     next_widget)
+{
+   if ((event->keyval == GDK_Return) && (event->state & GDK_SHIFT_MASK)) {
+      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+      /* Call clist_selection to handle any cleanup such as a modified record */
+      cb_clist_selection(clist, clist_row_selected, 0, GINT_TO_POINTER(1), NULL);
+      gtk_widget_grab_focus(GTK_WIDGET(next_widget));
+      return TRUE;
+   }
+
+   return FALSE;
+}
+
+static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
+			      MemoList **memo_list, int category, int main)
+{
+   int num_entries, entries_shown;
    size_t copy_max_length;
    char *last;
    gchar *empty_line[] = { "" };
@@ -1024,61 +1146,37 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 
    jp_logf(JP_LOG_DEBUG, "memo_update_clist()\n");
 
-   row_count=(GTK_CLIST(clist))->rows;
 
-   /* Need to remove pointers to data we are about to delete */
-   for (i=0; i<row_count; i++) {
-      gtk_clist_set_row_data(GTK_CLIST(clist), i, NULL);
-   }
-
-   free_MemoList(&memo_list);
+   free_MemoList(memo_list);
 
    /* Need to get all records including private ones for the tooltips calculation */
-   num_entries = get_memos2(&memo_list, SORT_ASCENDING, 2, 2, 1, CATEGORY_ALL);
+   num_entries = get_memos2(memo_list, SORT_ASCENDING, 2, 2, 1, CATEGORY_ALL);
 
-   /* Start by clearing existing entry */
-   memo_clear_details();
-
-   if (memo_list==NULL) {
-      /* Remove row 1 if the last item has just been deleted */
-      if (row_count == 1) {
-         gtk_clist_remove(GTK_CLIST(clist),0);
-      }
-      if (tooltip_widget) {
-	 gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, _("0 records"), NULL);
-      }
-      return;
+   /* Start by clearing existing entry if in main window */
+   if (main) {
+      memo_clear_details();
    }
 
+   /* Freeze clist to prevent flicker during updating */
    gtk_clist_freeze(GTK_CLIST(clist));
+   if (main)
+	   gtk_signal_disconnect_by_func(GTK_OBJECT(clist),
+				 GTK_SIGNAL_FUNC(cb_clist_selection), NULL);
+   gtk_clist_clear(GTK_CLIST(clist));
 
    show_priv = show_privates(GET_PRIVATES);
 
-   for (temp_memo = memo_list; temp_memo; temp_memo=temp_memo->next) {
-      if ( ((temp_memo->mmemo.attrib & 0x0F) != category) &&
-	  category != CATEGORY_ALL) {
-	 continue;
-      }
-
-      if ((show_priv != SHOW_PRIVATES) && 
-	  (temp_memo->mmemo.attrib & dlpRecAttrSecret)) {
-	 continue;
-      }
-   }
-
    entries_shown=0;
-   for (temp_memo = memo_list; temp_memo; temp_memo=temp_memo->next) {
+   for (temp_memo = *memo_list; temp_memo; temp_memo=temp_memo->next) {
       if ( ((temp_memo->mmemo.attrib & 0x0F) != category) &&
 	  category != CATEGORY_ALL) {
 	 continue;
       }
 
       /* Do masking like Palm OS 3.5 */
-      if ((show_priv == MASK_PRIVATES) && 
+      if ((show_priv == MASK_PRIVATES) &&
 	  (temp_memo->mmemo.attrib & dlpRecAttrSecret)) {
-	 if (entries_shown+1>row_count) {
-	    gtk_clist_append(GTK_CLIST(clist), empty_line);
-	 }
+	 gtk_clist_append(GTK_CLIST(clist), empty_line);
 	 gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0, "----------------------------------------");
 	 clear_mymemo(&temp_memo->mmemo);
 	 gtk_clist_set_row_data(GTK_CLIST(clist), entries_shown, &(temp_memo->mmemo));
@@ -1087,14 +1185,16 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 	 continue;
       }
       /* End Masking */
-      if ((show_priv != SHOW_PRIVATES) && 
+
+      /* Hide the private records if need be */
+      if ((show_priv != SHOW_PRIVATES) &&
 	  (temp_memo->mmemo.attrib & dlpRecAttrSecret)) {
 	 continue;
       }
 
-      if (entries_shown+1>row_count) {
-	 gtk_clist_append(GTK_CLIST(clist), empty_line);
-      }
+      /* Add entry to clist */
+      gtk_clist_append(GTK_CLIST(clist), empty_line);
+
       sprintf(str, "%d. ", entries_shown + 1);
 
       len1 = strlen(str);
@@ -1113,6 +1213,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
       gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0, str2);
       gtk_clist_set_row_data(GTK_CLIST(clist), entries_shown, &(temp_memo->mmemo));
 
+      /* Highlight row background depending on status */
       switch (temp_memo->mmemo.rt) {
        case NEW_PC_REC:
        case REPLACEMENT_PALM_REC:
@@ -1120,6 +1221,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 			  CLIST_NEW_RED, CLIST_NEW_GREEN, CLIST_NEW_BLUE);
 	 break;
        case DELETED_PALM_REC:
+       case DELETED_PC_REC:
 	 set_bg_rgb_clist_row(clist, entries_shown,
 			  CLIST_DEL_RED, CLIST_DEL_GREEN, CLIST_DEL_BLUE);
 	 break;
@@ -1129,7 +1231,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 	 break;
        default:
 	 if (temp_memo->mmemo.attrib & dlpRecAttrSecret) {
-	    set_bg_rgb_clist_row(clist, entries_shown, 
+	    set_bg_rgb_clist_row(clist, entries_shown,
 			     CLIST_PRIVATE_RED, CLIST_PRIVATE_GREEN, CLIST_PRIVATE_BLUE);
 	 } else {
 	    gtk_clist_set_row_style(GTK_CLIST(clist), entries_shown, NULL);
@@ -1140,68 +1242,72 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 
    jp_logf(JP_LOG_DEBUG, "entries_shown=%d\n", entries_shown);
 
-   for (i=row_count-1; i>=entries_shown; i--) {
-      gtk_clist_set_row_data(GTK_CLIST(clist), i, NULL);
-      gtk_clist_remove(GTK_CLIST(clist), i);
-   }
+   gtk_signal_connect(GTK_OBJECT(clist), "select_row",
+		      GTK_SIGNAL_FUNC(cb_clist_selection), NULL);
 
-   /*If there is an item in the list, select the first one */
+   /* If there are items in the list, highlight the selected row */
    if ((main) && (entries_shown>0)) {
-      gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
-      cb_clist_selection(clist, 0, 0, (GdkEventButton *)455, NULL);
+      /* Select the existing requested row, or row 0 if that is impossible */
+      if (clist_row_selected <= entries_shown) {
+	 clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
+	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), clist_row_selected)) {
+	    gtk_clist_moveto(GTK_CLIST(clist), clist_row_selected, 0, 0.5, 0.0);
+	 }
+      }
+      else
+      {
+	 clist_select_row(GTK_CLIST(clist), 0, 0);
+      }
    }
 
+   /* Unfreeze clist after all changes */
    gtk_clist_thaw(GTK_CLIST(clist));
 
    if (tooltip_widget) {
-      sprintf(str, _("%d of %d records"), entries_shown, num_entries);
-      gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, str, NULL);
+      if (memo_list==NULL) {
+	    gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, _("0 records"), NULL);
+      }
+      else {
+	 sprintf(str, _("%d of %d records"), entries_shown, num_entries);
+	 gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, str, NULL);
+      }
    }
 
    if (main) {
       connect_changed_signals(CONNECT_SIGNALS);
    }
 
+   /* return focus to clist after any big operation which requires a redraw */
+   gtk_widget_grab_focus(GTK_WIDGET(clist));
+
    jp_logf(JP_LOG_DEBUG, "Leaving memo_update_clist()\n");
 }
 
 static int memo_find()
 {
-   int r, found_at, total_count;
+   int r, found_at;
 
    if (glob_find_id) {
       r = clist_find_id(clist,
 			glob_find_id,
-			&found_at,
-			&total_count);
+			&found_at);
       if (r) {
-	 if (total_count == 0) {
-	    total_count = 1;
-	 }
-	 gtk_clist_select_row(GTK_CLIST(clist), found_at, 0);
-	 cb_clist_selection(clist, found_at, 0, (gpointer)455, NULL);
+	 clist_select_row(GTK_CLIST(clist), found_at, 0);
 	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), found_at)) {
 	    gtk_clist_moveto(GTK_CLIST(clist), found_at, 0, 0.5, 0.0);
 	 }
       }
       glob_find_id = 0;
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
-/* This redraws the clist and goes back to the same line number */
+/* This redraws the clist */
 int memo_clist_redraw()
 {
-   int line_num;
+   memo_update_clist(clist, category_menu1, &glob_memo_list, memo_category, TRUE);
 
-   line_num = clist_row_selected;
-
-   memo_update_clist(clist, category_menu1, glob_memo_list, memo_category, TRUE);
-
-   gtk_clist_select_row(GTK_CLIST(clist), line_num, 0);
-   cb_clist_selection(clist, line_num, 0, (gpointer)455, NULL);
-
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int memo_cycle_cat()
@@ -1230,7 +1336,9 @@ int memo_cycle_cat()
 	 break;
       }
    }
-   return 0;
+   clist_row_selected = 0;
+
+   return EXIT_SUCCESS;
 }
 
 int memo_refresh()
@@ -1245,16 +1353,16 @@ int memo_refresh()
    } else {
       index=find_sorted_cat(memo_category)+1;
    }
-   memo_update_clist(clist, category_menu1, glob_memo_list, memo_category, TRUE);
+   memo_update_clist(clist, category_menu1, &glob_memo_list, memo_category, TRUE);
    if (index<0) {
-      jp_logf(JP_LOG_WARN, "Category not legal\n");
-   } else {	 
+      jp_logf(JP_LOG_WARN, _("Category is not legal\n"));
+   } else {
       gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu1), index);
       gtk_check_menu_item_set_active
 	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item1[index]), TRUE);
    }
    memo_find();
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int memo_gui_cleanup()
@@ -1267,6 +1375,10 @@ int memo_gui_cleanup()
    }
    free_MemoList(&glob_memo_list);
    connect_changed_signals(DISCONNECT_SIGNALS);
+#ifdef ENABLE_GTK2
+   glob_gtk_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
+   glob_x11_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY));
+#endif
 #ifdef ENABLE_GTK2
    set_pref(PREF_MEMO_PANE, gtk_paned_get_position(GTK_PANED(pane)), NULL, TRUE);
 #else
@@ -1274,7 +1386,7 @@ int memo_gui_cleanup()
 #endif
    set_pref(PREF_LAST_MEMO_CATEGORY, memo_category, NULL, TRUE);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -1293,7 +1405,9 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    GtkWidget *vscrollbar;
 #endif
    long ivalue;
-   const char *svalue;
+   GtkAccelGroup *accel_group;
+   long char_set;
+   char *cat_name;
 
    clist_row_selected=0;
    record_changed=CLEAR_FLAG;
@@ -1305,8 +1419,11 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
    get_memo_app_info(&memo_app_info);
 
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
    for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
-      sort_l[i].Pcat = memo_app_info.category.name[i];
+      cat_name = charset_p2newj(memo_app_info.category.name[i], 31, char_set);
+      strcpy(sort_l[i].Pcat, cat_name);
+      free(cat_name);
       sort_l[i].cat_num = i;
    }
    qsort(sort_l, NUM_MEMO_CAT_ITEMS, sizeof(struct sorted_cats), cat_compare);
@@ -1324,9 +1441,31 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
       memo_category=CATEGORY_ALL;
    }
 
+#ifdef ENABLE_GTK2
+   /* Preserve clipboard buffer across applications */
+   if (glob_gtk_clipboard_text != NULL)
+   {
+      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD),
+                             glob_gtk_clipboard_text, -1);
+      g_free(glob_gtk_clipboard_text);
+      glob_gtk_clipboard_text = NULL;
+   }
+   if (glob_x11_clipboard_text != NULL)
+   {
+      gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
+                             glob_x11_clipboard_text, -1);
+      g_free(glob_x11_clipboard_text);
+      glob_x11_clipboard_text = NULL;
+   }
+#endif
+
+   accel_group = gtk_accel_group_new();
+   gtk_window_add_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(vbox)),
+      accel_group);
+
    pane = gtk_hpaned_new();
-   get_pref(PREF_MEMO_PANE, &ivalue, &svalue);
-   gtk_paned_set_position(GTK_PANED(pane), ivalue + 2);
+   get_pref(PREF_MEMO_PANE, &ivalue, NULL);
+   gtk_paned_set_position(GTK_PANED(pane), ivalue + PANE_CREEP);
 
    gtk_box_pack_start(GTK_BOX(hbox), pane, TRUE, TRUE, 5);
 
@@ -1371,16 +1510,6 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 		      GTK_SIGNAL_FUNC(cb_edit_cats), NULL);
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
 
-
-   /* Memo32 check box */
-   memo32_checkbox = gtk_check_button_new_with_label(_("Use Memo32 (pedit32)"));
-   gtk_box_pack_start(GTK_BOX(vbox1), memo32_checkbox, FALSE, FALSE, 0);
-   get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memo32_checkbox), ivalue);
-   gtk_signal_connect(GTK_OBJECT(memo32_checkbox), "clicked",
-		      GTK_SIGNAL_FUNC(cb_use_memo32), NULL);
-
-
    /* Put the memo list window up */
    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    /*gtk_widget_set_usize(GTK_WIDGET(scrolled_window), 330, 100); */
@@ -1410,9 +1539,6 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
    gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(clist));
 
-   /* The focus doesn't do any good on the application button */
-   gtk_widget_grab_focus(GTK_WIDGET(clist));
-
    /*
     * The right hand part of the main window follows:
     */
@@ -1421,39 +1547,55 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
 
    /* Add record modification buttons on right side */
-   delete_record_button = gtk_button_new_with_label(_("Delete"));
+
+   /* Create Cancel button */
+   CREATE_BUTTON(cancel_record_button, _("Cancel"), CANCEL, _("Cancel the modifications"), GDK_Escape, 0, "ESC")
+   gtk_signal_connect(GTK_OBJECT(cancel_record_button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_cancel), NULL);
+
+   /* Delete Button */
+   CREATE_BUTTON(delete_record_button, _("Delete"), DELETE, _("Delete the selected record"), GDK_d, GDK_CONTROL_MASK, "Ctrl+D")
    gtk_signal_connect(GTK_OBJECT(delete_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_delete_memo),
 		      GINT_TO_POINTER(DELETE_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), delete_record_button, TRUE, TRUE, 0);
 
-   copy_record_button = gtk_button_new_with_label(_("Copy"));
+   /* Undelete Button */
+   CREATE_BUTTON(undelete_record_button, _("Undelete"), UNDELETE, _("Undelete the selected record"), 0, 0, "")
+   gtk_signal_connect(GTK_OBJECT(undelete_record_button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_undelete_memo),
+		      GINT_TO_POINTER(UNDELETE_FLAG));
+
+   /* Create "Copy" button */
+   CREATE_BUTTON(copy_record_button, _("Copy"), COPY, _("Copy the selected record"), GDK_o, GDK_CONTROL_MASK, "Ctrl+O")
    gtk_signal_connect(GTK_OBJECT(copy_record_button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_add_new_record), 
+		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(COPY_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), copy_record_button, TRUE, TRUE, 0);
 
-   new_record_button = gtk_button_new_with_label(_("New Record"));
+   /* Create "New" button */
+   CREATE_BUTTON(new_record_button, _("New Record"), NEW, _("Add a new record"), GDK_n, GDK_CONTROL_MASK, "Ctrl+N")
    gtk_signal_connect(GTK_OBJECT(new_record_button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_add_new_record), 
+		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(CLEAR_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), new_record_button, TRUE, TRUE, 0);
 
-   add_record_button = gtk_button_new_with_label(_("Add Record"));
+   /* Create "Add Record" button */
+   CREATE_BUTTON(add_record_button, _("Add Record"), ADD, _("Add the new record"), GDK_Return, GDK_CONTROL_MASK, "Ctrl+Enter")
    gtk_signal_connect(GTK_OBJECT(add_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(NEW_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), add_record_button, TRUE, TRUE, 0);
+#ifndef ENABLE_STOCK_BUTTONS
    gtk_widget_set_name(GTK_WIDGET(GTK_LABEL(GTK_BIN(add_record_button)->child)),
 		       "label_high");
+#endif
 
-   apply_record_button = gtk_button_new_with_label(_("Apply Changes"));
+   /* Create "apply changes" button */
+   CREATE_BUTTON(apply_record_button, _("Apply Changes"), APPLY, _("Commit the modifications"), GDK_Return, GDK_CONTROL_MASK, "Ctrl+Enter")
    gtk_signal_connect(GTK_OBJECT(apply_record_button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_add_new_record), 
+		      GTK_SIGNAL_FUNC(cb_add_new_record),
 		      GINT_TO_POINTER(MODIFY_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), apply_record_button, TRUE, TRUE, 0);
+#ifndef ENABLE_STOCK_BUTTONS
    gtk_widget_set_name(GTK_WIDGET(GTK_LABEL(GTK_BIN(apply_record_button)->child)),
 		       "label_high");
+#endif
 
 
    /*Separator */
@@ -1503,13 +1645,25 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(hbox_temp), vscrollbar, FALSE, FALSE, 0);
 #endif
 
+   /* Capture the Enter & Shift-Enter key combinations to move back and 
+    * forth between the left- and right-hand sides of the display. */
+   gtk_signal_connect(GTK_OBJECT(clist), "key_press_event",
+		      GTK_SIGNAL_FUNC(cb_key_pressed_left_side), memo_text);
+
+   gtk_signal_connect(GTK_OBJECT(memo_text), "key_press_event",
+		      GTK_SIGNAL_FUNC(cb_key_pressed_right_side), clist);
+
+   /**********************************************************************/
+
    gtk_widget_show_all(vbox);
    gtk_widget_show_all(hbox);
 
    gtk_widget_hide(add_record_button);
    gtk_widget_hide(apply_record_button);
+   gtk_widget_hide(undelete_record_button);
+   gtk_widget_hide(cancel_record_button);
 
    memo_refresh();
 
-   return 0;
+   return EXIT_SUCCESS;
 }
