@@ -1,4 +1,7 @@
-/* monthview_gui.c
+/* $Id: monthview_gui.c,v 1.32 2005/01/29 12:09:01 rousseau Exp $ */
+
+/*******************************************************************************
+ * monthview_gui.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -75,8 +78,7 @@ static gboolean cb_destroy(GtkWidget *widget)
    return FALSE;
 }
 
-static void
-cb_quit(GtkWidget *widget, gpointer data)
+static void cb_quit(GtkWidget *widget, gpointer data)
 {
    int w, h;
 
@@ -87,9 +89,7 @@ cb_quit(GtkWidget *widget, gpointer data)
    gtk_widget_destroy(data);
 }
 
-static void
-cb_month_move(GtkWidget *widget,
-	      gpointer   data)
+static void cb_month_move(GtkWidget *widget, gpointer data)
 {
    if (GPOINTER_TO_INT(data)==-1) {
       glob_month_date.tm_mday=15;
@@ -128,8 +128,7 @@ static void cb_month_print(GtkWidget *widget, gpointer data)
 
 /*----------------------------------------------------------------------*/
 
-static void
-cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
+static void cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
    static int prev_day=-1;
    GString *gstr;
@@ -158,6 +157,17 @@ cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
 #endif
 }
 
+/* Called when a day is clicked on the month view */
+static void cb_enter_selected_day(GtkWidget *widget, 
+                                  GdkEvent  *event, 
+				  gpointer   data)
+{
+   int day = GPOINTER_TO_INT(data) + 1 - glob_offset;
+
+   /* Redisplay the day view based on the date the user clicked on */
+   datebook_gui_setdate(glob_month_date.tm_year, glob_month_date.tm_mon, day);
+}
+
 /*
  * Hide, or show month boxes (days) according to the month.
  * Also, set a global offset for indexing day 1.
@@ -167,11 +177,15 @@ void hide_show_month_boxes()
 {
    int n;
    int dow, ndim;
+   int now_today;
    long fdow;
-   char str[8];
+   char str[40];
    int d;
    GtkWidget *text;
 
+   /* Determine today for highlighting */
+   now_today = get_highlighted_today(&glob_month_date);
+
    get_month_info(glob_month_date.tm_mon, 1, glob_month_date.tm_year, &dow, &ndim);
 
    get_pref(PREF_FDOW, &fdow, NULL);
@@ -192,7 +206,8 @@ void hide_show_month_boxes()
 #else
       text = glob_month_texts[n];
 #endif
-      g_snprintf(str, sizeof(str), "%d", d);
+      g_snprintf(str, sizeof(str), "%d%s", d,
+		 d == now_today ? _(" (TODAY)") : "");
       gtk_label_set_text(GTK_LABEL(glob_month_labels[n]), str);
       if (n<7) {
 	 if (d>0) {
@@ -245,7 +260,7 @@ void create_month_boxes_texts(GtkWidget *month_vbox)
 #ifdef ENABLE_GTK2
 	    /* text variable only used to save some typing */
 	    text = glob_month_text_views[n] = gtk_text_view_new();
-	    glob_month_text_buffers[n] = 
+	    glob_month_text_buffers[n] =
 	      G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)));
 	    gtk_widget_set_usize(GTK_WIDGET(text), 10, 10);
 	    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text), FALSE);
@@ -254,20 +269,23 @@ void create_month_boxes_texts(GtkWidget *month_vbox)
 	    /* motion notify is overkill but the enter_notify event doesn't work */
 	    gtk_signal_connect(GTK_OBJECT(text), "motion_notify_event",
 			       GTK_SIGNAL_FUNC(cb_enter_notify), GINT_TO_POINTER(n));
-	    gtk_box_pack_start(GTK_BOX(vbox), text, TRUE, TRUE, 0);
 #else
 	    text = glob_month_texts[n] = gtk_text_new(NULL, NULL);
 	    gtk_widget_set_usize(GTK_WIDGET(glob_month_texts[n]), 10, 10);
 	    gtk_text_set_word_wrap(GTK_TEXT(glob_month_texts[n]), FALSE);
 	    gtk_signal_connect(GTK_OBJECT(glob_month_texts[n]), "enter_notify_event",
 			       GTK_SIGNAL_FUNC(cb_enter_notify), GINT_TO_POINTER(n));
-	    gtk_box_pack_start(GTK_BOX(vbox), text, TRUE, TRUE, 0);
 #endif
+	    gtk_signal_connect(GTK_OBJECT(text), "button_press_event",
+			       GTK_SIGNAL_FUNC(cb_enter_selected_day),
+			       GINT_TO_POINTER(n));
+
+	    gtk_box_pack_start(GTK_BOX(vbox), text, TRUE, TRUE, 0);
 	 }
       }
    }
    glob_last_hbox_row = hbox_row;
-   
+
 #ifdef ENABLE_GTK2
    text = gtk_text_view_new();
    big_text_buffer = G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)));
@@ -332,7 +350,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
    }
 
    /* Set Month name label */
-   strftime(str, sizeof(str), "%B %Y", date_in);
+   jp_strftime(str, sizeof(str), "%B %Y", date_in);
    gtk_label_set_text(GTK_LABEL(glob_month_month_label), str);
 
    memcpy(&date, date_in, sizeof(struct tm));
@@ -369,7 +387,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 #ifdef ENABLE_DATEBK
 	 get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
 	 if (use_db3_tags) {
-	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    ret = db3_parse_tag(temp_al->mappt.appt.note, &db3_type, &db4);
 	    jp_logf(JP_LOG_DEBUG, "category = 0x%x\n", db4.category);
 	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) {
@@ -378,7 +396,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 	    }
 	 }
 #endif
-	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
+	 if (isApptOnDate(&(temp_al->mappt.appt), &date)) {
 	    if (num_shown) {
 #ifdef ENABLE_GTK2
 	       gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(text_buffers[n]), "\n", -1);
@@ -390,20 +408,24 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 	       gstr=g_string_new("");
 	    }
 	    num_shown++;
-	    if (temp_al->ma.a.event) {
+	    if (temp_al->mappt.appt.event) {
 	       strcpy(desc, "*");
 	    } else {
 	       get_pref_time_no_secs(datef);
-	       strftime(desc, sizeof(desc), datef, &(temp_al->ma.a.begin));
+	       jp_strftime(desc, sizeof(desc), datef, &(temp_al->mappt.appt.begin));
 	       strcat(desc, " ");
 	    }
 	    g_string_append(gstr, desc);
-	    g_string_append(gstr, temp_al->ma.a.description);
-	    if (temp_al->ma.a.description) {
-	       strncat(desc, temp_al->ma.a.description, 36);
+	    g_string_append(gstr, temp_al->mappt.appt.description);
+	    if (temp_al->mappt.appt.description) {
+	       strncat(desc, temp_al->mappt.appt.description, 36);
 	       desc[35]='\0';
 	    }
 	    remove_cr_lfs(desc);
+
+	    /* Append number of anniversary years if enabled & appropriate */
+	    append_anni_years(desc, 35, &date, &temp_al->mappt.appt);
+
 #ifdef ENABLE_GTK2
 	    gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(text_buffers[n]), desc, -1);
 #else
@@ -415,7 +437,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
    }
    free_AppointmentList(&a_list);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 void monthview_gui(struct tm *date_in)
@@ -432,17 +454,20 @@ void monthview_gui(struct tm *date_in)
    char str[256];
    char str_dow[256];
    long fdow;
-   const char *str_fdow;
    char title[200];
-   unsigned long w, h;
+   long w, h;
 
    if (window) {
-      return;
+       /* Delete any existing window to ensure that new window is biased
+	* around currently selected date and so that the new window
+	* contents are updated with any changes on the day view.
+	*/
+       gtk_widget_destroy(window);
    }
 
    memcpy(&glob_month_date, date_in, sizeof(struct tm));
 
-   get_pref(PREF_FDOW, &fdow, &str_fdow);
+   get_pref(PREF_FDOW, &fdow, NULL);
 
    get_pref(PREF_MONTHVIEW_WIDTH, &w, NULL);
    get_pref(PREF_MONTHVIEW_HEIGHT, &h, NULL);
@@ -476,7 +501,7 @@ void monthview_gui(struct tm *date_in)
    button = gtk_button_new();
    arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT);
    gtk_container_add(GTK_CONTAINER(button), arrow);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked", 
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_month_move),
 		      GINT_TO_POINTER(-1));
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 3);
@@ -498,14 +523,14 @@ void monthview_gui(struct tm *date_in)
    button = gtk_button_new();
    arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
    gtk_container_add(GTK_CONTAINER(button), arrow);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked", 
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_month_move),
 		      GINT_TO_POINTER(1));
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 3);
 
 
    /* Month name label */
-   strftime(str, sizeof(str), "%B %Y", &glob_month_date);
+   jp_strftime(str, sizeof(str), "%B %Y", &glob_month_date);
    glob_month_month_label = gtk_label_new(str);
    gtk_box_pack_start(GTK_BOX(vbox), glob_month_month_label, FALSE, FALSE, 0);
 
@@ -523,7 +548,7 @@ void monthview_gui(struct tm *date_in)
    hbox = gtk_hbox_new(TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    for (i=0; i<7; i++) {
-      strftime(str_dow, sizeof(str_dow), "%A", &date);
+      jp_strftime(str_dow, sizeof(str_dow), "%A", &date);
       label = gtk_label_new(str_dow);
       gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
       add_days_to_date(&date, 1);
