@@ -1,4 +1,7 @@
-/* prefs.c
+/* $Id: prefs.c,v 1.60 2005/10/07 19:30:59 rousseau Exp $ */
+
+/*******************************************************************************
+ * prefs.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -27,19 +30,22 @@
 #include "utils.h"
 #include "prefs.h"
 #include "log.h"
+#ifdef ENABLE_GTK2
+#include "otherconv.h"
+#endif
 
 /*These are the default settings */
 /*name, usertype, filetype, ivalue, char *svalue, svalue_size; */
 static prefType glob_prefs[NUM_PREFS] = {
-     {EPN"rc", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"jpilotrc", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"time", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"sdate", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"ldate", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"fdow", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"show_deleted", INTTYPE, INTTYPE, 0, NULL, 0},
      {"show_modified", INTTYPE, INTTYPE, 0, NULL, 0},
-     {"hide_completed", INTTYPE, INTTYPE, 0, NULL, 0},
-     {"highlight_days", INTTYPE, INTTYPE, 1, NULL, 0},
+     {"todo_hide_completed", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"datebook_highlight_days", INTTYPE, INTTYPE, 1, NULL, 0},
      {"port", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"rate", CHARTYPE, INTTYPE, 4, NULL, 0},
      {"user", CHARTYPE, CHARTYPE, 0, NULL, 0},
@@ -58,14 +64,20 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"print_one_per_page", INTTYPE, INTTYPE, 0, NULL ,0},
      {"print_blank_lines", INTTYPE, INTTYPE, 1, NULL, 0},
      {"print_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"char_set", INTTYPE, INTTYPE, CHAR_SET_LATIN1, NULL, 0},
+     {"char_set", INTTYPE, INTTYPE,
+#ifdef ENABLE_GTK2
+     CHAR_SET_1252_UTF
+#else
+     CHAR_SET_LATIN1
+#endif
+	, NULL, 0},
      {"sync_datebook", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_address", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_todo", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_memo", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_memo32", INTTYPE, INTTYPE, 0, NULL, 0},
-     {"address_page", INTTYPE, INTTYPE, 0, NULL, 0},
-     {"output_height", INTTYPE, INTTYPE, 40, NULL, 0},
+     {"address_notebook_page", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"output_height", INTTYPE, INTTYPE, 60, NULL, 0},
      {"open_alarm_windows", INTTYPE, INTTYPE, 1, NULL, 0},
      {"do_alarm_command", INTTYPE, INTTYPE, 0, NULL, 0},
      {"alarm_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
@@ -98,7 +110,6 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"todo_hide_not_due", INTTYPE, INTTYPE, 0, NULL, 0},
      {"todo_completion_date", INTTYPE, INTTYPE, 0, NULL, 0},
      {"install_path", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"memo_show_prefs", INTTYPE, INTTYPE, 1, NULL, 0},
      {"monthview_width", INTTYPE, INTTYPE, 640, NULL, 0},
      {"monthview_height", INTTYPE, INTTYPE, 480, NULL, 0},
      {"weekview_width", INTTYPE, INTTYPE, 640, NULL, 0},
@@ -107,7 +118,18 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"last_addr_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
      {"last_todo_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
      {"last_memo_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
+     {"mail_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"version", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"utf_encoding", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"confirm_file_install", INTTYPE, INTTYPE, 1, NULL, 0},
+     {"todo_days_due", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"todo_days_till_due", INTTYPE, INTTYPE, 7, NULL, 0},
+     {"show_tooltips", INTTYPE, INTTYPE, 1, NULL, 0},
+     {"datebook_note_pane", INTTYPE, INTTYPE, 75, NULL, 0},
+     {"datebook_hi_today", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"datebook_anni_years", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"keyring_pane", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"expense_pane", INTTYPE, INTTYPE, 0, NULL, 0},
 };
 
 struct name_list {
@@ -148,6 +170,10 @@ void pref_init()
 	 glob_prefs[i].svalue=strdup("jpilot-dial --lv 0 --rv 50 %n");
 	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
 	 break;
+       case PREF_MAIL_COMMAND:
+	 glob_prefs[i].svalue=strdup("mozilla -remote \"mailto(\%s)\"");
+	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
+	 break;
        default:
 	 glob_prefs[i].svalue=strdup("");
 	 glob_prefs[i].svalue_size=1;
@@ -202,13 +228,12 @@ int get_pref_time_no_secs(char *datef)
    /* "%I:%M:%S" */
    /* "%I:%M" */
    /* "%I:%M%p" */
-   long ivalue;
    const char *svalue;
    int i1, i2;
 
-   get_pref(PREF_TIME, &ivalue, &svalue);
+   get_pref(PREF_TIME, NULL, &svalue);
    if (!svalue) {
-      return -1;
+      return EXIT_FAILURE;
    }
    for (i1=0, i2=0; ; i1++, i2++) {
       if (svalue[i2]=='S') {
@@ -224,17 +249,16 @@ int get_pref_time_no_secs(char *datef)
 	 break;
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int get_pref_time_no_secs_no_ampm(char *datef)
 {
-   long ivalue;
    const char *svalue;
 
-   get_pref(PREF_TIME, &ivalue, &svalue);
+   get_pref(PREF_TIME, NULL, &svalue);
    if (!svalue) {
-      return -1;
+      return EXIT_FAILURE;
    }
    if (svalue) {
       strncpy(datef, svalue, 5);
@@ -243,27 +267,9 @@ int get_pref_time_no_secs_no_ampm(char *datef)
       datef[0]='\0';
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
-int get_pref_dmy_order()
-{
-   long n;
-
-   get_pref(PREF_SHORTDATE, &n, NULL);
-   if (n<1) {
-      return PREF_MDY;
-   }
-   if ((n>0) && (n<4)) {
-      return PREF_DMY;
-   }
-   if ((n>3)) {
-      return PREF_YMD;
-   }
-   return 0;
-}
-
-
 /*This function is used internally to free up any memory that prefs is using */
 /* I'm not using this function right now.
 void free_name_list(struct name_list **Plist)
@@ -305,9 +311,9 @@ static int get_rcfile_name(int n, char *rc_copy)
 	       jp_logf(JP_LOG_DEBUG, "found %s\n", dirent->d_name);
 	       new_entry = malloc(sizeof(struct name_list));
 	       if (!new_entry) {
-		  jp_logf(JP_LOG_FATAL, "get_rcfile_name(): Out of memory\n");
-		  return -1;
-	       }  
+		  jp_logf(JP_LOG_FATAL, "get_rcfile_name(): %s\n", _("Out of memory"));
+		  return EXIT_FAILURE;
+	       }
 	       new_entry->name = strdup(dirent->d_name);
 	       new_entry->next = dir_list;
 	       dir_list = new_entry;
@@ -330,9 +336,9 @@ static int get_rcfile_name(int n, char *rc_copy)
 	       jp_logf(JP_LOG_DEBUG, "found %s\n", dirent->d_name);
 	       new_entry = malloc(sizeof(struct name_list));
 	       if (!new_entry) {
-		  jp_logf(JP_LOG_FATAL, "get_rcfile_name(): Out of memory 2\n");
-		  return -1;
-	       }  
+		  jp_logf(JP_LOG_FATAL, "get_rcfile_name(): %s 2\n", _("Out of memory"));
+		  return EXIT_FAILURE;
+	       }
 	       new_entry->name = strdup(dirent->d_name);
 	       new_entry->next = dir_list;
 	       dir_list = new_entry;
@@ -355,10 +361,10 @@ static int get_rcfile_name(int n, char *rc_copy)
    }
 
    if (found) {
-      return 0;
+      return EXIT_SUCCESS;
    } else {
       rc_copy[0]='\0';
-      return -1;
+      return EXIT_FAILURE;
    }
 }
 
@@ -382,12 +388,12 @@ int get_pref_possibility(int which, int n, char *pref_str)
    };
 
    const char *long_date_formats[] = {
-      "%B %d, %Y",
-      "%d %B %Y",
-      "%d. %B %Y",
-      "%d %B, %Y",
-      "%Y. %B. %d",
-      "%Y %B %d"
+      N_("%B %d, %Y"),
+      N_("%d %B %Y"),
+      N_("%d. %B %Y"),
+      N_("%d %B, %Y"),
+      N_("%Y. %B. %d"),
+      N_("%Y %B %d")
    };
 
    const char *time_formats[] = {
@@ -403,7 +409,10 @@ int get_pref_possibility(int which, int n, char *pref_str)
       "%H,%M"
    };
 
-   static char *days[2];
+   static char *days[] = {
+      N_("Sunday"),
+      N_("Monday")
+   };
 
    static const char *rates[] = {
       "300",
@@ -422,14 +431,21 @@ int get_pref_possibility(int which, int n, char *pref_str)
    static const char *char_sets[] = {
       "Latin1 / No conversion",
       "Japanese",
-      "Host ISO-8859-2 <-> Palm Windows1250 (EE)", 
+      "Host ISO-8859-2 <-> Palm Windows1250 (EE)",
       "Host Windows1251 <-> Palm KOI8-R",
       "Host KOI8-R <-> Palm Windows-1251",
       "Chinese(Big5)",
       "Korean",
-      "Host UTF-8 <-> Palm Windows1250 (EE)",
-      "Host UTF-8 <-> Palm Windows1252", /* JPA */
-      "Host UTF-8 <-> Palm Windows1253 (Greek)"
+      "UTF: Latin 2, Eastern Europe (CP1250)",
+      "UTF: Latin 1, Western Europe (CP1252)",
+      "UTF: Greek (CP1253)",
+      "UTF: Latin 2, Eastern Europe (ISO8859-2)",
+      "UTF: Cyrillic (KOI8-R)",
+      "UTF: Cyrillic (CP1251)",
+      "UTF: Simplified Chinese (GBK)",
+      "UTF: Japanese (SJIS)",
+      "UTF: Hebrew (CP1255)",
+      "UTF: Traditional Chinese (BIG-5)",
    };
 
    static const char *paper_sizes[] = {
@@ -437,9 +453,6 @@ int get_pref_possibility(int which, int n, char *pref_str)
       "A4"
    };
 
-   days[0] = _("Sunday");
-   days[1] = _("Monday");
-
    switch(which) {
 
     case PREF_RCFILE:
@@ -449,7 +462,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
     case PREF_TIME:
       if ((n >= NUM_TIMES) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
       strcpy(pref_str, time_formats[n]);
       break;
@@ -457,7 +470,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
     case PREF_SHORTDATE:
       if ((n >= NUM_SHORTDATES) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
       strcpy(pref_str, short_date_formats[n]);
       break;
@@ -465,7 +478,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
     case PREF_LONGDATE:
       if ((n >= NUM_LONGDATES) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
       strcpy(pref_str, long_date_formats[n]);
       break;
@@ -473,15 +486,15 @@ int get_pref_possibility(int which, int n, char *pref_str)
     case PREF_FDOW:
       if ((n > 1) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
-      strcpy(pref_str, days[n]);
+      strcpy(pref_str, _(days[n]));
       break;
 
     case PREF_RATE:
       if ((n >= NUM_RATES) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
       strcpy(pref_str, rates[n]);
       break;
@@ -489,7 +502,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
     case PREF_CHAR_SET:
       if ((n >= NUM_CHAR_SETS) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
       strcpy(pref_str, char_sets[n]);
       break;
@@ -497,7 +510,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
     case PREF_PAPER_SIZE:
       if ((n >= NUM_PAPER_SIZES) || (n<0)) {
 	 pref_str[0]='\0';
-	 return -1;
+	 return EXIT_FAILURE;
       }
       strcpy(pref_str, paper_sizes[n]);
       break;
@@ -505,10 +518,10 @@ int get_pref_possibility(int which, int n, char *pref_str)
     default:
       pref_str[0]='\0';
       jp_logf(JP_LOG_DEBUG, "Unknown preference type\n");
-      return -1;
+      return EXIT_FAILURE;
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -518,7 +531,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
 int jp_get_pref(prefType prefs[], int which, long *n, const char **string)
 {
    if (which < 0) {
-      return -1;
+      return EXIT_FAILURE;
    }
    if (n) {
       *n = prefs[which].ivalue;
@@ -532,13 +545,13 @@ int jp_get_pref(prefType prefs[], int which, long *n, const char **string)
 	 *string = NULL;
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int get_pref(int which, long *n, const char **string)
 {
    if (which > NUM_PREFS) {
-      return -1;
+      return EXIT_FAILURE;
    }
    return jp_get_pref(glob_prefs, which, n, string);
 }
@@ -588,7 +601,7 @@ char *pref_lstrncpy_realloc(char **dest, const char *src, int *size, int max_siz
 	 *dest=realloc(*dest, new_size);
       }
       if (!(*dest)) {
-	 return 0;
+	 return "";
       }
       *size=new_size;
    }
@@ -604,7 +617,7 @@ int jp_set_pref(prefType prefs[], int which, long n, const char *string)
    const char *Pstr;
 
    if (which < 0) {
-      return -1;
+      return EXIT_FAILURE;
    }
    prefs[which].ivalue = n;
    if (string == NULL) {
@@ -616,7 +629,7 @@ int jp_set_pref(prefType prefs[], int which, long n, const char *string)
       pref_lstrncpy_realloc(&(prefs[which].svalue), Pstr,
 			    &(prefs[which].svalue_size), MAX_PREF_VALUE);
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int set_pref(int which, long n, const char *string, int save)
@@ -625,7 +638,7 @@ int set_pref(int which, long n, const char *string, int save)
    int r;
 
    if (which > NUM_PREFS) {
-      return -1;
+      return EXIT_FAILURE;
    }
    str=string;
    if ((which==PREF_RCFILE) ||
@@ -654,7 +667,7 @@ int set_pref_possibility(int which, long n, int save)
    int r;
 
    if (which > NUM_PREFS) {
-      return -1;
+      return EXIT_FAILURE;
    }
    if (glob_prefs[which].usertype == CHARTYPE) {
       get_pref_possibility(which, n, svalue);
@@ -668,6 +681,13 @@ int set_pref_possibility(int which, long n, int save)
       pref_write_rc_file();
    }
    /* #endif */
+
+#ifdef ENABLE_GTK2
+   if (PREF_CHAR_SET == which)
+      if (otherconv_init())
+	 printf("Error: could not set encoding\n");
+#endif
+
    return r;
 }
 
@@ -718,18 +738,18 @@ static int validate_glob_prefs()
       glob_prefs[PREF_SHOW_MODIFIED].ivalue = 0;
    }
 
-   if (glob_prefs[PREF_HIDE_COMPLETED].ivalue > 1) {
-      glob_prefs[PREF_HIDE_COMPLETED].ivalue = 1;
+   if (glob_prefs[PREF_TODO_HIDE_COMPLETED].ivalue > 1) {
+      glob_prefs[PREF_TODO_HIDE_COMPLETED].ivalue = 1;
    }
-   if (glob_prefs[PREF_HIDE_COMPLETED].ivalue < 0) {
-      glob_prefs[PREF_HIDE_COMPLETED].ivalue = 0;
+   if (glob_prefs[PREF_TODO_HIDE_COMPLETED].ivalue < 0) {
+      glob_prefs[PREF_TODO_HIDE_COMPLETED].ivalue = 0;
    }
 
-   if (glob_prefs[PREF_HIGHLIGHT].ivalue > 1) {
-      glob_prefs[PREF_HIGHLIGHT].ivalue = 1;
+   if (glob_prefs[PREF_DATEBOOK_HIGHLIGHT_DAYS].ivalue > 1) {
+      glob_prefs[PREF_DATEBOOK_HIGHLIGHT_DAYS].ivalue = 1;
    }
-   if (glob_prefs[PREF_HIGHLIGHT].ivalue < 0) {
-      glob_prefs[PREF_HIGHLIGHT].ivalue = 0;
+   if (glob_prefs[PREF_DATEBOOK_HIGHLIGHT_DAYS].ivalue < 0) {
+      glob_prefs[PREF_DATEBOOK_HIGHLIGHT_DAYS].ivalue = 0;
    }
 
    if (glob_prefs[PREF_RATE].ivalue >= NUM_RATES) {
@@ -793,7 +813,7 @@ static int validate_glob_prefs()
 	 break;
       }
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
@@ -806,14 +826,14 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
 
    in=jp_open_home_file(filename, "r");
    if (!in) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    while (!feof(in)) {
-      fgets(line, 1024, in);
+      fgets(line, sizeof(line), in);
       if (feof(in)) break;
-      line[1023] = ' ';
-      line[1023] = '\0';
+      line[sizeof(line)-2] = ' ';
+      line[sizeof(line)-1] = '\0';
       field1 = strtok(line, " ");
       field2 = (field1 != NULL)	? strtok(NULL, "\n") : NULL;/* jonh */
       if ((field1 == NULL) || (field2 == NULL)) {
@@ -831,7 +851,7 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
 	       if (pref_lstrncpy_realloc(&(prefs[i].svalue), field2,
 					&(prefs[i].svalue_size),
 					MAX_PREF_VALUE)==NULL) {
-		  jp_logf(JP_LOG_WARN, "Out of memory: read_rc_file()\n");
+		  jp_logf(JP_LOG_WARN, "read_rc_file(): %s\n", _("Out of memory"));
 		  continue;
 	       }
 	    }
@@ -840,7 +860,7 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
    }
    fclose(in);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int pref_read_rc_file()
@@ -863,7 +883,7 @@ int jp_pref_write_rc_file(char *filename, prefType prefs[], int num_prefs)
 
    out=jp_open_home_file(filename,"w" );
    if (!out) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    for(i=0; i<num_prefs; i++) {
@@ -878,7 +898,7 @@ int jp_pref_write_rc_file(char *filename, prefType prefs[], int num_prefs)
    }
    fclose(out);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int pref_write_rc_file()
