@@ -1,4 +1,7 @@
-/* prefs.h
+/* $Id: prefs.h,v 1.41 2005/10/07 19:30:59 rousseau Exp $ */
+
+/*******************************************************************************
+ * prefs.h
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #ifndef __PREFS_H__
 #define __PREFS_H__
 
@@ -28,8 +32,8 @@
 #define PREF_FDOW 4 /*First Day Of the Week */
 #define PREF_SHOW_DELETED 5
 #define PREF_SHOW_MODIFIED 6
-#define PREF_HIDE_COMPLETED 7
-#define PREF_HIGHLIGHT 8
+#define PREF_TODO_HIDE_COMPLETED 7
+#define PREF_DATEBOOK_HIGHLIGHT_DAYS 8
 #define PREF_PORT 9
 #define PREF_RATE 10
 #define PREF_USER 11
@@ -84,28 +88,34 @@
 #define PREF_DIAL_COMMAND 60
 #define PREF_DATEBOOK_TODO_PANE 61
 #define PREF_DATEBOOK_TODO_SHOW 62
-#define PREF_HIDE_NOT_DUE 63
+#define PREF_TODO_HIDE_NOT_DUE 63
 #define PREF_TODO_COMPLETION_DATE 64
 #define PREF_INSTALL_PATH 65
-#define PREF_SHOW_MEMO_PREFS 66
-#define PREF_MONTHVIEW_WIDTH 67
-#define PREF_MONTHVIEW_HEIGHT 68
-#define PREF_WEEKVIEW_WIDTH 69
-#define PREF_WEEKVIEW_HEIGHT 70
-#define PREF_LAST_DATE_CATEGORY 71
-#define PREF_LAST_ADDR_CATEGORY 72
-#define PREF_LAST_TODO_CATEGORY 73
-#define PREF_LAST_MEMO_CATEGORY 74
+#define PREF_MONTHVIEW_WIDTH 66
+#define PREF_MONTHVIEW_HEIGHT 67
+#define PREF_WEEKVIEW_WIDTH 68
+#define PREF_WEEKVIEW_HEIGHT 69
+#define PREF_LAST_DATE_CATEGORY 70
+#define PREF_LAST_ADDR_CATEGORY 71
+#define PREF_LAST_TODO_CATEGORY 72
+#define PREF_LAST_MEMO_CATEGORY 73
+#define PREF_MAIL_COMMAND 74
 #define PREF_VERSION 75
+#define PREF_UTF_ENCODING 76
+#define PREF_CONFIRM_FILE_INSTALL 77
+#define PREF_TODO_DAYS_DUE 78
+#define PREF_TODO_DAYS_TILL_DUE 79
+#define PREF_SHOW_TOOLTIPS 80
+#define PREF_DATEBOOK_NOTE_PANE 81
+#define PREF_DATEBOOK_HI_TODAY 82
+#define PREF_DATEBOOK_ANNI_YEARS 83
+#define PREF_KEYRING_PANE 84
+#define PREF_EXPENSE_PANE 85
 
-#define NUM_PREFS 76
+#define NUM_PREFS 86
 
 #define MAX_PREF_NUM_BACKUPS 99
 
-#define PREF_MDY 0
-#define PREF_DMY 1
-#define PREF_YMD 2
-
 #define CHAR_SET_LATIN1   0 /* English, European, Latin based languages */
 #define CHAR_SET_JAPANESE 1
 #define CHAR_SET_1250     2 /* Czech, Polish (Unix: ISO-8859-2) */
@@ -113,10 +123,18 @@
 #define CHAR_SET_1251_B   4 /* Russian; palm win1251, host koi8-r */
 #define CHAR_SET_TRADITIONAL_CHINESE  5 /* Taiwan Chinese */
 #define CHAR_SET_KOREAN   6 /* Korean Hangul */
-#define CHAR_SET_1250UTF  7 /* Czech, Polish (Unix: UTF-8) */
-#define CHAR_SET_LATINUTF 8 /* JPA : Latin European (Unix: UTF-8) */
-#define CHAR_SET_1253UTF  9 /* Modern Greek (Palm: Windows 1253, Host: UTF-8) */
-#define NUM_CHAR_SETS    10
+#define CHAR_SET_UTF      7
+#define CHAR_SET_1250_UTF  7 /* Czech, Polish (latin2, CP1250) */
+#define CHAR_SET_1252_UTF  8 /* Latin European (latin1, CP1252) */
+#define CHAR_SET_1253_UTF  9 /* Modern Greek (CP1253) */
+#define CHAR_SET_ISO8859_2_UTF 10 /* Czech, Polish (latin2, ISO8859-2) */
+#define CHAR_SET_KOI8_R_UTF 11 /* Cyrillic (KOI8-R) */
+#define CHAR_SET_1251_UTF 12 /* Cyrillic (CP1251) */
+#define CHAR_SET_GBK_UTF 13 /* Chinese (GB2312) */
+#define CHAR_SET_SJIS_UTF 14 /* Japanese (SJIS) */
+#define CHAR_SET_1255_UTF 15 /* Hebrew (CP1255) */
+#define CHAR_SET_BIG5_UTF 16 /* Chinese (BIG-5) */
+#define NUM_CHAR_SETS    17
 
 #define MAX_PREF_VALUE 200
 
