@@ -1,4 +1,7 @@
-/* prefs_gui.c
+/* $Id: prefs_gui.c,v 1.47 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * prefs_gui.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -33,8 +36,11 @@ static GtkWidget *main_window;
 static GtkWidget *port_entry;
 static GtkWidget *backups_entry;
 static GtkWidget *alarm_command_entry;
+static GtkWidget *mail_command_entry;
+static GtkWidget *todo_days_due_entry;
 
 extern int glob_app;
+extern GtkTooltips *glob_tooltips;
 
 #ifdef COLORS
 
@@ -42,7 +48,7 @@ extern int glob_app;
 void r(GtkWidget *w, gpointer data)
 {
    GtkStyle *style;
-   
+
    style = gtk_rc_get_style(GTK_WIDGET(w));
    if (style) gtk_rc_style_unref(style);
    if (GTK_IS_CONTAINER(w)) {
@@ -54,10 +60,10 @@ void set_colors()
 {
    GtkStyle* style;
    int i;
-   
+
    r(main_window, NULL);
    r(window, NULL);
-   
+
    gtk_rc_reparse_all();
    read_gtkrc_file();
    gtk_rc_reparse_all();
@@ -94,7 +100,6 @@ static void cb_pref_menu(GtkWidget *widget,
    return;
 }
 
-
 int make_pref_menu(GtkWidget **pref_menu, int pref_num)
 {
    GtkWidget *menu_item;
@@ -125,9 +130,11 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
       }
       switch (pref_num) {
        case PREF_SHORTDATE:
-       case PREF_LONGDATE:
        case PREF_TIME:
-	 strftime(human_text, MAX_PREF_VALUE, format_text, now);
+	 jp_strftime(human_text, MAX_PREF_VALUE, format_text, now);
+	 break;
+       case PREF_LONGDATE:
+	 jp_strftime(human_text, MAX_PREF_VALUE, _(format_text), now);
 	 break;
        default:
 	 strncpy(human_text, format_text, MAX_PREF_VALUE);
@@ -149,56 +156,62 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
    }
    gtk_option_menu_set_menu(GTK_OPTION_MENU(*pref_menu), menu);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
-void cb_show_deleted(GtkWidget *widget, gpointer data)
+void cb_backups_entry(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_SHOW_DELETED, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+   const char *entry_text;
+   int num_backups;
+
+   entry_text = gtk_entry_get_text(GTK_ENTRY(backups_entry));
+   sscanf(entry_text, "%d", &num_backups);
+
+   if (num_backups < 1) {
+      num_backups = 1;
+   }
+   if (num_backups > 99) {
+      num_backups = 99;
+   }
+
+   set_pref(PREF_NUM_BACKUPS, num_backups, NULL, FALSE);
 }
-void cb_show_modified(GtkWidget *widget, gpointer data)
+
+void cb_checkbox_todo_days_till_due(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_SHOW_MODIFIED, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+   int num_days;
+   const char *entry_text;
+
+   entry_text = gtk_entry_get_text(GTK_ENTRY(todo_days_due_entry));
+
+   sscanf(entry_text, "%d", &num_days);
+
+   set_pref(PREF_TODO_DAYS_TILL_DUE, num_days, NULL, TRUE);
 }
-void cb_highlight(GtkWidget *widget, gpointer data)
+
+void cb_checkbox_show_tooltips(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_HIGHLIGHT, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+   if (GTK_TOGGLE_BUTTON(widget)->active)
+      gtk_tooltips_enable(glob_tooltips);
+   else
+      gtk_tooltips_disable(glob_tooltips);
+
+   set_pref(PREF_SHOW_TOOLTIPS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
-void cb_use_db3(GtkWidget *widget, gpointer data)
+
+void cb_text_entry(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_USE_DB3, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+   const char *entry_text;
+
+   entry_text = gtk_entry_get_text(GTK_ENTRY(widget));
+   set_pref(GPOINTER_TO_INT(data), 0, entry_text, FALSE);
 }
-void cb_sync_datebook(GtkWidget *widget, gpointer data)
+
+void cb_checkbox_set_pref(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_SYNC_DATEBOOK, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-void cb_sync_address(GtkWidget *widget, gpointer data)
-{
-   set_pref(PREF_SYNC_ADDRESS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-void cb_sync_todo(GtkWidget *widget, gpointer data)
-{
-   set_pref(PREF_SYNC_TODO, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-void cb_sync_memo(GtkWidget *widget, gpointer data)
-{
-   set_pref(PREF_SYNC_MEMO, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-void cb_sync_memo32(GtkWidget *widget, gpointer data)
-{
-   set_pref(PREF_SYNC_MEMO32, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-#ifdef ENABLE_MANANA
-void cb_sync_manana(GtkWidget *widget, gpointer data)
-{
-   set_pref(PREF_SYNC_MANANA, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-#endif
-void cb_use_jos(GtkWidget *widget,
-		gpointer data)
-{
-   set_pref(PREF_USE_JOS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+   set_pref(GPOINTER_TO_INT(data), GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
+
 #ifdef ENABLE_PLUGINS
 void cb_sync_plugin(GtkWidget *widget,
 		    gpointer data)
@@ -228,58 +241,23 @@ void cb_sync_plugin(GtkWidget *widget,
    write_plugin_sync_file();
 }
 #endif
-void cb_open_alarm(GtkWidget *widget,
-		   gpointer data)
-{
-   set_pref(PREF_OPEN_ALARM_WINDOWS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-void cb_do_command(GtkWidget *widget,
-		   gpointer data)
-{
-   set_pref(PREF_DO_ALARM_COMMAND, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
-}
-
-
 
 static gboolean cb_destroy(GtkWidget *widget)
 {
-   const char *entry_text;
-   const char *backups_text;
-   int num_backups;
-
    jp_logf(JP_LOG_DEBUG, "Cleanup\n");
 
-   entry_text = gtk_entry_get_text(GTK_ENTRY(port_entry));
-   jp_logf(JP_LOG_DEBUG, "port_entry = [%s]\n", entry_text);
-   set_pref(PREF_PORT, 0, entry_text, FALSE);
-
-   entry_text = gtk_entry_get_text(GTK_ENTRY(alarm_command_entry));
-   jp_logf(JP_LOG_DEBUG, "alarm_command_entry = [%s]\n", entry_text);
-   set_pref(PREF_ALARM_COMMAND, 0, entry_text, FALSE);
-
-   backups_text = gtk_entry_get_text(GTK_ENTRY(backups_entry));
-   jp_logf(JP_LOG_DEBUG, "backups_entry = [%s]\n", backups_text);
-   num_backups = atoi(backups_text);
-   if (num_backups < 1) {
-      num_backups = 1;
-   }
-   if (num_backups > 99) {
-      num_backups = 99;
-   }
-   set_pref(PREF_NUM_BACKUPS, num_backups, NULL, TRUE);
-
    pref_write_rc_file();
 
    window = NULL;
-   if (glob_app==DATEBOOK) {
-      cb_app_button(NULL, GINT_TO_POINTER(REDRAW));
-   }
+
+   /* Preference changes can affect visual elements of applications.
+    * Redraw the screen to incorporate any changes made. */
+   cb_app_button(NULL, GINT_TO_POINTER(REDRAW));
+
    return FALSE;
 }
 
-static void
-  cb_quit(GtkWidget *widget,
-	   gpointer   data)
+static void cb_quit(GtkWidget *widget, gpointer data)
 {
    jp_logf(JP_LOG_DEBUG, "cb_quit\n");
    if (GTK_IS_WIDGET(data)) {
@@ -287,6 +265,27 @@ static void
    }
 }
 
+/* This function adds a simple option checkbutton for the supplied text +
+ * option.
+ */
+static void
+add_checkbutton(const char *text, int which, GtkWidget *vbox,
+		void cb(GtkWidget *widget, gpointer data))
+{
+   /* Create button */
+   GtkWidget *checkbutton = gtk_check_button_new_with_label(text);
+   gtk_box_pack_start(GTK_BOX(vbox), checkbutton, FALSE, FALSE, 0);
+   gtk_widget_show(checkbutton);
+
+   /* Set the button state based on option value */
+   if (get_pref_int_default(which, 0))
+       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+
+   /* Set button callback */
+   gtk_signal_connect(GTK_OBJECT(checkbutton), "clicked", GTK_SIGNAL_FUNC(cb),
+		      GINT_TO_POINTER(which));
+}
+
 void cb_prefs_gui(GtkWidget *widget, gpointer data)
 {
    GtkWidget *checkbutton;
@@ -297,6 +296,10 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    GtkWidget *vbox;
    GtkWidget *vbox_locale;
    GtkWidget *vbox_settings;
+   GtkWidget *vbox_datebook;
+   GtkWidget *vbox_address;
+   GtkWidget *vbox_todo;
+   GtkWidget *vbox_memo;
    GtkWidget *vbox_alarms;
    GtkWidget *vbox_conduits;
    GtkWidget *hbox_temp;
@@ -315,7 +318,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    if (window) {
       jp_logf(JP_LOG_DEBUG, "pref_window is already up\n");
 #ifdef ENABLE_GTK2
-      /* Shift focus to existing window if called again 
+      /* Shift focus to existing window if called again
          and window is still alive. */
       gtk_window_present(GTK_WINDOW(window));
 #endif
@@ -337,13 +340,21 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    vbox = gtk_vbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(window), vbox);
 
-   vbox_locale = gtk_vbox_new(FALSE, 0);
+   vbox_locale   = gtk_vbox_new(FALSE, 0);
    vbox_settings = gtk_vbox_new(FALSE, 0);
-   vbox_alarms = gtk_vbox_new(FALSE, 0);
+   vbox_datebook = gtk_vbox_new(FALSE, 0);
+   vbox_address  = gtk_vbox_new(FALSE, 0);
+   vbox_todo     = gtk_vbox_new(FALSE, 0);
+   vbox_memo     = gtk_vbox_new(FALSE, 0);
+   vbox_alarms   = gtk_vbox_new(FALSE, 0);
    vbox_conduits = gtk_vbox_new(FALSE, 0);
 
    gtk_container_set_border_width(GTK_CONTAINER(vbox_locale), 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_settings), 5);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox_datebook), 5);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox_address), 5);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox_todo), 5);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox_memo), 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_alarms), 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_conduits), 5);
 
@@ -354,13 +365,22 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_locale, label);
    label = gtk_label_new(_("Settings"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_settings, label);
+   label = gtk_label_new(_("Datebook"));
+   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_datebook, label);
+   label = gtk_label_new(_("Address"));
+   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_address, label);
+   label = gtk_label_new(_("ToDo"));
+   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_todo, label);
+   label = gtk_label_new(_("Memo"));
+   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_memo, label);
    label = gtk_label_new(_("Alarms"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_alarms, label);
    label = gtk_label_new(_("Conduits"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_conduits, label);
    gtk_box_pack_start(GTK_BOX(vbox), notebook, FALSE, FALSE, 0);
 
-   /* Table for Locale */
+   /************************************************************/
+   /* Locale preference tab */
    table = gtk_table_new(5, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table),0);
    gtk_table_set_col_spacings(GTK_TABLE(table),0);
@@ -405,7 +425,6 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    get_pref(PREF_LONGDATE, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
-
    /* Time */
    label = gtk_label_new(_("Time format "));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
@@ -419,8 +438,8 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    get_pref(PREF_TIME, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
-
    /* FDOW */
+#  ifndef ENABLE_GTK2
    label = gtk_label_new(_("The first day of the week is "));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 4, 5);
@@ -432,9 +451,10 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 
    get_pref(PREF_FDOW, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
+#  endif
 
-
-   /* Table for Settings*/
+   /**********************************************************************/
+   /* Settings preference tab */
    table = gtk_table_new(4, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table),0);
    gtk_table_set_col_spacings(GTK_TABLE(table),0);
@@ -453,7 +473,6 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    get_pref(PREF_RCFILE, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
-
    /* Port */
    label = gtk_label_new(_("Serial Port (/dev/ttyS0, /dev/pilot)"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
@@ -467,7 +486,9 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(port_entry), cstr);
    }
-
+   gtk_signal_connect(GTK_OBJECT(port_entry),
+		      "changed", GTK_SIGNAL_FUNC(cb_text_entry),
+		      GINT_TO_POINTER(PREF_PORT));
 
    /* Rate */
    label = gtk_label_new(_("Serial Rate (Does not affect USB)"));
@@ -495,200 +516,140 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    get_pref(PREF_NUM_BACKUPS, &ivalue, &cstr);
    sprintf(temp_str, "%ld", ivalue);
    gtk_entry_set_text(GTK_ENTRY(backups_entry), temp_str);
-
+   gtk_signal_connect(GTK_OBJECT(backups_entry),
+		      "changed", GTK_SIGNAL_FUNC(cb_backups_entry),
+		      NULL);
 
    /* Show deleted files check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Show deleted records (default NO)"));
-   gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SHOW_DELETED, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton),
-		      "clicked", GTK_SIGNAL_FUNC(cb_show_deleted),
-		      GINT_TO_POINTER(PREF_SHOW_DELETED));
+   add_checkbutton(_("Show deleted records (default NO)"),
+		   PREF_SHOW_DELETED, vbox_settings, cb_checkbox_set_pref);
 
    /* Show modified files check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Show modified deleted records (default NO)"));
-   gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SHOW_MODIFIED, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_show_modified),
-		      GINT_TO_POINTER(PREF_SHOW_MODIFIED));
+   add_checkbutton(_("Show modified deleted records (default NO)"),
+		   PREF_SHOW_MODIFIED, vbox_settings, cb_checkbox_set_pref);
 
+   /* Confirm file installation */
+   add_checkbutton(
+      _("Ask confirmation for file installation (J-Pilot -> PDA) (default YES)"),
+      PREF_CONFIRM_FILE_INSTALL, vbox_settings, cb_checkbox_set_pref);
+
+   /* Show tooltips check box */
+   add_checkbutton(_("Show popup tooltips (default YES)"),
+		   PREF_SHOW_TOOLTIPS, vbox_settings,
+		   cb_checkbox_show_tooltips);
+
+   /**********************************************************************/
+   /* Datebook preference tab */
 
    /* Show highlight days check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Highlight calendar days with appointments"));
-   gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_HIGHLIGHT, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_highlight), NULL);
-
+   add_checkbutton(_("Highlight calendar days with appointments"),
+		   PREF_DATEBOOK_HIGHLIGHT_DAYS, vbox_datebook,
+		   cb_checkbox_set_pref);
 
 #ifdef ENABLE_DATEBK
    /* Show use DateBk check box */
-   checkbutton = gtk_check_button_new_with_label(_("Use DateBk note tags"));
-   gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_USE_DB3, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_use_db3), NULL);
+   add_checkbutton(_("Use DateBk note tags"),
+		   PREF_USE_DB3, vbox_datebook, cb_checkbox_set_pref);
 #else
    label = gtk_label_new(_("DateBk support disabled in this build"));
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-   gtk_box_pack_start(GTK_BOX(vbox_settings), label, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_datebook), label, FALSE, FALSE, 0);
 #endif
 
+    /* Highlight today on month and week view */
+    add_checkbutton(_("Annotate today in day, week, and month views"),
+ 		   PREF_DATEBOOK_HI_TODAY, vbox_datebook, cb_checkbox_set_pref);
+ 
+    /* Show number of years on anniversaries in month and week view */
+    add_checkbutton(
+ 	_("Append years on anniversaries in day, week, and month views"),
+ 	PREF_DATEBOOK_ANNI_YEARS, vbox_datebook,
+ 	cb_checkbox_set_pref);
+ 
 
-   /* Show sync datebook check box */
-   checkbutton = gtk_check_button_new_with_label(_("Sync datebook"));
-   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SYNC_DATEBOOK, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_sync_datebook), NULL);
+   /**********************************************************************/
+   /* Address preference tab */
 
-   /* Show sync address check box */
-   checkbutton = gtk_check_button_new_with_label(_("Sync address"));
-   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SYNC_ADDRESS, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_sync_address), NULL);
+   /* Command to use for e-mailing from address book */
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_address), hbox_temp, FALSE, FALSE, 0);
 
-   /* Show sync todo check box */
-   checkbutton = gtk_check_button_new_with_label(_("Sync todo"));
-   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SYNC_TODO, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_sync_todo), NULL);
+   label = gtk_label_new(_("Mail Command"));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 0);
 
-   /* Show sync memo check box */
-   checkbutton = gtk_check_button_new_with_label(_("Sync memo"));
-   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SYNC_MEMO, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_sync_memo), NULL);
+   mail_command_entry = gtk_entry_new_with_max_length(MAX_PREF_VALUE - 2);
 
-   /* Show sync Memo32 check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Sync memo32 (pedit32)"));
-   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SYNC_MEMO32, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+   get_pref(PREF_MAIL_COMMAND, &ivalue, &cstr);
+   if (cstr) {
+      gtk_entry_set_text(GTK_ENTRY(mail_command_entry), cstr);
    }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_sync_memo32), NULL);
+   gtk_signal_connect(GTK_OBJECT(mail_command_entry),
+		      "changed", GTK_SIGNAL_FUNC(cb_text_entry),
+		      GINT_TO_POINTER(PREF_MAIL_COMMAND));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), mail_command_entry, TRUE, TRUE, 1);
+
+   label = gtk_label_new(_("%s is replaced by the e-mail address"));
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+   gtk_box_pack_start(GTK_BOX(vbox_address), label, FALSE, FALSE, 0);
+
+   /**********************************************************************/
+   /* ToDo preference tab */
+
+   /* The hide completed check box */
+   add_checkbutton(_("Hide Completed ToDos"),
+		   PREF_TODO_HIDE_COMPLETED, vbox_todo, cb_checkbox_set_pref);
+
+   /* The hide todos not yet due check box */
+   add_checkbutton(_("Hide ToDos not yet due"),
+		   PREF_TODO_HIDE_NOT_DUE, vbox_todo, cb_checkbox_set_pref);
+
+   /* The record todo completion date check box */
+   add_checkbutton(_("Record Completion Date"),
+		   PREF_TODO_COMPLETION_DATE, vbox_todo, cb_checkbox_set_pref);
 
 #ifdef ENABLE_MANANA
-   /* Show sync Ma~nana check box */
-   checkbutton = gtk_check_button_new_with_label(_("Sync Manana"));
-   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_SYNC_MANANA, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_sync_manana), NULL);
-#endif
-   get_pref(PREF_CHAR_SET, &ivalue, &cstr);
-   if (ivalue == CHAR_SET_JAPANESE) {
-      /*Show use Japanese Kana extention check box */
-      checkbutton = gtk_check_button_new_with_label(_("Use J-OS (Not Japanese PalmOS:WorkPad/CLIE)"));
-      gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
-      get_pref(PREF_USE_JOS, &ivalue, &cstr);
-      gtk_widget_show(checkbutton);
-      if (ivalue) {
-	 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-      }
-      gtk_signal_connect(GTK_OBJECT(checkbutton), 
-			 "clicked", GTK_SIGNAL_FUNC(cb_use_jos), NULL);	   
-   }
-   
-#ifdef  ENABLE_PLUGINS
-   if (!skip_plugins) {
-
-      plugin_list=NULL;
-
-      plugin_list = get_plugin_list();
-
-      for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
-	 Pplugin = (struct plugin_s *)temp_list->data;
-	 if (Pplugin) {
-	    /* Make a checkbox for each plugin */
-	    g_snprintf(temp, sizeof(temp), "Sync %s (%s)", Pplugin->name, Pplugin->full_path);
-	    checkbutton = gtk_check_button_new_with_label(temp);
-	    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
-	    gtk_widget_show(checkbutton);
-	    if (Pplugin->sync_on) {
-	       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-	    }
-	    gtk_signal_connect(GTK_OBJECT(checkbutton), "clicked", 
-			       GTK_SIGNAL_FUNC(cb_sync_plugin),
-			       GINT_TO_POINTER(Pplugin->number));
-	 }
-      }
-   }
-
+   /* Use Manana check box */
+   add_checkbutton(_("Use Manana database"),
+		   PREF_MANANA_MODE, vbox_todo, cb_checkbox_set_pref);
 #endif
 
+   /* Default Number of Days Due for ToDos */
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_todo), hbox_temp, FALSE, FALSE, 0);
 
-   /* Alarms Preferences */
-   /* Show open alarm windows check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Open alarm windows for appointment reminders"));
-   gtk_box_pack_start(GTK_BOX(vbox_alarms), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_OPEN_ALARM_WINDOWS, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_open_alarm), NULL);
+   add_checkbutton(_("Use default number of days due"),
+		   PREF_TODO_DAYS_DUE, hbox_temp, cb_checkbox_set_pref);
+
+   todo_days_due_entry = gtk_entry_new_with_max_length(MAX_PREF_VALUE - 2);
+   get_pref(PREF_TODO_DAYS_TILL_DUE, &ivalue, &cstr);
+   temp[0]='\0';
+   g_snprintf(temp, sizeof(temp), "%ld", ivalue);
+      gtk_entry_set_text(GTK_ENTRY(todo_days_due_entry), temp);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), todo_days_due_entry, FALSE, FALSE, 0);
+
+   gtk_signal_connect(GTK_OBJECT(todo_days_due_entry),
+		      "changed", GTK_SIGNAL_FUNC(cb_checkbox_todo_days_till_due),
+		      NULL);
+
+   gtk_widget_show_all(hbox_temp);
+
+   /**********************************************************************/
+   /* Memo preference tab */
+
+   /* Memo32 check box */
+   add_checkbutton(_("Use Memo32 (pedit32)"),
+		   PREF_MEMO32_MODE, vbox_memo, cb_checkbox_set_pref);
+
+   /**********************************************************************/
+   /* Alarms preference tab */
 
    /* Show open alarm windows check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Execute this command"));
-   gtk_box_pack_start(GTK_BOX(vbox_alarms), checkbutton, FALSE, FALSE, 0);
-   get_pref(PREF_DO_ALARM_COMMAND, &ivalue, &cstr);
-   gtk_widget_show(checkbutton);
-   if (ivalue) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
-   }
-   gtk_signal_connect(GTK_OBJECT(checkbutton), 
-		      "clicked", GTK_SIGNAL_FUNC(cb_do_command), NULL);
+   add_checkbutton(_("Open alarm windows for appointment reminders"),
+		   PREF_OPEN_ALARM_WINDOWS, vbox_alarms, cb_checkbox_set_pref);
+
+   /* Show open alarm windows check box */
+   add_checkbutton(_("Execute this command"),
+		   PREF_DO_ALARM_COMMAND, vbox_alarms, cb_checkbox_set_pref);
 
    /* Shell warning */
    label = gtk_label_new(_("WARNING: executing arbitrary shell commands can be dangerous!!!"));
@@ -707,6 +668,9 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(alarm_command_entry), cstr);
    }
+   gtk_signal_connect(GTK_OBJECT(alarm_command_entry),
+		      "changed", GTK_SIGNAL_FUNC(cb_text_entry),
+		      GINT_TO_POINTER(PREF_ALARM_COMMAND));
    gtk_box_pack_start(GTK_BOX(hbox_temp), alarm_command_entry, FALSE, FALSE, 0);
 
    label = gtk_label_new(_("%t is replaced with the alarm time"));
@@ -733,6 +697,66 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    label = gtk_label_new(_("%N (note substitution) is disabled in this build"));
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
+#endif
+
+   /**********************************************************************/
+   /* Conduits preference tab */
+
+   /* Show sync datebook check box */
+   add_checkbutton(_("Sync datebook"),
+		   PREF_SYNC_DATEBOOK, vbox_conduits, cb_checkbox_set_pref);
+
+   /* Show sync address check box */
+   add_checkbutton(_("Sync address"),
+		   PREF_SYNC_ADDRESS, vbox_conduits, cb_checkbox_set_pref);
+
+   /* Show sync todo check box */
+   add_checkbutton(_("Sync todo"),
+		   PREF_SYNC_TODO, vbox_conduits, cb_checkbox_set_pref);
+
+   /* Show sync memo check box */
+   add_checkbutton(_("Sync memo"),
+		   PREF_SYNC_MEMO, vbox_conduits, cb_checkbox_set_pref);
+
+   /* Show sync Memo32 check box */
+   add_checkbutton(_("Sync memo32 (pedit32)"),
+		   PREF_SYNC_MEMO32, vbox_conduits, cb_checkbox_set_pref);
+
+#ifdef ENABLE_MANANA
+   /* Show sync Ma~nana check box */
+   add_checkbutton(_("Sync Manana"),
+		   PREF_SYNC_MANANA, vbox_conduits, cb_checkbox_set_pref);
+#endif
+   get_pref(PREF_CHAR_SET, &ivalue, &cstr);
+   if (ivalue == CHAR_SET_JAPANESE || ivalue == CHAR_SET_SJIS_UTF) {
+       /* Show use Japanese Kana extention check box */
+       add_checkbutton(_("Use J-OS (Not Japanese PalmOS:WorkPad/CLIE)"),
+		       PREF_USE_JOS, vbox_settings, cb_checkbox_set_pref);
+   }
+
+#ifdef  ENABLE_PLUGINS
+   if (!skip_plugins) {
+      plugin_list=NULL;
+      plugin_list = get_plugin_list();
+
+      for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
+	 Pplugin = (struct plugin_s *)temp_list->data;
+	 if (Pplugin) {
+	    /* Make a checkbox for each plugin */
+	    g_snprintf(temp, sizeof(temp), _("Sync %s (%s)"), Pplugin->name, Pplugin->full_path);
+	    checkbutton = gtk_check_button_new_with_label(temp);
+	    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
+	    gtk_widget_show(checkbutton);
+	    if (Pplugin->sync_on) {
+	       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+	    }
+	    gtk_signal_connect(GTK_OBJECT(checkbutton), "clicked",
+			       GTK_SIGNAL_FUNC(cb_sync_plugin),
+			       GINT_TO_POINTER(Pplugin->number));
+	 }
+      }
+   }
+
 #endif
 
    /* Create a "Done" button */
