@@ -1,4 +1,7 @@
-/* print.c
+/* $Id: print.c,v 1.31 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * print.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 2000-2002 by Judd Montgomery
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include <stdio.h>
@@ -32,6 +35,7 @@
 #include "print_logo.h"
 #include "print_headers.h"
 #include "log.h"
+#include "i18n.h"
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -44,7 +48,7 @@ void ps_strncat(char *dest, char *src, int n);
 
 char *PaperSizes[] = { "Letter", "Legal", "Statement", "Tabloid", "Ledger",
                        "Folio", "Quarto", "7x9", "9x11", "9x12", "10x13",
-                       "10x14", "Executive", "A0", "A1", "A2", "A3", "A4",  
+                       "10x14", "Executive", "A0", "A1", "A2", "A3", "A4",
                        "A5", "A6", "A7", "A8", "A9", "A10", "B0", "B1", "B2",
                        "B3", "B4", "B5", "B6", "B7", "B8", "B9", "B10",
                        "ISOB0", "ISOB1", "ISOB2", "ISOB3", "ISOB4", "ISOB5",
@@ -62,10 +66,9 @@ char *PaperSizes[] = { "Letter", "Legal", "Statement", "Tabloid", "Ledger",
 
 FILE *print_open()
 {
-   long ivalue;
    const char *command;
 
-   get_pref(PREF_PRINT_COMMAND, &ivalue, &command);
+   get_pref(PREF_PRINT_COMMAND, NULL, &command);
    if (command) {
       return popen(command, "w");
    } else {
@@ -82,21 +85,21 @@ int courier_12()
 {
    /* fprintf(out, "/Courier 12 selectfont\n"); */
    fprintf(out, "%cC12\n", FLAG_CHAR);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int courier_bold_12()
 {
    /* fprintf(out, "/Courier-Bold 12 selectfont\n"); */
    fprintf(out, "%cCB12\n", FLAG_CHAR);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int clip_to_box(float x1, float y1, float x2, float y2)
 {
    fprintf(out, "%g inch %g inch %g inch %g inch rectclip\n",
            x1, y1, x2 - x1, y2 - y1);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int puttext(float x, float y, char *text)
@@ -110,7 +113,7 @@ int puttext(float x, float y, char *text)
    ps_strncat(buf, text, 2 * len);
    fprintf(out, "%g inch %g inch moveto (%s) show\n", x, y, buf);
    free(buf);
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int header()
@@ -121,45 +124,30 @@ int header()
    fprintf(out,
 	   "%%!PS-Adobe-2.0\n"
 	   "%%%%Creator: J-Pilot\n"
-	   "%%%%CreationDate: %s\n"
+	   "%%%%CreationDate: %s"
            "%%%%DocumentData: Clean7Bit\n"
-	   /* XXX Title */
 	   "%%%%Orientation: Portrait\n"
-           /* XXX BoundingBox */
            "%%DocumentFonts: Times-Roman Times-Bold Courier Courier-Bold\n"
            "%%%%Magnification: 1.0000\n"
            "%%%%Pages: 1\n"
            "%%%%EndComments\n"
            "%%%%BeginProlog\n"
-           "%%%%BeginResource: procset\n"
-	   "/inch {72 mul} def\n"
-	   "/Recode {\n"
-	   "  exch\n"
-	   "  findfont\n"
-	   "  dup length dict\n"
-	   "  begin\n"
-	   "    { def } forall\n"
-	   "    /Encoding ISOLatin1Encoding def\n"
-	   "    currentdict\n"
-	   "  end\n"
-	   "  definefont pop\n"
-	   "} bind def\n\n"
-	   "/Times-Roman      /Times-Roman-ISOLatin1 Recode\n"
-	   "/Times-Bold       /Times-Bold-ISOLatin1 Recode\n"
-	   "/Courier          /Courier-ISOLatin1 Recode\n"
-	   "/Courier-Bold     /Courier-Bold-ISOLatin1 Recode\n"
-           "%%%%EndResource\n"
-           "%%%%EndProlog\n"
-           "%%%%Page: 1 1\n\n",
-	   ctime(&ltime));
-
+	   ,ctime(&ltime));
    fprintf(out, "/PageSize (%s) def\n\n", PaperSizes[PAPER_Letter]);
-   print_common_header(out);
+   print_common_prolog(out);
+   fprintf(out,
+           "%%%%EndProlog\n"
+           "%%%%BeginSetup\n");
+   print_common_setup(out);
    fprintf(out, "595 612 div 842 792 div Min dup scale %% HACK!!!! (CMB)\n");
            /* This hack pre-scales to compensate for the standard scaling
               mechanism below, to avoid me having to redo the layout of
               the dayview for the A4 standard size page. */
-   return 0;
+   fprintf(out,
+           "%%%%EndSetup\n"
+           "%%%%Page: 1 1\n\n");
+
+   return EXIT_SUCCESS;
 }
 
 int print_dayview(struct tm *date, AppointmentList *a_list)
@@ -169,22 +157,21 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
    time_t ltime;
    struct tm *now;
    const char *svalue;
-   long ivalue;
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_NUMERIC,"C");
+   char *current_locale;
+   current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
    header();
    /* Draw the 2 gray columns and header block */
    print_day_header(out);
 
-   /* Put the Month name up */
+   /* Put the month name up */
    fprintf(out, "/Times-Bold-ISOLatin1 findfont 20 scalefont setfont\n"
-           "newpath 0 setgray\n");
-   get_pref(PREF_LONGDATE, &ivalue, &svalue);
-   /* strftime(str, sizeof(str), "%B %d, %Y", date); */
-   strftime(str, sizeof(str), svalue, date);
+                "newpath 0 setgray\n");
+   get_pref(PREF_LONGDATE, NULL, &svalue);
+   strftime(str, sizeof(str), _(svalue), date);
    puttext(0.5, 10.25, str);
 
    /* Put the weekday name up */
@@ -194,32 +181,30 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
 
    /* Put the time of printing up */
    fprintf(out, "newpath\n"
-           "/Times-Roman-ISOLatin1 findfont 10 scalefont setfont\n");
+                "/Times-Roman-ISOLatin1 findfont 10 scalefont setfont\n");
 
    time(&ltime);
    now = localtime(&ltime);
-   get_pref(PREF_SHORTDATE, &ivalue, &svalue);
-   g_snprintf(datef, sizeof(date), "%s %s", "Printed on: ", svalue);
+   get_pref(PREF_SHORTDATE, NULL, &svalue);
+   g_snprintf(datef, sizeof(datef), "%s %s", "Printed on: ", svalue);
    strftime(str, sizeof(str), datef, now);
    puttext(0.5, 0.9, str);
    puttext(7.5, 0.9, "J-Pilot");
    fprintf(out, "stroke\n");
+
    print_logo(out, 40, 90, 0.35);
 
+   /* Put the appointments on the dayview calendar */
    fill_in(date, a_list);
 
    fprintf(out, "showpage\n");
-
-   fprintf(out,
-	   "%%%%PageTrailer\n"
-	   "%%%%Trailer\n"
-	   "%%%%EOF\n");
+   fprintf(out, "%%%%EOF\n");
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"");
+   setlocale(LC_ALL, current_locale);
 #endif
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int fill_in(struct tm *date, AppointmentList *a_list)
@@ -255,10 +240,10 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	 clip_to_box(5.0, 0.5, 8.0, 9.5);
       }
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-	 if (temp_al->ma.a.description == NULL) {
+	 if (temp_al->mappt.appt.description == NULL) {
 	    continue;
 	 }
-	 if (temp_al->ma.a.event) {
+	 if (temp_al->mappt.appt.event) {
 	    strcpy(str, " ");
 	    if (!am) {
 	       continue;
@@ -267,7 +252,7 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	    y=default_y - defaults1 * step;
 	    defaults1++;
 	 } else {
-	    hour24 = temp_al->ma.a.begin.tm_hour;
+	    hour24 = temp_al->mappt.appt.begin.tm_hour;
 	    if ((hour24 > 11) && (am)) {
 	       continue;
 	    }
@@ -276,7 +261,7 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	    }
 
 	    get_pref_time_no_secs(datef);
-	    strftime(str, sizeof(str), datef, &temp_al->ma.a.begin);
+	    strftime(str, sizeof(str), datef, &temp_al->mappt.appt.begin);
 
 	    if (hour24 > 11) {
 	       x=indent2;
@@ -296,14 +281,13 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	       }
 	    }
 	 }
-	 if (temp_al->ma.a.description) {
+	 if (temp_al->mappt.appt.description) {
 	    strcat(str, " ");
-	    strncat(str, temp_al->ma.a.description, sizeof(str)-strlen(str)-2);
+	    strncat(str, temp_al->mappt.appt.description, sizeof(str)-strlen(str)-2);
 	    str[128]='\0';
 	 }
 	 if (y > 1.0) {
 	    puttext(x, y, str);
-	    /* printf("[%s]\n", str);*/
 	 } else {
 	    jp_logf(JP_LOG_WARN, "Too many appointments, dropping one\n");
 	 }
@@ -311,16 +295,16 @@ int fill_in(struct tm *date, AppointmentList *a_list)
       fprintf(out, "grestore\n");
    }
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int print_days_appts(struct tm *date)
-{   
+{
    AppointmentList *a_list;
 
    out = print_open();
    if (!out) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
    a_list = NULL;
@@ -333,7 +317,7 @@ int print_days_appts(struct tm *date)
 
    free_AppointmentList(&a_list);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int f_indent_print(FILE *f, int indent, char *str) {
@@ -360,7 +344,7 @@ int f_indent_print(FILE *f, int indent, char *str) {
       }
       fprintf(f, "%c", *P);
    }
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*----------------------------------------------------------------------
@@ -408,7 +392,7 @@ int days_in_mon(struct tm *date)
  *			appointments.
  *----------------------------------------------------------------------*/
 
-char *MonthNames[] = { 
+char *MonthNames[] = {
    "January", "February", "March", "April", "May", "June", "July",
    "August", "September", "October", "November", "December"
 };
@@ -423,7 +407,6 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    int dow;
    int ndim;
    int n;
-   const char *svalue;
    long fdow;
    int mask;
 #ifdef ENABLE_DATEBK
@@ -433,6 +416,9 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    long use_db3_tags;
    struct db4_struct db4;
 #endif
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    /*------------------------------------------------------------------
     * Set up the PostScript output file, and print the header to it.
@@ -446,12 +432,32 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 #endif
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_NUMERIC, "C");
+   current_locale = setlocale(LC_NUMERIC,"C");
 #endif
-   if (! (out = print_open())) return(-1);
-   fprintf(out, "%%!PS-Adobe-2.0\n\n"
-	   "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
+   if (! (out = print_open())) return(EXIT_FAILURE);
+
+   fprintf(out,
+	   "%%!PS-Adobe-2.0\n"
+	   "%%%%Creator: J-Pilot\n"
+	   "%%%%CreationDate: %s"
+           "%%%%DocumentData: Clean7Bit\n"
+	   "%%%%Orientation: Landscape\n\n"
+           "%%DocumentFonts: Times-Roman Times-Bold Courier Courier-Bold\n"
+           "%%%%Magnification: 1.0000\n"
+           "%%%%Pages: 1\n"
+           "%%%%EndComments\n"
+           "%%%%BeginProlog\n"
+	   ,ctime(&ltime));
+   fprintf(out, "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
+   print_common_prolog(out);
+   fprintf(out,
+           "%%%%EndProlog\n"
+           "%%%%BeginSetup\n");
+   print_common_setup(out);
    print_month_header(out);
+   fprintf(out,
+           "%%%%EndSetup\n"
+           "%%%%Page: 1 1\n\n");
 
    /*------------------------------------------------------------------
     * Extract the appointments
@@ -474,7 +480,7 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    date.tm_isdst=-1;
    mktime(&date);
 
-   get_pref(PREF_FDOW, &fdow, &svalue);
+   get_pref(PREF_FDOW, &fdow, NULL);
 
    fprintf(out,
 	   "(%s, %d) %d (%s) (%s version %s) %ld InitialisePage\n\n",
@@ -499,7 +505,7 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
 #ifdef ENABLE_DATEBK
 	 if (use_db3_tags) {
-	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    ret = db3_parse_tag(temp_al->mappt.appt.note, &db3_type, &db4);
 	    /* jp_logf(JP_LOG_DEBUG, "category = 0x%x\n", db4.category); */
 	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) {
@@ -508,21 +514,21 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 	    }
 	 }
 #endif
-	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
+	 if (isApptOnDate(&(temp_al->mappt.appt), &date)) {
 	    char tmp[20];
 	    char datef1[20];
 	    char datef2[20];
 	    tmp[0]='\0';
-	    if ( ! temp_al->ma.a.event) {
+	    if ( ! temp_al->mappt.appt.event) {
 	       get_pref_time_no_secs(datef1);
 	       g_snprintf(datef2, sizeof(datef2), "(%s )", datef1);
-	       strftime(tmp, sizeof(tmp), datef2, &(temp_al->ma.a.begin));
+	       strftime(tmp, sizeof(tmp), datef2, &(temp_al->mappt.appt.begin));
 	       tmp[19]='\0';
 	    }
 	    desc[0]='\0';
-	    if (temp_al->ma.a.description) {
-	       ps_strncat(desc, temp_al->ma.a.description, 40);
-	       desc[40]='\0';
+	    if (temp_al->mappt.appt.description) {
+	       ps_strncat(desc, temp_al->mappt.appt.description, 100);
+	       desc[100]='\0';
 	    }
 	    remove_cr_lfs(desc);
 	    fprintf(out, "%s (%s) %simedItem\n", tmp, desc,
@@ -555,13 +561,14 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    fprintf(out, "grestore\n");
    print_logo(out, 20, 30, 0.35);
    fprintf(out, "\nshowpage\n");
+   fprintf(out, "%%%%EOF\n");
 
    print_close(out);
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL, "");
+   setlocale(LC_NUMERIC, current_locale);
 #endif
-     return(0);
+   return EXIT_SUCCESS;
 }
 
 /*----------------------------------------------------------------------
@@ -583,7 +590,7 @@ void reset_first_last(void)
 void check_first_last(AppointmentList *al)
 {
    struct tm *ApptTime;
-   ApptTime = &(al->ma.a.begin);
+   ApptTime = &(al->mappt.appt.begin);
    if (ApptTime->tm_hour == first_hour) {
       if (ApptTime->tm_min < first_min) first_min = ApptTime->tm_min;
    }
@@ -592,7 +599,7 @@ void check_first_last(AppointmentList *al)
       first_min  = ApptTime->tm_min;
    }
 
-   ApptTime = &(al->ma.a.end);
+   ApptTime = &(al->mappt.appt.end);
    if (ApptTime->tm_hour == last_hour) {
       if (ApptTime->tm_min > last_min) last_min = ApptTime->tm_min;
    } else if (ApptTime->tm_hour > last_hour) {
@@ -622,33 +629,53 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    long use_db3_tags;
    struct db4_struct db4;
 #endif
-
-   /*------------------------------------------------------------------
-    * Set up the PostScript output file, and print the header to it.
-    *------------------------------------------------------------------*/
-   time(&ltime);
-   today_date = localtime(&ltime);
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
 #ifdef ENABLE_DATEBK
    get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
 #endif
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_NUMERIC, "C");
+   current_locale = setlocale(LC_NUMERIC,"C");
 #endif
-     
-   if (! (out = print_open())) return(-1);
+
+   /*------------------------------------------------------------------
+    * Set up the PostScript output file, and print the header to it.
+    *------------------------------------------------------------------*/
+   if (! (out = print_open())) return(EXIT_FAILURE);
+
+   time(&ltime);
+   fprintf(out,
+	   "%%!PS-Adobe-2.0\n"
+	   "%%%%Creator: J-Pilot\n"
+	   "%%%%CreationDate: %s"
+           "%%%%DocumentData: Clean7Bit\n"
+	   "%%%%Orientation: Landscape\n"
+           "%%DocumentFonts: Times-Roman Times-Bold Courier Courier-Bold\n"
+           "%%%%Magnification: 1.0000\n"
+           "%%%%Pages: 1\n"
+           "%%%%EndComments\n"
+           "%%%%BeginProlog\n"
+	   ,ctime(&ltime));
    /*------------------------------------------------------------------
     * These are preferences for page size (passed in), first and last
     * hours on the plan (default; scales if earlier or later are present),
     * and whether to print dashes across the page.
     *------------------------------------------------------------------*/
-   fprintf(out, "%%!PS-Adobe-2.0\n\n"
-	   "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
+   fprintf(out, "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
    fprintf(out, "/FirstHour  9 def\n"
-	   "/LastHour  22 def\n");
+	        "/LastHour  22 def\n");
    fprintf(out, "/Dashes true def\n");
-
+   print_common_prolog(out);
+   fprintf(out,
+           "%%%%EndProlog\n"
+           "%%%%BeginSetup\n");
+   print_common_setup(out);
    print_week_header(out);
+   fprintf(out,
+           "%%%%EndSetup\n"
+           "%%%%Page: 1 1\n\n");
 
    /*------------------------------------------------------------------
     * Run through the appointments, looking for earliest and latest
@@ -662,13 +689,13 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
 #ifdef ENABLE_DATEBK
 	 if (use_db3_tags) {
-	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    ret = db3_parse_tag(temp_al->mappt.appt.note, &db3_type, &db4);
 	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) continue;
 	 }
 #endif
-	 if (isApptOnDate(&(temp_al->ma.a), &date))
-	   if ( ! temp_al->ma.a.event)
+	 if (isApptOnDate(&(temp_al->mappt.appt), &date))
+	   if ( ! temp_al->mappt.appt.event)
 	     check_first_last(temp_al);
       }
    }
@@ -679,6 +706,7 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
     * the printing process
     *------------------------------------------------------------------*/
 
+   today_date = localtime(&ltime);
    fprintf(out,
 	   "%%------------------------------------------------------------\n"
 	   "%% This is today's date, the date of printing, plus the hour\n"
@@ -708,7 +736,7 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
 #ifdef ENABLE_DATEBK
 	 if (use_db3_tags) {
-	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    ret = db3_parse_tag(temp_al->mappt.appt.note, &db3_type, &db4);
 	    jp_logf(JP_LOG_DEBUG, "category = 0x%x\n", db4.category);
 	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) {
@@ -717,29 +745,29 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
 	    }
 	 }
 #endif
-	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
+	 if (isApptOnDate(&(temp_al->mappt.appt), &date)) {
 	    memset(desc, 0, sizeof(desc));
 	    memset(short_date, 0, sizeof(short_date));
 
-	    if ( ! temp_al->ma.a.event)
+	    if ( ! temp_al->mappt.appt.event)
 	      {
 		 char t1[6], t2[6], ht[3], mt[3];
 		 int j, m;
 
-		 strftime(ht, sizeof(ht), "%H", &(temp_al->ma.a.begin));
-		 strftime(mt, sizeof(mt), "%M", &(temp_al->ma.a.begin));
+		 strftime(ht, sizeof(ht), "%H", &(temp_al->mappt.appt.begin));
+		 strftime(mt, sizeof(mt), "%M", &(temp_al->mappt.appt.begin));
 		 m = atoi(mt);
 		 snprintf(t1, sizeof(t1), "%s.%02d", ht, (int)((m * 100.)/60));
 
-		 strftime(ht, sizeof(ht), "%H", &(temp_al->ma.a.end));
-		 strftime(mt, sizeof(mt), "%M", &(temp_al->ma.a.end));
+		 strftime(ht, sizeof(ht), "%H", &(temp_al->mappt.appt.end));
+		 strftime(mt, sizeof(mt), "%M", &(temp_al->mappt.appt.end));
 		 m = atoi(mt);
 		 snprintf(t2, sizeof(t2), "%s.%02d", ht, (int)((m * 100.)/60));
 		 sprintf(short_date, "%s %s ", t1, t2);
 		 for (j=0; j<30;j++) short_date[j] =tolower(short_date[j]);
 	      }
-	    if (temp_al->ma.a.description) {
-	       ps_strncat(desc, temp_al->ma.a.description, 250);
+	    if (temp_al->mappt.appt.description) {
+	       ps_strncat(desc, temp_al->mappt.appt.description, 250);
 	       remove_cr_lfs(desc);
 	    }
 	    fprintf(out, "%s (%s) itemline\n", short_date, desc);
@@ -748,32 +776,32 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    }
    free_AppointmentList(&a_list);
    fprintf(out, "\nfinishprinting\n");
+   fprintf(out, "%%%%EOF\n");
    print_close(out);
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL, "");
+   setlocale(LC_ALL, current_locale);
 #endif
-     return(0);
+     return EXIT_SUCCESS;
 }
 
 /*----------------------------------------------------------------------
- * 
+ *
  * Address code
- * 
+ *
  *----------------------------------------------------------------------*/
 
 int print_address_header()
 {
    time_t ltime;
    struct tm *date;
-   long ivalue;
    const char *svalue;
    char str[256];
 
    time(&ltime);
    date = localtime(&ltime);
 
-   get_pref(PREF_SHORTDATE, &ivalue, &svalue);
+   get_pref(PREF_SHORTDATE, NULL, &svalue);
    strftime(str, sizeof(str), svalue, date);
 
    fprintf(out,
@@ -852,7 +880,7 @@ int print_address_header()
 	   "findfont\n"
 	   "dup length dict\n"
 	   "begin\n"
-	   "{ def\n" 
+	   "{ def\n"
 	   "} forall\n"
 	   "/Encoding ISOLatin1Encoding def\n"
 	   "currentdict\n"
@@ -889,7 +917,7 @@ int print_address_header()
 	   "} bind def\n"
 	   "%%\n"
 	   "%%\n");
-   fprintf(out, 
+   fprintf(out,
 	   "/setup\n"
 	   "{\n"
 	   "/Times-Roman-ISOLatin1 10 selectfont\n"
@@ -979,7 +1007,7 @@ int print_address_header()
 	   "0 %%The page number minus 1\n"
 	   "setup printit\n"
 	   );
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int print_addresses(AddressList *address_list)
@@ -988,22 +1016,25 @@ int print_addresses(AddressList *address_list)
    long lines_between_recs;
    AddressList *temp_al;
    struct AddressAppInfo address_app_info;
-   struct Address *a;
+   struct Address *addr;
    int show1, show2, show3;
    int j, i, i2;
    int len;
-   /*This is because the palm doesn\'t show the address entries in order */
+   /* This is because the palm doesn't show the address entries in order */
    int order[22]={0,1,13,2,3,4,5,6,7,8,9,10,11,12,14,15,16,17,18,19,20,21
    };
    char str[100];
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    out = print_open();
    if (!out) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_NUMERIC,"C");
+   current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
    get_address_app_info(&address_app_info);
@@ -1029,18 +1060,18 @@ int print_addresses(AddressList *address_list)
       fprintf(out, "%cHLINE\n", FLAG_CHAR);
 
       str[0]='\0';
-      if (temp_al->ma.a.entry[show1] || temp_al->ma.a.entry[show2]) {
-	 if (temp_al->ma.a.entry[show1] && temp_al->ma.a.entry[show2]) {
-	    g_snprintf(str, sizeof(str), "%s, %s", temp_al->ma.a.entry[show1], temp_al->ma.a.entry[show2]);
+      if (temp_al->maddr.addr.entry[show1] || temp_al->maddr.addr.entry[show2]) {
+	 if (temp_al->maddr.addr.entry[show1] && temp_al->maddr.addr.entry[show2]) {
+	    g_snprintf(str, sizeof(str), "%s, %s", temp_al->maddr.addr.entry[show1], temp_al->maddr.addr.entry[show2]);
 	 }
-	 if (temp_al->ma.a.entry[show1] && ! temp_al->ma.a.entry[show2]) {
-	    strncpy(str, temp_al->ma.a.entry[show1], 48);
+	 if (temp_al->maddr.addr.entry[show1] && ! temp_al->maddr.addr.entry[show2]) {
+	    strncpy(str, temp_al->maddr.addr.entry[show1], 48);
 	 }
-	 if (! temp_al->ma.a.entry[show1] && temp_al->ma.a.entry[show2]) {
-	    strncpy(str, temp_al->ma.a.entry[show2], 48);
+	 if (! temp_al->maddr.addr.entry[show1] && temp_al->maddr.addr.entry[show2]) {
+	    strncpy(str, temp_al->maddr.addr.entry[show2], 48);
 	 }
-      } else if (temp_al->ma.a.entry[show3]) {
-	    strncpy(str, temp_al->ma.a.entry[show3], 48);
+      } else if (temp_al->maddr.addr.entry[show3]) {
+	    strncpy(str, temp_al->maddr.addr.entry[show3], 48);
       } else {
 	    strcpy(str, "-Unnamed-");
       }
@@ -1049,13 +1080,13 @@ int print_addresses(AddressList *address_list)
       fprintf(out, "%s\n", str);
       courier_12();
 
-      a = &(temp_al->ma.a);
+      addr = &(temp_al->maddr.addr);
       for (i=0; i<NUM_ADDRESS_ENTRIES; i++) {
 	 i2=order[i];
-	 if (a->entry[i2]) {
+	 if (addr->entry[i2]) {
 	    if (i2>2 && i2<8) {
-	       fprintf(out, "%s: ", address_app_info.phoneLabels[a->phoneLabel[i2-3]]);
-	       len = strlen(address_app_info.phoneLabels[a->phoneLabel[i2-3]]);
+	       fprintf(out, "%s: ", address_app_info.phoneLabels[addr->phoneLabel[i2-3]]);
+	       len = strlen(address_app_info.phoneLabels[addr->phoneLabel[i2-3]]);
 	    } else {
 	       fprintf(out, "%s: ", address_app_info.labels[i2]);
 	       len = strlen(address_app_info.labels[i2]);
@@ -1063,7 +1094,7 @@ int print_addresses(AddressList *address_list)
 	    for (j=16-len; j; j--) {
 	       fprintf(out, " ");
 	    }
-	    f_indent_print(out, 18, a->entry[i2]);
+	    f_indent_print(out, 18, addr->entry[i2]);
 	    fprintf(out, "\n");
 	 }
       }
@@ -1079,22 +1110,21 @@ int print_addresses(AddressList *address_list)
    print_close(out);
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"");
+   setlocale(LC_ALL, current_locale);
 #endif
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
- * 
+ *
  * ToDo code
- * 
+ *
  */
 int print_todos(ToDoList *todo_list, char *category_name)
 {
    long one_rec_per_page;
    long lines_between_recs;
-   long ivalue;
    ToDoList *temp_l;
    struct ToDo *todo;
    int indent;
@@ -1102,26 +1132,30 @@ int print_todos(ToDoList *todo_list, char *category_name)
    char str[100];
    time_t ltime;
    struct tm *now;
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    out = print_open();
    if (!out) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_NUMERIC,"C");
+   current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
    fprintf(out, "%%!PS-Adobe-2.0\n\n"
            "/PageSize (%s) def\n\n", PaperSizes[PAPER_Letter]);
-   print_common_header(out);
+   print_common_prolog(out);
+   print_common_setup(out);
    fprintf(out, "/CategoryName (%s) def\n", category_name);
    print_todo_header(out);
 
    get_pref(PREF_PRINT_ONE_PER_PAGE, &one_rec_per_page, NULL);
    get_pref(PREF_NUM_BLANK_LINES, &lines_between_recs, NULL);
 
-   get_pref(PREF_SHORTDATE, &ivalue, &datef);
+   get_pref(PREF_SHORTDATE, NULL, &datef);
    now = localtime(&ltime);
    strftime(str, sizeof(str), datef, now);
    indent=strlen(str) + 8;
@@ -1142,36 +1176,54 @@ int print_todos(ToDoList *todo_list, char *category_name)
       fprintf(out, "(%s) ", str);
 
       if (todo->description) {
-	 fprintf(out, "(%s) ", todo->description);
+	 int len;
+	 char *buf;
+
+	 len = strlen(todo->description);
+	 buf = (char *) malloc(2 * len + 1);
+	 memset(buf, 0, 2 * len + 1);
+	 ps_strncat(buf, todo->description, 2 * len);
+
+	 fprintf(out, "(%s) ", buf);
+	 free(buf);
       } else {
          fprintf(out, "() ");
       }
 
       if ((todo->note) && todo->note[0]) {
-	 fprintf(out, "(%s)", todo->note);
+	 int len;
+	 char *buf;
+
+	 len = strlen(todo->note);
+	 buf = (char *) malloc(2 * len + 1);
+	 memset(buf, 0, 2 * len + 1);
+	 ps_strncat(buf, todo->note, 2 * len);
+
+	 fprintf(out, "(%s) ", buf);
+	 free(buf);
       } else {
          fprintf(out, "()");
       }
       fprintf(out, " Todo\n");
 
-      if (one_rec_per_page) {	
-	 fprintf(out, "NewPage\n");	
-      }	
+      if (one_rec_per_page) {
+	 fprintf(out, "NewPage\n");
+      }
    }
    fprintf(out, "showpage\n");
 
    print_close(out);
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"");
+   setlocale(LC_ALL, current_locale);
 #endif
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 /*
- * 
+ *
  * Memo code
- * 
+ *
  */
 int print_memos(MemoList *memo_list)
 {
@@ -1180,14 +1232,17 @@ int print_memos(MemoList *memo_list)
    MemoList *temp_l;
    struct Memo *memo;
    int i;
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    out = print_open();
    if (!out) {
-      return -1;
+      return EXIT_FAILURE;
    }
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_NUMERIC,"C");
+   current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
    print_address_header();
@@ -1218,10 +1273,10 @@ int print_memos(MemoList *memo_list)
    fprintf(out, "%cEND\n", FLAG_CHAR);
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"");
+   setlocale(LC_ALL, current_locale);
 #endif
 
    print_close(out);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
