@@ -1,4 +1,7 @@
-/* print_gui.c
+/* $Id: print_gui.c,v 1.15 2004/12/07 06:51:08 rikster5 Exp $ */
+
+/*******************************************************************************
+ * print_gui.c
  * A module of J-Pilot http://jpilot.org
  *
  * Copyright (C) 2000-2002 by Judd Montgomery
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -153,13 +156,13 @@ int print_gui(GtkWidget *main_window, int app, int date_button, int mon_week_day
    char temp_str[10];
    char temp[256];
    const char *svalue;
-   GSList *group;   
+   GSList *group;
 
    jp_logf(JP_LOG_DEBUG, "print_gui\n");
    if (GTK_IS_WINDOW(window)) {
       jp_logf(JP_LOG_DEBUG, "print_gui window is already up\n");
       gdk_window_raise(window->window);
-      return 0;
+      return EXIT_SUCCESS;
    }
    print_dialog=0;
    radio_button_one=NULL;
@@ -318,7 +321,7 @@ int print_gui(GtkWidget *main_window, int app, int date_button, int mon_week_day
    print_command_entry = gtk_entry_new_with_max_length(250);
    gtk_box_pack_start(GTK_BOX(vbox), print_command_entry, FALSE, FALSE, 0);
 
-   get_pref(PREF_PRINT_COMMAND, &ivalue, &svalue);
+   get_pref(PREF_PRINT_COMMAND, NULL, &svalue);
    gtk_entry_set_text(GTK_ENTRY(print_command_entry), svalue);
 
 
