@@ -1,6 +1,9 @@
-/* todo.c
+/* $Id: todo.c,v 1.39 2005/09/26 03:27:59 judd Exp $ */
+
+/*******************************************************************************
+ * todo.c
  * A module of J-Pilot http://jpilot.org
- * 
+ *
  * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,7 +18,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
 
 #include "config.h"
 #include "i18n.h"
@@ -190,7 +193,7 @@ int todo_sort(ToDoList **todol, int sort_order)
 
    if (count<2) {
       /* We don't have to sort less than 2 items */
-      return 0;
+      return EXIT_SUCCESS;
    }
 
    get_todo_app_info(&ai);
@@ -200,8 +203,8 @@ int todo_sort(ToDoList **todol, int sort_order)
    /* Allocate an array to be qsorted */
    sort_todol = calloc(count, sizeof(ToDoList *));
    if (!sort_todol) {
-      jp_logf(JP_LOG_WARN, "todo_sort(): Out of Memory\n");
-      return 0;
+      jp_logf(JP_LOG_WARN, "todo_sort(): %s\n", _("Out of memory"));
+      return EXIT_FAILURE;
    }
 
    /* Set our array to be a list of pointers to the nodes in the linked list */
@@ -230,7 +233,7 @@ int todo_sort(ToDoList **todol, int sort_order)
 
    free(sort_todol);
 
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 /*
@@ -242,14 +245,14 @@ void pc_todo_validate_correct(struct ToDo *todo)
    if (todo->description) {
       if ((strlen(todo->description)+1 > MAX_TODO_DESC_LEN)) {
 	 jp_logf(JP_LOG_WARN, "%s\n", todo->description);
-	 jp_logf(JP_LOG_WARN, _("Warning ToDo description too long, truncating to %d\n"), MAX_TODO_DESC_LEN-1);
+	 jp_logf(JP_LOG_WARN, _("ToDo description text > %d, truncating to %d\n"), MAX_TODO_DESC_LEN, MAX_TODO_DESC_LEN-1);
 	 todo->description[MAX_TODO_DESC_LEN-1]='\0';
       }
    }
    if (todo->note) {
       if ((strlen(todo->note)+1 > MAX_TODO_NOTE_LEN)) {
 	 jp_logf(JP_LOG_WARN, "%s\n", todo->note);
-	 jp_logf(JP_LOG_WARN, _("Warning ToDo note too long, truncating to %d\n"), MAX_TODO_NOTE_LEN-1);
+	 jp_logf(JP_LOG_WARN, _("ToDo note text > %d, truncating to %d\n"), MAX_TODO_NOTE_LEN, MAX_TODO_NOTE_LEN-1);
 	 todo->note[MAX_TODO_NOTE_LEN-1]='\0';
       }
    }
@@ -258,8 +261,12 @@ void pc_todo_validate_correct(struct ToDo *todo)
 int pc_todo_write(struct ToDo *todo, PCRecType rt, unsigned char attrib,
 		  unsigned int *unique_id)
 {
+#ifndef PILOT_LINK_0_12
    unsigned char record[65536];
    int rec_len;
+#else /* PILOT_LINK_0_12 */
+   pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
    buf_rec br;
    long char_set;
 #ifdef ENABLE_MANANA
@@ -269,24 +276,39 @@ int pc_todo_write(struct ToDo *todo, PCRecType rt, unsigned char attrib,
    get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set != CHAR_SET_LATIN1) {
       if (todo->description) {
-	 charset_j2p((unsigned char *)todo->description, strlen(todo->description)+1, char_set);
+	 charset_j2p(todo->description, strlen(todo->description)+1, char_set);
       }
       if (todo->note) {
-	 charset_j2p((unsigned char *)todo->note, strlen(todo->note)+1, char_set);
+	 charset_j2p(todo->note, strlen(todo->note)+1, char_set);
       }
    }
 
    pc_todo_validate_correct(todo);
+#ifndef PILOT_LINK_0_12
    rec_len = pack_ToDo(todo, record, 65535);
    if (!rec_len) {
       PRINT_FILE_LINE;
       jp_logf(JP_LOG_WARN, "pack_ToDo %s\n", _("error"));
-      return -1;
+      return EXIT_FAILURE;
    }
+#else /* PILOT_LINK_0_12 */
+   RecordBuffer = pi_buffer_new(0);
+   if (pack_ToDo(todo, RecordBuffer, todo_v1) == -1) {
+      PRINT_FILE_LINE;
+      jp_logf(JP_LOG_WARN, "pack_ToDo %s\n", _("error"));
+      pi_buffer_free(RecordBuffer);
+      return EXIT_FAILURE;
+   }
+#endif /* PILOT_LINK_0_12 */
    br.rt=rt;
    br.attrib = attrib;
+#ifndef PILOT_LINK_0_12
    br.buf = record;
    br.size = rec_len;
+#else /* PILOT_LINK_0_12 */
+   br.buf = RecordBuffer->data;
+   br.size = RecordBuffer->used;
+#endif /* PILOT_LINK_0_12 */
    /* Keep unique ID intact */
    if (unique_id) {
       br.unique_id = *unique_id;
@@ -308,7 +330,11 @@ int pc_todo_write(struct ToDo *todo, PCRecType rt, unsigned char attrib,
       *unique_id = br.unique_id;
    }
 
-   return 0;
+#ifdef PILOT_LINK_0_12
+   pi_buffer_free(RecordBuffer);
+#endif
+
+   return EXIT_SUCCESS;
 }
 
 void free_ToDoList(ToDoList **todo)
@@ -325,10 +351,9 @@ void free_ToDoList(ToDoList **todo)
 
 int get_todo_app_info(struct ToDoAppInfo *ai)
 {
-   int num, i, r;
+   int num, r;
    int rec_size;
    unsigned char *buf;
-   long char_set;
 #ifdef ENABLE_MANANA
    long ivalue;
 #endif
@@ -340,7 +365,7 @@ int get_todo_app_info(struct ToDoAppInfo *ai)
    strcpy(ai->category.name[0], "Unfiled");
 
 #ifdef ENABLE_MANANA
-   get_pref(PREF_MANANA_MODE, &ivalue, NULL);                                  
+   get_pref(PREF_MANANA_MODE, &ivalue, NULL);
    if (ivalue) {
       strcpy(DBname, "MaanaDB");
    } else {
@@ -356,34 +381,25 @@ int get_todo_app_info(struct ToDoAppInfo *ai)
       if (buf) {
 	 free(buf);
       }
-      return -1;
+      return EXIT_FAILURE;
    }
    num = unpack_ToDoAppInfo(ai, buf, rec_size);
    if (buf) {
       free(buf);
    }
    if (num <= 0) {
-      jp_logf(JP_LOG_WARN, _("%s:%d Error reading %s\n"), __FILE__, __LINE__, DBname);
-      return -1;
+      jp_logf(JP_LOG_WARN, _("%s:%d Error reading file: %s\n"), __FILE__, __LINE__, DBname);
+      return EXIT_FAILURE;
    }
 
-   get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_LATIN1) {
-      for (i = 0; i < 16; i++) {
-	 if (ai->category.name[i][0] != '\0') {
-	    charset_p2j((unsigned char *)ai->category.name[i], 16, char_set);
-	 }
-      }
-   }
-
-   return 0;
+   return EXIT_SUCCESS;
 }
 
 int get_todos(ToDoList **todo_list, int sort_order)
 {
    return get_todos2(todo_list, sort_order, 1, 1, 1, 1, CATEGORY_ALL);
 }
-/* 
+/*
  * sort_order: 0=descending,  1=ascending
  * modified, deleted and private, completed:
  *  0 for no, 1 for yes, 2 for use prefs
@@ -402,6 +418,9 @@ int get_todos2(ToDoList **todo_list, int sort_order,
    buf_rec *br;
    long char_set;
    char *buf;
+#ifdef PILOT_LINK_0_12
+   pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
 #ifdef ENABLE_MANANA
    long ivalue;
 #endif
@@ -423,7 +442,7 @@ int get_todos2(ToDoList **todo_list, int sort_order,
       keep_priv = privates;
    }
    if (completed==2) {
-      get_pref(PREF_HIDE_COMPLETED, &hide_completed, NULL);
+      get_pref(PREF_TODO_HIDE_COMPLETED, &hide_completed, NULL);
    } else {
       hide_completed = !completed;
    }
@@ -435,11 +454,17 @@ int get_todos2(ToDoList **todo_list, int sort_order,
    get_pref(PREF_MANANA_MODE, &ivalue, NULL);
    if (ivalue) {
       num = jp_read_DB_files("MaanaDB", &records);
+      if (-1 == num)
+        return 0;
    } else {
       num = jp_read_DB_files("ToDoDB", &records);
+      if (-1 == num)
+        return 0;
    }
 #else
    num = jp_read_DB_files("ToDoDB", &records);
+   if (-1 == num)
+      return 0;
 #endif
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
@@ -455,20 +480,35 @@ int get_todos2(ToDoList **todo_list, int sort_order,
 	 continue;
       }
 
-      if ( ((br->rt==DELETED_PALM_REC) && (!keep_deleted)) ||
-	  ((br->rt==MODIFIED_PALM_REC) && (!keep_modified)) ) {
+      if ( ((br->rt==DELETED_PALM_REC)  && (!keep_deleted)) ||
+	   ((br->rt==DELETED_PC_REC)    && (!keep_deleted)) ||
+	   ((br->rt==MODIFIED_PALM_REC) && (!keep_modified)) ) {
 	 continue;
       }
-      if ((keep_priv != SHOW_PRIVATES) && 
+      if ((keep_priv != SHOW_PRIVATES) &&
 	  (br->attrib & dlpRecAttrSecret)) {
 	 continue;
       }
 
+#ifndef PILOT_LINK_0_12
       num = unpack_ToDo(&todo, br->buf, br->size);
+#else /* PILOT_LINK_0_12 */
+      RecordBuffer = pi_buffer_new(br->size);
+      memcpy(RecordBuffer->data, br->buf, br->size);
+      RecordBuffer->used = br->size;
+#endif /* PILOT_LINK_0_12 */
 
+#ifndef PILOT_LINK_0_12
       if (num <= 0) {
 	 continue;
       }
+#else /* PILOT_LINK_0_12 */
+      if (unpack_ToDo(&todo, RecordBuffer, todo_v1) == -1) {
+	 pi_buffer_free(RecordBuffer);
+	 continue;
+      }
+      pi_buffer_free(RecordBuffer);
+#endif /* PILOT_LINK_0_12 */
 
       if ( ((br->attrib & 0x0F) != category) && category != CATEGORY_ALL) {
 	 continue;
@@ -480,7 +520,7 @@ int get_todos2(ToDoList **todo_list, int sort_order,
 
       get_pref(PREF_CHAR_SET, &char_set, NULL);
       if (todo.description) {
-         buf = charset_p2newj((unsigned char *)todo.description, strlen(todo.description)+1, char_set);
+         buf = charset_p2newj(todo.description, strlen(todo.description)+1, char_set);
          if (buf) {
 	    /* JPA use new conversion routines
 	     if ((buf = (char *)malloc(strlen(todo.description)*2+1)) != NULL) {
@@ -492,7 +532,7 @@ int get_todos2(ToDoList **todo_list, int sort_order,
 	 }
       }
       if (todo.note) {
-	 buf = charset_p2newj((unsigned char *)todo.note, strlen(todo.note)+1, char_set);
+	 buf = charset_p2newj(todo.note, strlen(todo.note)+1, char_set);
          if (buf) {
 	    /* JPA use new conversion routines
 	     if ((buf = (char *)malloc(strlen(todo.note)*2+1)) != NULL) {
@@ -505,7 +545,7 @@ int get_todos2(ToDoList **todo_list, int sort_order,
       }
       temp_todo_list = malloc(sizeof(ToDoList));
       if (!temp_todo_list) {
-	 jp_logf(JP_LOG_WARN, "get_todos2(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "get_todos2(): %s\n", _("Out of memory"));
 	 break;
       }
       memcpy(&(temp_todo_list->mtodo.todo), &todo, sizeof(struct ToDo));
