@@ -1,4 +1,7 @@
-/* utils.h
+/* $Id: utils.h,v 1.52 2005/10/24 19:15:41 judd Exp $ */
+
+/*******************************************************************************
+ * utils.h
  * A module of J-Pilot http://jpilot.org
  * 
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #ifndef __UTILS_H__
 #define __UTILS_H__
 
@@ -33,7 +37,6 @@
 #include "libplugin.h"
 #include "japanese.h"
 #include "cp1250.h"
-#include "cp1253.h"
 #include "russian.h"
 
 #define PRINT_FILE_LINE printf("%s line %d\n", __FILE__, __LINE__)
@@ -52,6 +55,15 @@
 
 #define	FPI_STRING	"-//Judd Montgomery//NONSGML "PN" "VERSION"//EN"
 
+/* GTK 1 seems to have a problem with reading and setting the pane
+ * differing by 2 pixels.  This may not be true on every system.
+ */
+#ifdef ENABLE_GTK2
+# define PANE_CREEP 0
+#else
+# define PANE_CREEP 2
+#endif
+
 /*This is how often the clock updates in milliseconds */
 #define CLOCK_TICK 1000
 
@@ -131,6 +143,11 @@
 #define DAT_TODO_FILE        12
 #define DAT_MEMO_FILE        13
 
+/* Pilot-link 0.12 is broken and missing pi_uid_t */
+#ifdef PILOT_LINK_0_12
+typedef recordid_t pi_uid_t;
+#endif
+
 extern unsigned int glob_find_id;
 
 typedef enum {
@@ -145,26 +162,26 @@ typedef struct {
    PCRecType rt;
    unsigned int unique_id;
    unsigned char attrib;
-   struct Appointment a;
+   struct Appointment appt;
 } MyAppointment;
 
 typedef struct AppointmentList_s {
    AppType app_type;
    struct AppointmentList_s *next;
-   MyAppointment ma;
+   MyAppointment mappt;
 } AppointmentList;
 
 typedef struct {
    PCRecType rt;
    unsigned int unique_id;
    unsigned char attrib;
-   struct Address a;
+   struct Address addr;
 } MyAddress;
 
 typedef struct AddressList_s {
    AppType app_type;
    struct AddressList_s *next;
-   MyAddress ma;
+   MyAddress maddr;
 } AddressList;
 
 typedef struct {
@@ -203,10 +220,15 @@ struct search_record
 
 struct sorted_cats
 {
-   char *Pcat;
+   char Pcat[32];
    int cat_num;
 };
 
+/*
+ * Returns usage string that needs to be freed by the caller
+ */
+void fprint_usage_string(FILE *out);
+
 int cat_compare(const void *v1, const void *v2);
 
 gint timeout_date(gpointer data);
@@ -240,11 +262,10 @@ int find_next_offset(mem_rec_header *mem_rh, long fpos,
 
 /*The VP is a pointer to MyAddress, MyAppointment, etc. */
 int delete_pc_record(AppType app_type, void *VP, int flag);
+int undelete_pc_record(AppType app_type, void *VP, int flag);
 
 void get_month_info(int month, int day, int year, int *dow, int *ndim);
 
-void get_this_month_info(int *dow, int *ndim);
-
 time_t pilot_time_to_unix_time (unsigned long raw_time);
 
 unsigned long unix_time_to_pilot_time (time_t t);
@@ -306,10 +327,13 @@ void set_bg_rgb_clist_row(GtkWidget *clist, int row, int r, int g, int b);
 
 void set_fg_rgb_clist_cell(GtkWidget *clist, int row, int col, int r, int g, int b);
 
+void clist_select_row(GtkCList *clist, 
+                      int       row,
+		      int       column);
+
 int clist_find_id(GtkWidget *clist,
 		  unsigned int unique_id,
-		  int *found_at,
-		  int *total_count);
+		  int *found_at);
 
 int check_copy_DBs_to_home();
 
@@ -361,6 +385,8 @@ int address_gui_cleanup();
 int todo_gui_cleanup();
 int memo_gui_cleanup();
 
+void datebook_gui_setdate(int year, int month, int day);
+
 /*
  * Copy src string into dest while escaping quotes with double quotes.
  * dest could be as long as strlen(src)*2.
@@ -369,13 +395,7 @@ int memo_gui_cleanup();
 int str_to_csv_str(char *dest, char *src);
 
 /*
- * Do the opposite of str_to_csv_str, unescaping double quotes.
- * Return value is the number of chars written to dest.
- */
-int csv_str_to_str(char *dest, char *src);
-
-/*
- * Copy src string into dest while escaping carraige returns,
+ * Copy src string into dest while escaping carriage returns,
  * backslashes, commas and semicolons.  Also do line wrapping.
  * dest could be as long as strlen(src) * 2 + strlen(src) / 30.
  * Return value is the number of chars written to dest.
@@ -430,7 +450,7 @@ void cb_app_button(GtkWidget *widget, gpointer data);
 void call_plugin_gui(int number, int unique_id);
 
 /*datebook_gui */
-int datebook_refresh(int first);
+int datebook_refresh(int first, int do_init);
 
 /*address_gui */
 int address_refresh();
@@ -480,23 +500,38 @@ int pdb_file_modify_record(char *DB_name, void *record_in, int size_in,
  * attrp is the attributes
  * catp is the category (index)
  */
+#ifdef PILOT_LINK_0_12
+int pdb_file_read_record_by_id(char *DB_name, 
+			       pi_uid_t uid,
+			       void **bufp, size_t *sizep, int *idxp,
+			       int *attrp, int * catp);
+#else
 int pdb_file_read_record_by_id(char *DB_name, 
 			       pi_uid_t uid,
 			       void **bufp, int *sizep, int *idxp,
 			       int *attrp, int * catp);
+#endif
 /*
  * DB_name should be without filename ext, e.g. MemoDB
  * bufp is the packed app info block
  * size_in is the size of bufp
  */
+#ifdef PILOT_LINK_0_12
+int pdb_file_write_app_block(char *DB_name, void *bufp, size_t size_in);
+#else
 int pdb_file_write_app_block(char *DB_name, void *bufp, int size_in);
+#endif
 
 /*
- * This copies the datebase (pdb, or prc) and writes the DBInfo privided
+ * This copies the database (pdb, or prc) and writes the DBInfo privided
  * since there is no other way to set it in a file.
  */
 int pdb_file_write_dbinfo(char *DB_name, struct DBInfo *Pinfo_in);
 
+void append_anni_years(char *desc, int max, struct tm *date,
+		       struct Appointment *appt);
+int get_highlighted_today(struct tm *date);
+
 /* category.c */
 /*
  * widget is a widget inside the main window used to get main window handle
@@ -531,13 +566,12 @@ char *multibyte_safe_memccpy(char *dst, const char *src, int c, size_t len);
  * convert char code 
  *************************************/
 /* host character set to Palm character set */
-void charset_j2p(unsigned char *buf, int max_len, long char_set);
+void charset_j2p(char *buf, int max_len, long char_set);
 
 /* Palm character set to host character set */
-void charset_p2j(unsigned char *buf, int max_len, int char_set);
-unsigned char *charset_p2newj(const unsigned char *buf, int max_len, int char_set);
-
-void jp_charset_p2j(unsigned char *buf, int max_len);
-void jp_charset_j2p(unsigned char *buf, int max_len);
+void charset_p2j(char *buf, int max_len, int char_set);
+char *charset_p2newj(const char *buf, int max_len, int char_set);
 
+void jp_charset_p2j(char *buf, int max_len);
+size_t jp_strftime(char *s, size_t max, const char *format, const struct tm *tm);
 #endif
